@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 import scipy.sparse as sp
 import GridCal.Engine.Core.topology as tp
@@ -74,8 +76,12 @@ class GeneratorData:
 
         return data
 
-    def get_island(self, bus_idx):
-        return tp.get_elements_of_the_island(self.C_bus_gen.T, bus_idx)
+    def get_island(self, bus_idx, t_idx=0):
+        if self.ngen:
+            return tp.get_elements_of_the_island(self.C_bus_gen.T, bus_idx,
+                                                 active=self.generator_active[t_idx])
+        else:
+            return np.zeros(0, dtype=int)
 
     def get_injections(self):
         """
@@ -87,6 +93,9 @@ class GeneratorData:
         Q = pf_sign * self.generator_p * np.sqrt((1.0 - pf2) / (pf2 + 1e-20))
         return self.generator_p + 1.0j * Q
 
+    def get_effective_generation(self):
+        return self.generator_p * self.generator_active
+
     def get_injections_per_bus(self):
         return self.C_bus_gen * (self.get_injections() * self.generator_active)
 
