@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 import scipy.sparse as sp
 import GridCal.Engine.Core.topology as tp
@@ -30,16 +32,21 @@ class HvdcData:
 
         self.names = np.zeros(nhvdc, dtype=object)
 
-        self.loss_factor = np.zeros(nhvdc)
-        self.r = np.zeros(nhvdc)
+        self.angle_droop = np.zeros((nhvdc, ntime))
 
         self.control_mode = np.zeros(nhvdc, dtype=object)
 
+        self.dispatchable = np.zeros(nhvdc, dtype=int)
+
         self.active = np.zeros((nhvdc, ntime), dtype=bool)
         self.rate = np.zeros((nhvdc, ntime))
+        self.contingency_rate = np.zeros((nhvdc, ntime))
 
-        self.Pf = np.zeros((nhvdc, ntime))
+        self.r = np.zeros(nhvdc)
+
+        self.Pset = np.zeros((nhvdc, ntime))
         self.Pt = np.zeros((nhvdc, ntime))
+
         self.Vset_f = np.zeros((nhvdc, ntime))
         self.Vset_t = np.zeros((nhvdc, ntime))
 
@@ -69,16 +76,19 @@ class HvdcData:
 
         data.names = self.names[elm_idx]
         data.active = self.active[elm_idx]
+        data.dispatchable = self.dispatchable[elm_idx]
 
         data.rate = self.rate[tidx]
-        data.Pf = self.Pf[tidx]
-        data.Pt = self.Pt[tidx]
+        data.contingency_rate = self.contingency_rate[tidx]
+        data.Pset = self.Pset[tidx]
+
+        data.r = self.r[elm_idx]
+
         data.Vset_f = self.Vset_f[tidx]
         data.Vset_t = self.Vset_t[tidx]
 
-        data.loss_factor = self.loss_factor[elm_idx]
+        data.angle_droop = self.angle_droop[elm_idx]
 
-        data.r = self.r[elm_idx]
         data.control_mode = self.control_mode[elm_idx]
 
         data.Qmin_f = self.Qmin_f[elm_idx]
@@ -97,22 +107,17 @@ class HvdcData:
     def get_bus_indices_t(self):
         return self.C_hvdc_bus_t * np.arange(self.C_hvdc_bus_t.shape[1])
 
-    def get_island(self, bus_idx):
+    def get_island(self, bus_idx, t_idx=0):
         """
         Get HVDC indices of the island given by the bus indices
         :param bus_idx: list of bus indices
         :return: list of HVDC lines indices
         """
-        return tp.get_elements_of_the_island(self.C_hvdc_bus_f + self.C_hvdc_bus_t, bus_idx)
-
-    def get_injections_per_bus(self):
-        F = self.C_hvdc_bus_f.T * (self.active * self.Pf)
-        T = self.C_hvdc_bus_t.T * (self.active * self.Pt)
-        return F + T
-
-    @property
-    def Pbus(self):
-        return self.get_injections_per_bus()
+        if self.nhvdc:
+            return tp.get_elements_of_the_island(self.C_hvdc_bus_f + self.C_hvdc_bus_t, bus_idx,
+                                                 active=self.active[:, t_idx])
+        else:
+            return np.zeros(0, dtype=int)
 
     def get_qmax_from_per_bus(self):
         """
@@ -142,11 +147,13 @@ class HvdcData:
         """
         return self.C_hvdc_bus_t.T * (self.Qmin_t * self.active.T).T
 
-    def get_loading(self):
-        return self.Pf / (self.rate + 1e-20)
-
-    def get_losses(self):
-        return (self.Pf.T * self.loss_factor).T
+    def get_angle_droop_in_pu_rad(self, Sbase):
+        """
+        Get the angle droop in pu/rad
+        :param Sbase:
+        :return:
+        """
+        return self.angle_droop * 57.295779513 / Sbase
 
     def __len__(self):
         return self.nhvdc
