@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 import scipy.sparse as sp
 import GridCal.Engine.Core.topology as tp
@@ -68,8 +70,11 @@ class ShuntData:
 
         return data
 
-    def get_island(self, bus_idx):
-        return tp.get_elements_of_the_island(self.C_bus_shunt.T, bus_idx)
+    def get_island(self, bus_idx, t_idx=0):
+        if self.nshunt:
+            return tp.get_elements_of_the_island(self.C_bus_shunt.T, bus_idx, active=self.shunt_active[t_idx])
+        else:
+            return np.zeros(0, dtype=int)
 
     def get_controlled_per_bus(self):
         return self.C_bus_shunt * (self.shunt_controlled * self.shunt_active)
