@@ -1,9 +1,8 @@
-from GridCal.Engine.Core.topology import Graph
-
+from GridCal.Engine.Core.topology import find_islands, get_adjacency_matrix, get_elements_of_the_island
 from GridCal.Engine.Core.snapshot_pf_data import SnapshotData, compile_snapshot_circuit
 from GridCal.Engine.Core.snapshot_opf_data import SnapshotOpfData, compile_snapshot_opf_circuit
 from GridCal.Engine.Core.time_series_pf_data import TimeCircuit, compile_time_circuit
 from GridCal.Engine.Core.time_series_opf_data import OpfTimeCircuit, compile_opf_time_circuit
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
-from GridCal.Engine.Core.admittance_matrices import Admittance
+from GridCal.Engine.Core.admittance_matrices import Admittance, compute_linear_admittances, compute_admittances, compute_connectivity
 
