@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
 import numpy as np
@@ -33,7 +35,7 @@ def compute_connectivity(branch_active, Cf_, Ct_):
     return Cf.tocsc(), Ct.tocsc()
 
 
-def compute_admittances(R, X, G, B, k, m, mf, mt, theta, Beq, If, Cf, Ct, G0, a, b, c, Yshunt_bus):
+def compute_admittances(R, X, G, B, k, tap_module, vtap_f, vtap_t, tap_angle, Beq, If, Cf, Ct, G0, a, b, c, Yshunt_bus):
     """
     Compute the complete admittance matrices for the general power flow methods (Newton-Raphson based)
     :param R: array of branch resistance (p.u.)
@@ -41,10 +43,10 @@ def compute_admittances(R, X, G, B, k, m, mf, mt, theta, Beq, If, Cf, Ct, G0, a,
     :param G: array of branch conductance (p.u.)
     :param B: array of branch susceptance (p.u.)
     :param k: array of converter values: 1 for regular branches, sqrt(3) / 2 for VSC
-    :param m: array of tap modules (for all branches, regardless of their type)
-    :param mf: array of virtual taps at the "from" side
-    :param mt: array of virtual taps at the "to" side
-    :param theta: array of tap angles (for all branches, regardless of their type)
+    :param tap_module: array of tap modules (for all branches, regardless of their type)
+    :param vtap_f: array of virtual taps at the "from" side
+    :param vtap_t: array of virtual taps at the "to" side
+    :param tap_angle: array of tap angles (for all branches, regardless of their type)
     :param Beq: Array of equivalent susceptance
     :param If: Array of currents "from" in all the branches
     :param Cf: Connectivity branch-bus "from" with the branch states computed
@@ -64,13 +66,13 @@ def compute_admittances(R, X, G, B, k, m, mf, mt, theta, Beq, If, Cf, Ct, G0, a,
     ys = 1.0 / (R + 1.0j * X + 1e-20)  # series admittance
     bc2 = (G + 1j * B) / 2.0  # shunt admittance
     # k is already filled with the appropriate value for each type of branch
-    mp = k * m
+    mp = k * tap_module
 
     # compose the primitives
-    Yff = Gsw + (ys + bc2 + 1.0j * Beq) / (mp * mp * mf * mf)
-    Yft = -ys / (mp * np.exp(-1.0j * theta) * mf * mt)
-    Ytf = -ys / (mp * np.exp(1.0j * theta) * mt * mf)
-    Ytt = (ys + bc2) / (mt * mt)
+    Yff = Gsw + (ys + bc2 + 1.0j * Beq) / (mp * mp * vtap_f * vtap_f)
+    Yft = -ys / (mp * np.exp(-1.0j * tap_angle) * vtap_f * vtap_t)
+    Ytf = -ys / (mp * np.exp(1.0j * tap_angle) * vtap_t * vtap_f)
+    Ytt = (ys + bc2) / (vtap_t * vtap_t)
 
     # compose the matrices
     Yf = sp.diags(Yff) * Cf + sp.diags(Yft) * Ct
@@ -204,7 +206,7 @@ def compute_fast_decoupled_admittances(X, B, m, mf, mt, Cf, Ct):
     B2t = sp.diags(b2_tf) * Cf - sp.diags(b2_tt) * Ct
     B2 = Cf.T * B2f + Ct.T * B2t
 
-    return B1, B2
+    return B1.tocsc(), B2.tocsc()
 
 
 def compute_linear_admittances(nbr, X, R, m, active, Cf, Ct, ac, dc):
@@ -219,7 +221,7 @@ def compute_linear_admittances(nbr, X, R, m, active, Cf, Ct, ac, dc):
     :param Ct: Connectivity branch-bus "to" with the branch states computed
     :param ac: array of ac branches indices
     :param dc: array of dc branches indices
-    :return: Bbus, Bf
+    :return: Bbus, Bf, Btheta
     """
 
     m_abs = np.abs(m)
@@ -232,12 +234,28 @@ def compute_linear_admittances(nbr, X, R, m, active, Cf, Ct, ac, dc):
     else:
         b = 1.0 / (m_abs * X * active + 1e-20)  # for ac branches
 
-    b_tt = sp.diags(b)
+    b_tt = sp.diags(b)  # This is Bd from the
     Bf = b_tt * Cf - b_tt * Ct
     Bt = -b_tt * Cf + b_tt * Ct
     Bbus = Cf.T * Bf + Ct.T * Bt
+    Btheta = (b_tt * (Cf + Ct)).T
 
-    return Bbus, Bf
+    """
+    According to the KULeuven document "DC power flow in unit commitment models"
+    The DC power flow is:
+    
+    Pbus = (A^T x Bd x A) x bus_angles + (Bd x A)^T x branch_angles
+    
+    Identifying the already computed matrices, it becomes:
+    
+    Pbus = Bbus x bus_angles + Btheta x branch_angles
+    
+    If we solve for bus angles:
+    
+    bus_angles = Bbus^-1 x (Pbus - Btheta x branch_angles)
+    """
+
+    return Bbus, Bf, Btheta
 
 
 class Admittance:
