@@ -1,29 +1,28 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
-import os
 from typing import List, Dict, Tuple
 from uuid import getnode as get_mac, uuid4
-from datetime import timedelta
+from datetime import timedelta, datetime
 import networkx as nx
 from scipy.sparse import csc_matrix, lil_matrix
-from GridCal.Engine.basic_structures import Logger
-from GridCal.Gui.GeneralDialogues import *
+# from GridCal.Gui.GeneralDialogues import *
 from GridCal.Engine.Devices import *
-from GridCal.Engine.Simulations.PowerFlow.jacobian_based_power_flow import Jacobian
 from GridCal.Engine.Devices.editable_device import DeviceType
 
 
@@ -110,62 +109,64 @@ class MultiCircuit:
         # Should be able to accept Branches, Lines and Transformers alike
         # self.branches = list()
 
-        self.lines = list()  # type: List[Line]
+        self.lines: List[Line] = list()
 
-        self.dc_lines = list()  # type: List[DcLine]
+        self.dc_lines: List[DcLine] = list()
 
-        self.transformers2w = list()  # type: List[Transformer2W]
+        self.transformers2w: List[Transformer2W] = list()
 
-        self.hvdc_lines = list()  # type: List[HvdcLine]
+        self.hvdc_lines: List[HvdcLine] = list()
 
-        self.vsc_devices = list()  # type: List[VSC]
+        self.vsc_devices: List[VSC] = list()
 
-        self.upfc_devices = list()  # type List[UPFC]
+        self.upfc_devices: List[UPFC] = list()
 
-        self.switch_devices = list()  # type List[Switch]
+        self.switch_devices: List[Switch] = list()
+
+        self.transformers3w: List[Transformer3W] = list()
 
         # array of branch indices in the master circuit
         self.branch_original_idx = list()
 
         # Should accept buses
-        self.buses = list()  # type: List[Bus]
+        self.buses: List[Bus] = list()
 
         # array of bus indices in the master circuit
         self.bus_original_idx = list()
 
         # Dictionary relating the bus object to its index. Updated upon compilation
-        self.buses_dict = dict()  # type: Dict[Bus, int]
+        self.buses_dict: Dict[Bus, int] = dict()
 
         # List of overhead line objects
-        self.overhead_line_types = list()  # type: List[Tower]
+        self.overhead_line_types: List[Tower] = list()
 
         # list of wire types
-        self.wire_types = list()  # type: List[Wire]
+        self.wire_types: List[Wire] = list()
 
         # underground cable lines
-        self.underground_cable_types = list()  # type: List[UndergroundLineType]
+        self.underground_cable_types: List[UndergroundLineType] = list()
 
         # sequence modelled lines
-        self.sequence_line_types = list()  # type: List[SequenceLineType]
+        self.sequence_line_types: List[SequenceLineType] = list()
 
         # List of transformer types
-        self.transformer_types = list()  # type: List[TransformerType]
+        self.transformer_types: List[TransformerType] = list()
 
         # list of substations
         self.default_substation = Substation('Default substation')
-        self.substations = [self.default_substation]  # type: List[Substation]
+        self.substations: List[Substation] = [self.default_substation]
 
         # list of areas
         self.default_area = Area('Default area')
-        self.areas = [self.default_area]  # type: List[Area]
+        self.areas: List[Area] = [self.default_area]
 
         # list of zones
         self.default_zone = Zone('Default zone')
-        self.zones = [self.default_zone]  # type: List[Zone]
+        self.zones: List[Zone] = [self.default_zone]
 
         # list of countries
         self.default_country = Country('Default country')
-        self.countries = [self.default_country]  # type: List[Country]
+        self.countries: List[Country] = [self.default_country]
 
         # logger of events
         self.logger = Logger()
@@ -182,9 +183,6 @@ class MultiCircuit:
         # dictionary of branch objects -> branch indices
         self.branch_dictionary = dict()
 
-        # are there time series??
-        self.has_time_series = False
-
         # names of the buses
         self.bus_names = None
 
@@ -198,12 +196,14 @@ class MultiCircuit:
         self.objects_with_profiles = [Bus(),
                                       Load(),
                                       StaticGenerator(),
+                                      ExternalGrid(),
                                       Generator(),
                                       Battery(),
                                       Shunt(),
                                       Line(None, None),
                                       DcLine(None, None),
                                       Transformer2W(None, None),
+                                      Transformer3W(),
                                       HvdcLine(None, None),
                                       VSC(None, None),
                                       UPFC(None, None),
@@ -232,6 +232,40 @@ class MultiCircuit:
     def __str__(self):
         return str(self.name)
 
+    @property
+    def has_time_series(self):
+        return self.time_profile is not None
+
+    def get_zones(self):
+        return self.zones
+
+    def get_zone_number(self):
+        """
+        Get number of areas
+        :return: number of areas
+        """
+        return len(self.zones)
+
+    def get_areas(self):
+        return self.areas
+
+    def get_area_number(self):
+        """
+        Get number of areas
+        :return: number of areas
+        """
+        return len(self.areas)
+
+    def get_substations(self):
+        return self.substations
+
+    def get_substation_number(self):
+        """
+        Get number of areas
+        :return: number of areas
+        """
+        return len(self.substations)
+
     def get_bus_number(self):
         """
         Return the number of buses
@@ -239,12 +273,19 @@ class MultiCircuit:
         """
         return len(self.buses)
 
+    def get_branch_lists_wo_hvdc(self):
+        """
+        GEt list of the branch lists
+        :return:
+        """
+        return [self.lines, self.transformers2w,  self.vsc_devices, self.dc_lines, self.upfc_devices]
+
     def get_branch_lists(self):
         """
         GEt list of the branch lists
         :return:
         """
-        return [self.lines, self.transformers2w, self.hvdc_lines, self.vsc_devices, self.dc_lines, self.upfc_devices]
+        return self.get_branch_lists_wo_hvdc() + [self.hvdc_lines]
 
     def get_branch_number(self):
         """
@@ -256,6 +297,16 @@ class MultiCircuit:
             m += len(branch_list)
         return m
 
+    def get_branch_number_wo_hvdc(self):
+        """
+        return the number of branches (of all types)
+        :return: number
+        """
+        m = 0
+        for branch_list in self.get_branch_lists_wo_hvdc():
+            m += len(branch_list)
+        return m
+
     def get_time_number(self):
         """
         Return the number of buses
@@ -322,8 +373,6 @@ class MultiCircuit:
 
         self.branch_dictionary = dict()
 
-        self.has_time_series = False
-
         self.bus_names = None
 
         self.branch_names = None
@@ -333,12 +382,18 @@ class MultiCircuit:
     def get_buses(self):
         return self.buses
 
+    def get_bus_names(self):
+        return [e.name for e in self.buses]
+
     def get_branches_wo_hvdc(self):
         """
         Return all the branch objects
         :return: lines + transformers 2w + hvdc
         """
-        return self.lines + self.transformers2w + self.vsc_devices + self.dc_lines + self.upfc_devices + self.switch_devices
+        return self.lines + self.dc_lines + self.transformers2w + self.vsc_devices + self.upfc_devices + self.switch_devices
+
+    def get_branches_wo_hvdc_names(self):
+        return [e.name for e in self.get_branches_wo_hvdc()]
 
     def get_branches(self):
         """
@@ -347,7 +402,34 @@ class MultiCircuit:
         """
         return self.get_branches_wo_hvdc() + self.hvdc_lines
 
-    def get_loads(self):
+    def get_lines(self) -> List[Line]:
+        return self.lines
+
+    def get_transformers2w(self) -> List[Transformer2W]:
+        return self.transformers2w
+
+    def get_transformers2w_number(self) -> int:
+        return len(self.transformers2w)
+
+    def get_vsc(self) -> List[VSC]:
+        return self.vsc_devices
+
+    def get_dc_lines(self) -> List[DcLine]:
+        return self.dc_lines
+
+    def get_upfc(self) -> List[UPFC]:
+        return self.upfc_devices
+
+    def get_switches(self) -> List[Switch]:
+        return self.switch_devices
+
+    def get_hvdc(self) -> List[HvdcLine]:
+        return self.hvdc_lines
+
+    def get_hvdc_number(self) -> int:
+        return len(self.hvdc_lines)
+
+    def get_loads(self) -> List[Load]:
         """
         Returns a list of :ref:`Load<load>` objects in the grid.
         """
@@ -368,7 +450,28 @@ class MultiCircuit:
                 lst.append(elm.name)
         return np.array(lst)
 
-    def get_static_generators(self):
+    def get_external_grids(self) -> List[ExternalGrid]:
+        """
+        Returns a list of :ref:`ExternalGrid<external_grid>` objects in the grid.
+        """
+        lst = list()
+        for bus in self.buses:
+            for elm in bus.external_grids:
+                elm.bus = bus
+            lst = lst + bus.external_grids
+        return lst
+
+    def get_external_grid_names(self):
+        """
+        Returns a list of :ref:`ExternalGrid<external_grid>` names.
+        """
+        lst = list()
+        for bus in self.buses:
+            for elm in bus.external_grids:
+                lst.append(elm.name)
+        return np.array(lst)
+
+    def get_static_generators(self) -> List[StaticGenerator]:
         """
         Returns a list of :ref:`StaticGenerator<static_generator>` objects in the grid.
         """
@@ -389,7 +492,7 @@ class MultiCircuit:
                 lst.append(elm.name)
         return np.array(lst)
 
-    def get_shunts(self):
+    def get_shunts(self) -> List[Shunt]:
         """
         Returns a list of :ref:`Shunt<shunt>` objects in the grid.
         """
@@ -410,7 +513,7 @@ class MultiCircuit:
                 lst.append(elm.name)
         return np.array(lst)
 
-    def get_generators(self):
+    def get_generators(self) -> List[Generator]:
         """
         Returns a list of :ref:`Generator<generator>` objects in the grid.
         """
@@ -431,7 +534,7 @@ class MultiCircuit:
                 lst.append(elm.name)
         return np.array(lst)
 
-    def get_batteries(self):
+    def get_batteries(self) -> List[Battery]:
         """
         Returns a list of :ref:`Battery<battery>` objects in the grid.
         """
@@ -484,6 +587,9 @@ class MultiCircuit:
         elif element_type == DeviceType.ShuntDevice:
             return self.get_shunts()
 
+        elif element_type == DeviceType.ExternalGridDevice:
+            return self.get_external_grids()
+
         elif element_type == DeviceType.LineDevice:
             return self.lines
 
@@ -557,6 +663,9 @@ class MultiCircuit:
         elif element_type == DeviceType.ShuntDevice:
             return self.get_shunts()
 
+        elif element_type == DeviceType.ExternalGridDevice:
+            return self.get_external_grids()
+
         elif element_type == DeviceType.SubstationDevice:
             return [x.substation for x in self.buses]
 
@@ -584,6 +693,8 @@ class MultiCircuit:
         for i, gen in enumerate(loads):
             gen.P_prof -= results.load_shedding[:, i]
 
+        # TODO: implement more devices
+
     def copy(self):
         """
         Returns a deep (true) copy of this circuit.
@@ -648,8 +759,12 @@ class MultiCircuit:
 
         return catalogue_dict
 
-    def get_catalogue_dict_by_name(self, type_class=None):
-
+    def get_catalogue_dict_by_name(self, type_class: str = None):
+        """
+        Get the catalogue elements by name
+        :param type_class:
+        :return:
+        """
         d = dict()
 
         # ['Wires', 'Overhead lines', 'Underground lines', 'Sequence lines', 'Transformers']
@@ -722,7 +837,10 @@ class MultiCircuit:
         """
         """
         if self.time_profile is not None:
-            t = self.time_profile.astype(int).tolist()
+            # recommended way to get the unix datetimes
+            arr = (self.time_profile - pd.Timestamp("1970-01-01")) // pd.Timedelta("1s")
+            # t = (self.time_profile.array.astype(int) * 1e-9).tolist()  # UNIX time in seconds
+            t = arr.tolist()
         else:
             t = list()
         return {'time': t}
@@ -760,13 +878,35 @@ class MultiCircuit:
         self.bus_dictionary = {bus.idtag: i for i, bus in enumerate(self.buses)}
 
         for branch_list in self.get_branch_lists():
-            for i, branch in enumerate(branch_list):
+            for branch in branch_list:
                 f = self.bus_dictionary[branch.bus_from.idtag]
                 t = self.bus_dictionary[branch.bus_to.idtag]
                 self.graph.add_edge(f, t)
 
         return self.graph
 
+    def build_graph_real_power_flow(self, current_flow):
+        """
+        Returns a networkx DiGraph object of the grid.
+
+        Arguments:
+
+            **current_flow** (list): power_flow.results.If object
+        """
+        self.graph_real_power_flow = nx.DiGraph()
+
+        current_flow_direction = np.real(current_flow) > 0
+
+        for branch_list in self.get_branch_lists():
+            for direction, branch in zip(current_flow_direction, branch_list):
+                f = self.bus_dictionary[branch.bus_from.idtag]
+                t = self.bus_dictionary[branch.bus_to.idtag]
+                if direction:
+                    self.graph_real_power_flow.add_edge(f, t)
+                else:
+                    self.graph_real_power_flow.add_edge(t, f)
+        return self.graph_real_power_flow
+
     def create_profiles(self, steps, step_length, step_unit, time_base: datetime = datetime.now()):
         """
         Set the default profiles in all the objects enabled to have profiles.
@@ -782,7 +922,7 @@ class MultiCircuit:
             **time_base** (datetime, datetime.now()): Date to start from
         """
 
-        index = [None] * steps
+        index = np.empty(steps, dtype=object)
         for i in range(steps):
             if step_unit == 'h':
                 index[i] = time_base + timedelta(hours=i * step_length)
@@ -813,6 +953,22 @@ class MultiCircuit:
             for elm in branch_list:
                 elm.create_profiles(index)
 
+    def set_time_profile(self, unix_data):
+
+        # try parse
+        try:
+            self.time_profile = pd.to_datetime(np.array(unix_data), unit='s', origin='unix')
+        except Exception as e:
+            # it may come in nanoseconds instead of seconds...
+            self.time_profile = pd.to_datetime(np.array(unix_data) / 1e9, unit='s', origin='unix')
+
+        for elm in self.buses:
+            elm.create_profiles(self.time_profile)
+
+        for branch_list in self.get_branch_lists():
+            for elm in branch_list:
+                elm.create_profiles(self.time_profile)
+
     def ensure_profiles_exist(self):
         """
         Format the pandas profiles in place using a time index.
@@ -827,57 +983,6 @@ class MultiCircuit:
             for elm in branch_list:
                 elm.ensure_profiles_exist(self.time_profile)
 
-    def get_node_elements_by_type(self, element_type: DeviceType):
-        """
-        Get set of elements and their parent nodes.
-
-        Arguments:
-
-            **element_type** (str): Element type, either "Load", "StaticGenerator",
-            "Generator", "Battery" or "Shunt"
-
-        Returns:
-
-            List of elements, list of matching parent buses
-        """
-        elements = list()
-        parent_buses = list()
-
-        if element_type == DeviceType.LoadDevice:
-            for bus in self.buses:
-                for elm in bus.loads:
-                    elements.append(elm)
-                    parent_buses.append(bus)
-
-        elif element_type == DeviceType.StaticGeneratorDevice:
-            for bus in self.buses:
-                for elm in bus.static_generators:
-                    elements.append(elm)
-                    parent_buses.append(bus)
-
-        elif element_type == DeviceType.GeneratorDevice:
-            for bus in self.buses:
-                for elm in bus.controlled_generators:
-                    elements.append(elm)
-                    parent_buses.append(bus)
-
-        elif element_type == DeviceType.BatteryDevice:
-            for bus in self.buses:
-                for elm in bus.batteries:
-                    elements.append(elm)
-                    parent_buses.append(bus)
-
-        elif element_type == DeviceType.ShuntDevice:
-            for bus in self.buses:
-                for elm in bus.shunts:
-                    elements.append(elm)
-                    parent_buses.append(bus)
-
-        else:
-            pass
-
-        return elements, parent_buses
-
     def get_bus_dict(self):
         """
         Return dictionary of buses
@@ -970,6 +1075,16 @@ class MultiCircuit:
             obj.create_profiles(self.time_profile)
         self.transformers2w.append(obj)
 
+    def add_transformer3w(self, obj: Transformer3W):
+        """
+        Add a transformer object
+        :param obj: Transformer3W instance
+        """
+
+        if self.time_profile is not None:
+            obj.create_profiles(self.time_profile)
+        self.transformers3w.append(obj)
+
     def add_hvdc(self, obj: HvdcLine):
         """
         Add a hvdc line object
@@ -1174,6 +1289,30 @@ class MultiCircuit:
 
         return api_obj
 
+    def add_external_grid(self, bus: Bus, api_obj=None):
+        """
+        Add a :ref:`Load<load>` object to a :ref:`Bus<bus>`.
+
+        Arguments:
+
+            **bus** (:ref:`Bus<bus>`): :ref:`Bus<bus>` object
+
+            **api_obj** (:ref:`Load<load>`): :ref:`Load<load>` object
+        """
+        if api_obj is None:
+            api_obj = ExternalGrid()
+        api_obj.bus = bus
+
+        if self.time_profile is not None:
+            api_obj.create_profiles(self.time_profile)
+
+        if api_obj.name == 'External grid':
+            api_obj.name += '@' + bus.name
+
+        bus.loads.append(api_obj)
+
+        return api_obj
+
     def add_battery(self, bus: Bus, api_obj=None):
         """
         Add a :ref:`Battery<battery>` object to a :ref:`Bus<bus>`.
@@ -1632,6 +1771,9 @@ class MultiCircuit:
         for bus in self.buses:
             bus.set_state(t)
 
+        for branch in self.get_branches():
+            branch.set_profile_values(t)
+
     def get_bus_branch_connectivity_matrix(self):
         """
         Get the branch-bus connectivity
@@ -1752,11 +1894,14 @@ class MultiCircuit:
         :return: average separation
         """
         separation = 0.0
-        branches = self.get_branch_lists()
-        for branch in branches:
-            s = np.sqrt((branch.bus_from.x - branch.bus_to.x)**2 + (branch.bus_from.y - branch.bus_to.y)**2)
-            separation += s
-        return separation / len(branches)
+        branch_lists = self.get_branch_lists()
+        n = 0
+        for branch_lst in branch_lists:
+            for branch in branch_lst:
+                s = np.sqrt((branch.bus_from.x - branch.bus_to.x)**2 + (branch.bus_from.y - branch.bus_to.y)**2)
+                separation += s
+                n += 1
+        return separation / n
 
     def add_circuit(self, circuit: "MultiCircuit", angle):
         """
@@ -1783,9 +1928,8 @@ class MultiCircuit:
         y0 = xm + r * np.sin(a)
 
         # modify the coordinates of the new circuit
-        min_x2, max_x2, min_y2, max_y2 = self.get_boundaries(circuit.buses)
-        branches2 = circuit.lines + circuit.transformers2w + circuit.hvdc_lines
-        sep2 = self.average_separation(branches2)
+        min_x2, max_x2, min_y2, max_y2 = self.get_boundaries()
+        sep2 = self.average_separation()
         factor = sep2 / sep1
         for bus in circuit.buses:
             bus.x = x0 + (bus.x - min_x2) * factor
@@ -1801,15 +1945,30 @@ class MultiCircuit:
                 for branch in lst:
                     branch.create_profiles(index=self.time_profile)
 
-        self.buses += circuit.buses
-        self.lines += circuit.lines
-        self.transformers2w += circuit.transformers2w
-        self.hvdc_lines += circuit.hvdc_lines
-        self.vsc_devices += circuit.vsc_devices
-        self.dc_lines += circuit.dc_lines
+        self.add_devices_list(self.buses, circuit.buses)
+        self.add_devices_list(self.lines, circuit.lines)
+        self.add_devices_list(self.transformers2w, circuit.transformers2w)
+        self.add_devices_list(self.hvdc_lines, circuit.hvdc_lines)
+        self.add_devices_list(self.vsc_devices, circuit.vsc_devices)
+        self.add_devices_list(self.dc_lines, circuit.dc_lines)
 
         return circuit.buses
 
+    def add_devices_list(self, original_list, new_list):
+        """
+        Add a list of devices to another keeping coherence
+        :param original_list:
+        :param new_list:
+        :return:
+        """
+        existing_uuids = {e.idtag for e in original_list}
+
+        for elm in new_list:
+            if elm.idtag in existing_uuids:
+                print(elm.name , 'uuid is repeated..generating new one')
+                elm.generate_uuid()
+            original_list.append(elm)
+
     def snapshot_balance(self):
         """
         Creates a report DataFrame with the snapshot active power balance
@@ -1905,11 +2064,12 @@ class MultiCircuit:
 
         return tolerance, exponent
 
-    def fill_xy_from_lat_lon(self, destructive=True, factor=0.01):
+    def fill_xy_from_lat_lon(self, destructive=True, factor=0.01, remove_offset=True):
         """
         fill the x and y value from the latitude and longitude values
         :param destructive: if true, the values are overwritten regardless, otherwise only if x and y are 0
         :param factor: Explosion factor
+        :param remove_offset: remove the sometimes huge offset coming from pyproj
         """
 
         n = len(self.buses)
@@ -1920,17 +2080,32 @@ class MultiCircuit:
             lat[i] = bus.latitude
 
         # perform the coordinate transformation
-        import pyproj
+        logger = Logger()
+        try:
+            import pyproj
+        except ImportError:
+            logger.add_error("pyproj is not installed")
+            return logger
+
         x, y = pyproj.Transformer.from_crs(4326, 25830, always_xy=True).transform(lon, lat)
         x *= factor
         y *= factor
 
+        # remove the offset
+        if remove_offset:
+            x_min = np.min(x)
+            y_max = np.max(y)
+            x -= x_min + 100  # 100 is a healthy offset
+            y -= y_max - 100  # 100 is a healthy offset
+
         # assign the values
         for i, bus in enumerate(self.buses):
             if destructive or (bus.x == 0.0 and bus.y == 0.0):
                 bus.x = x[i]
                 bus.y = -y[i]
 
+        return logger
+
     def fill_lat_lon_from_xy(self, destructive=True, factor=1.0, offset_x=0, offset_y=0):
         """
         Convert the coordinates to some random lat lon
@@ -1947,7 +2122,13 @@ class MultiCircuit:
             x[i] = bus.x * factor + offset_x
             y[i] = bus.y * factor + offset_y
 
-        import pyproj
+        logger = Logger()
+        try:
+            import pyproj
+        except ImportError:
+            logger.add_error("pyproj is not installed")
+            return logger
+
         proj_latlon = pyproj.Proj(proj='latlong', datum='WGS84')
         proj_xy = pyproj.Proj(proj="utm", zone=33, datum='WGS84')
         lonlat = pyproj.transform(proj_xy, proj_latlon, x, y)
@@ -1961,13 +2142,15 @@ class MultiCircuit:
                 bus.latitude = lat[i]
                 bus.longitude = lon[i]
 
+        return logger
+
     def import_bus_lat_lon(self, df: pd.DataFrame, bus_col, lat_col, lon_col):
         """
 
-        :param df:
-        :param bus_col:
-        :param lat_col:
-        :param lon_col:
+        :param df: Pandas DataFrame with the information
+        :param bus_col: bus column name
+        :param lat_col: latitude column name
+        :param lon_col: longitude column name
         :return:
         """
         logger = Logger()
@@ -2046,8 +2229,6 @@ class MultiCircuit:
                 gen.P_prof = df[col_name].values
             except KeyError:
                 logger.add_error("Missing in the model", col_name)
-                # gen.P_prof = np.zeros(nn)
-                # gen.Q_prof = np.zeros(nn)
 
         return logger
 
@@ -2191,4 +2372,140 @@ class MultiCircuit:
                 lst.append((k, branch, 1.0))
             elif branch.bus_from.zone == z2 and branch.bus_to.zone == z1:
                 lst.append((k, branch, -1.0))
-        return lst
\ No newline at end of file
+        return lst
+
+    def get_branch_area_connectivity_matrix(self, a1: List[Area], a2: List[Area]):
+        """
+        Get the inter area connectivity matrix
+        :param a1: list of sending areas
+        :param a2: list of receiving areas
+        :return: Connectivity of the branches to each sending or receiving area groups (branches, 2)
+        """
+        area_dict = {a: i for i, a in enumerate(self.areas)}
+
+        area1_list = [area_dict[a] for a in a1]
+        area2_list = [area_dict[a] for a in a2]
+
+        branches = self.get_branches()  # all including HVDC
+
+        conn = lil_matrix((len(branches), 2), dtype=int)
+
+        for k, elm in enumerate(branches):
+            i = area_dict[elm.bus_from.area]
+            j = area_dict[elm.bus_to.area]
+            if i != j:
+                if (i in area1_list) and (j in area2_list):
+                    # from->to matches the areas
+                    conn[k, 0] = 1
+                    conn[k, 1] = -1
+                elif (i in area2_list) and (j in area1_list):
+                    # reverse the sign
+                    conn[k, 0] = -1
+                    conn[k, 1] = 1
+
+        return conn.tocsc()
+
+    def change_base(self, Sbase_new):
+        """
+        Change the elements base impedance
+        :param Sbase_new: new base impedance in MVA
+        """
+        Sbase_old = self.Sbase
+
+        # get all the branches with impedance
+        elms = self.get_branches_wo_hvdc()
+
+        # change the base at each element
+        for elm in elms:
+            elm.change_base(Sbase_old, Sbase_new)
+
+        # assign the new base
+        self.Sbase = Sbase_new
+
+    def fuse_devices(self):
+        """
+        Fuse all the different devices in a node to a single device per node
+        :return:
+        """
+        for bus in self.buses:
+            bus.fuse_devices()
+
+    def re_index_time(self, year=None, hours_per_step=1):
+        """
+        Generate sequential time steps to correct the time_profile
+        :param year: base year, if None, this year is taken
+        :param hours_per_step: number of hours per step, by default 1 hour by step
+        """
+        if year is None:
+            t0 = datetime.now()
+            year = t0.year
+
+        nt = self.get_time_number()
+        t0 = datetime(year=year, month=1, day=1)
+        tm = [t0 + timedelta(hours=t * hours_per_step) for t in range(nt)]
+        self.time_profile = pd.to_datetime(tm)
+
+    def set_generators_active_profile_from_their_active_power(self):
+        """
+        Modify the generators active profile to match the active power profile
+        if P=0, active = False else active=True
+        """
+        for g in self.get_generators():
+            g.active_prof = g.P_prof.astype(bool)
+
+    def set_batteries_active_profile_from_their_active_power(self):
+        """
+        Modify the batteries active profile to match the active power profile
+        if P=0, active = False else active=True
+        """
+        for g in self.get_batteries():
+            g.active_prof = g.P_prof.astype(bool)
+
+    def correct_inconsistencies(self, logger: Logger, maximum_difference=0.1, min_vset=0.98, max_vset=1.02):
+        """
+        Correct devices' inconsistencies
+        :param logger: logger to store the events
+        :param maximum_difference: proportion to be under or above (i.e.
+                                   Transformer HV=41.9, bus HV=45 41.9/45 = 0.93 ->
+                                   0.9 <= 0.93 <= 1.1, so its ok
+        :param min_vset: minimum voltage set point (p.u.)
+        :param max_vset: maximum voltage set point (p.u.)
+        :return:
+        """
+        for elm in self.transformers2w:
+            elm.fix_inconsistencies(logger,
+                                    maximum_difference=maximum_difference)
+
+        for elm in self.get_generators():
+            elm.fix_inconsistencies(logger,
+                                    min_vset=min_vset,
+                                    max_vset=max_vset)
+
+    def normalize_bus_positions_offset(self, base_offset=100):
+        """
+        Normalize the massive offset that may be in the Qt graphic objects' positions
+        :param base_offset:
+        :return:
+        """
+        n = len(self.buses)
+        x = np.zeros(n, dtype=int)
+        y = np.zeros(n, dtype=int)
+
+        # read values
+        for i, bus in enumerate(self.buses):
+            gr = bus.graphic_obj
+            x[i] = gr.x()
+            y[i] = gr.y()
+
+        # correct values
+        x_min = np.min(x)
+        y_max = np.max(y)
+        x -= x_min + base_offset  # 100 is a healthy offset
+        y -= y_max - base_offset  # 100 is a healthy offset
+
+        # assign values
+        for i, bus in enumerate(self.buses):
+            bus.x = x[i]
+            bus.y = y[i]
+            if bus.graphic_obj is not None:
+                bus.graphic_obj.set_position(x[i], y[i])
