@@ -1,39 +1,41 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 import numba as nb
 import pandas as pd
 import scipy.sparse as sp
-from typing import List
+from typing import List, Tuple
 
 from GridCal.Engine.basic_structures import Logger
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.basic_structures import BranchImpedanceMode
 import GridCal.Engine.Core.topology as tp
-from GridCal.Engine.Simulations.PowerFlow.jacobian_based_power_flow import Jacobian
+from GridCal.Engine.Simulations.PowerFlow.NumericalMethods.ac_jacobian import Jacobian
 from GridCal.Engine.Core.common_functions import compile_types
 from GridCal.Engine.Simulations.sparse_solve import get_sparse_type
-import GridCal.Engine.Core.DataStructures as ds
+import GridCal.Engine.Core.Compilers.circuit_to_data as gc_compiler
 import GridCal.Engine.Core.admittance_matrices as ycalc
 from GridCal.Engine.Devices.enumerations import TransformerControlType, ConverterControlType
 
 sparse_type = get_sparse_type()
 
 
-@nb.njit()
+@nb.njit(cache=True)
 def compose_generator_voltage_profile(nbus, ntime,
                                       gen_bus_indices, gen_vset, gen_status, gen_is_controlled,
                                       bat_bus_indices, bat_vset, bat_status, bat_is_controlled,
@@ -120,6 +122,46 @@ def compose_generator_voltage_profile(nbus, ntime,
     return V
 
 
+def get_inter_areas_branch(F, T, buses_areas_1, buses_areas_2):
+    """
+    Get the branches that join two areas
+    :param buses_areas_1: Area from
+    :param buses_areas_2: Area to
+    :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+    """
+    nbr = len(F)
+    lst: List[Tuple[int, float]] = list()
+    for k in range(nbr):
+        if F[k] in buses_areas_1 and T[k] in buses_areas_2:
+            lst.append((k, 1.0))
+        elif F[k] in buses_areas_2 and T[k] in buses_areas_1:
+            lst.append((k, -1.0))
+    return lst
+
+
+def get_devices_per_areas(Cdev, buses_in_a1, buses_in_a2):
+    """
+    Get the generators that belong to the Area 1, Area 2 and the rest of areas
+    :param buses_in_a1: List of bus indices of the area 1
+    :param buses_in_a2: List of bus indices of the area 2
+    :return: Tree lists: (gens_in_a1, gens_in_a2, gens_out) each of the lists contains (bus index, generator index) tuples
+    """
+    gens_in_a1 = list()
+    gens_in_a2 = list()
+    gens_out = list()
+    for j in range(Cdev.shape[1]):  # for each bus
+        for ii in range(Cdev.indptr[j], Cdev.indptr[j + 1]):
+            i = Cdev.indices[ii]
+            if i in buses_in_a1:
+                gens_in_a1.append((i, j))  # i: bus idx, j: gen idx
+            elif i in buses_in_a2:
+                gens_in_a2.append((i, j))  # i: bus idx, j: gen idx
+            else:
+                gens_out.append((i, j))  # i: bus idx, j: gen idx
+
+    return gens_in_a1, gens_in_a2, gens_out
+
+
 class SnapshotData:
 
     def __init__(self, nbus, nline, ndcline, ntr, nvsc, nupfc, nhvdc, nload, ngen, nbatt, nshunt, nstagen, sbase, ntime=1):
@@ -173,20 +215,20 @@ class SnapshotData:
         # --------------------------------------------------------------------------------------------------------------
         # Data structures
         # --------------------------------------------------------------------------------------------------------------
-        self.bus_data = ds.BusData(nbus=nbus, ntime=ntime)
-        self.branch_data = ds.BranchData(nbr=self.nbr, nbus=nbus, ntime=ntime)
-        self.line_data = ds.LinesData(nline=nline, nbus=nbus, ntime=ntime)
-        self.dc_line_data = ds.DcLinesData(ndcline=ndcline, nbus=nbus, ntime=ntime)
-        self.transformer_data = ds.TransformerData(ntr=ntr, nbus=nbus, ntime=ntime)
-        self.vsc_data = ds.VscData(nvsc=nvsc, nbus=nbus, ntime=ntime)
-        self.upfc_data = ds.UpfcData(nelm=nupfc, nbus=nbus, ntime=ntime)
-        self.hvdc_data = ds.HvdcData(nhvdc=nhvdc, nbus=nbus, ntime=ntime)
+        self.bus_data = gc_compiler.BusData(nbus=nbus, ntime=ntime)
+        self.branch_data = gc_compiler.BranchData(nbr=self.nbr, nbus=nbus, ntime=ntime)
+        self.line_data = gc_compiler.LinesData(nline=nline, nbus=nbus, ntime=ntime)
+        self.dc_line_data = gc_compiler.DcLinesData(ndcline=ndcline, nbus=nbus, ntime=ntime)
+        self.transformer_data = gc_compiler.TransformerData(ntr=ntr, nbus=nbus, ntime=ntime)
+        self.vsc_data = gc_compiler.VscData(nvsc=nvsc, nbus=nbus, ntime=ntime)
+        self.upfc_data = gc_compiler.UpfcData(nelm=nupfc, nbus=nbus, ntime=ntime)
+        self.hvdc_data = gc_compiler.HvdcData(nhvdc=nhvdc, nbus=nbus, ntime=ntime)
 
-        self.load_data = ds.LoadData(nload=nload, nbus=nbus, ntime=ntime)
-        self.static_generator_data = ds.StaticGeneratorData(nstagen=nstagen, nbus=nbus, ntime=ntime)
-        self.battery_data = ds.BatteryData(nbatt=nbatt, nbus=nbus, ntime=ntime)
-        self.generator_data = ds.GeneratorData(ngen=ngen, nbus=nbus, ntime=ntime)
-        self.shunt_data = ds.ShuntData(nshunt=nshunt, nbus=nbus, ntime=ntime)
+        self.load_data = gc_compiler.LoadData(nload=nload, nbus=nbus, ntime=ntime)
+        self.static_generator_data = gc_compiler.StaticGeneratorData(nstagen=nstagen, nbus=nbus, ntime=ntime)
+        self.battery_data = gc_compiler.BatteryData(nbatt=nbatt, nbus=nbus, ntime=ntime)
+        self.generator_data = gc_compiler.GeneratorData(ngen=ngen, nbus=nbus, ntime=ntime)
+        self.shunt_data = gc_compiler.ShuntData(nshunt=nshunt, nbus=nbus, ntime=ntime)
 
         self.original_bus_idx = np.arange(self.nbus)
         self.original_branch_idx = np.arange(self.nbr)
@@ -209,10 +251,12 @@ class SnapshotData:
 
         self.Cf_ = None
         self.Ct_ = None
+        self.A_ = None
 
         self.Vbus_ = None
         self.Sbus_ = None
         self.Ibus_ = None
+        self.YloadBus_ = None
         self.Yshunt_from_devices_ = None
 
         self.Qmax_bus_ = None
@@ -233,6 +277,7 @@ class SnapshotData:
         # Admittances for Linear
         self.Bbus_ = None
         self.Bf_ = None
+        self.Btheta_ = None
         self.Bpqpv_ = None
         self.Bref_ = None
 
@@ -246,7 +291,7 @@ class SnapshotData:
         self.available_structures = ['Vbus',
                                      'Sbus',
                                      'Ibus',
-                                     'Ybus',
+                                     'Ybus',  'G', 'B',
                                      'Yf',
                                      'Yt',
                                      'Bbus',
@@ -265,6 +310,8 @@ class SnapshotData:
                                      'pv',
                                      'vd',
                                      'pqpv',
+                                     'tap_f',
+                                     'tap_t',
                                      'original_bus_idx',
                                      'original_branch_idx',
                                      'original_line_idx',
@@ -301,17 +348,14 @@ class SnapshotData:
         # battery
         Sbus += self.battery_data.get_injections_per_bus()
 
-        # HVDC forced power
-        if self.nhvdc:
-            # Pf and Pt come with the correct sign already
-            Sbus += self.hvdc_data.get_injections_per_bus()
+        # HVDC forced power is not handled here because of the possible islands
 
         if normalize:
             Sbus /= self.Sbase
 
         return Sbus
 
-    def consolidate_information(self):
+    def consolidate_information(self, use_stored_guess=False):
         """
         Consolidates the information of this object
         :return:
@@ -364,28 +408,29 @@ class SnapshotData:
         self.bus_data.bus_installed_power = self.generator_data.get_installed_power_per_bus()
         self.bus_data.bus_installed_power += self.battery_data.get_installed_power_per_bus()
 
-        self.bus_data.Vbus = compose_generator_voltage_profile(nbus=self.nbus,
-                                                               ntime=self.ntime,
-                                                               gen_bus_indices=self.generator_data.get_bus_indices(),
-                                                               gen_vset=self.generator_data.generator_v,
-                                                               gen_status=self.generator_data.generator_active,
-                                                               gen_is_controlled=self.generator_data.generator_controllable,
-                                                               bat_bus_indices=self.battery_data.get_bus_indices(),
-                                                               bat_vset=self.battery_data.battery_v,
-                                                               bat_status=self.battery_data.battery_active,
-                                                               bat_is_controlled=self.battery_data.battery_controllable,
-                                                               hvdc_bus_f=self.hvdc_data.get_bus_indices_f(),
-                                                               hvdc_bus_t=self.hvdc_data.get_bus_indices_t(),
-                                                               hvdc_status=self.hvdc_data.active,
-                                                               hvdc_vf=self.hvdc_data.Vset_f,
-                                                               hvdc_vt=self.hvdc_data.Vset_t,
-                                                               iBeqv=np.array(self.iBeqv, dtype=int),
-                                                               iVtma=np.array(self.iVtma, dtype=int),
-                                                               VfBeqbus=np.array(self.VfBeqbus, dtype=int),
-                                                               Vtmabus=np.array(self.Vtmabus, dtype=int),
-                                                               branch_status=self.branch_data.branch_active,
-                                                               br_vf=self.branch_data.vf_set,
-                                                               br_vt=self.branch_data.vt_set)
+        if not use_stored_guess:
+            self.bus_data.Vbus = compose_generator_voltage_profile(nbus=self.nbus,
+                                                                   ntime=self.ntime,
+                                                                   gen_bus_indices=self.generator_data.get_bus_indices(),
+                                                                   gen_vset=self.generator_data.generator_v,
+                                                                   gen_status=self.generator_data.generator_active,
+                                                                   gen_is_controlled=self.generator_data.generator_controllable,
+                                                                   bat_bus_indices=self.battery_data.get_bus_indices(),
+                                                                   bat_vset=self.battery_data.battery_v,
+                                                                   bat_status=self.battery_data.battery_active,
+                                                                   bat_is_controlled=self.battery_data.battery_controllable,
+                                                                   hvdc_bus_f=self.hvdc_data.get_bus_indices_f(),
+                                                                   hvdc_bus_t=self.hvdc_data.get_bus_indices_t(),
+                                                                   hvdc_status=self.hvdc_data.active,
+                                                                   hvdc_vf=self.hvdc_data.Vset_f,
+                                                                   hvdc_vt=self.hvdc_data.Vset_t,
+                                                                   iBeqv=np.array(self.iBeqv, dtype=int),
+                                                                   iVtma=np.array(self.iVtma, dtype=int),
+                                                                   VfBeqbus=np.array(self.VfBeqbus, dtype=int),
+                                                                   Vtmabus=np.array(self.Vtmabus, dtype=int),
+                                                                   branch_status=self.branch_data.branch_active,
+                                                                   br_vf=self.branch_data.vf_set,
+                                                                   br_vt=self.branch_data.vt_set)
 
         self.determine_control_indices()
 
@@ -553,18 +598,16 @@ class SnapshotData:
                 self.iVscL.append(k)
                 self.any_control = True
 
-            elif tpe == ConverterControlType.type_III_8:  # 6:Droop+Qac
-                self.iPfdp_va.append(k)
-                self.iQtma.append(k)
-
+            elif tpe == ConverterControlType.type_IV_I:  # 8:Vdc
+                self.iBeqv.append(k)
                 self.iVscL.append(k)
+
                 self.any_control = True
 
-            elif tpe == ConverterControlType.type_III_9:  # 4a:Droop-slack
-                self.iPfdp_va.append(k)
-                self.iVtma.append(k)
+            elif tpe == ConverterControlType.type_IV_II:  # 9:Pdc
+                self.iPfsh.append(k)
+                self.iBeqz.append(k)
 
-                self.iVscL.append(k)
                 self.any_control = True
 
             elif tpe == 0:
@@ -597,6 +640,9 @@ class SnapshotData:
         self.iPfdp_va = np.array(self.iPfdp_va, dtype=np.int)
         self.iVscL = np.array(self.iVscL, dtype=np.int)
 
+    def get_branch_df(self, t=0):
+        return self.branch_data.to_df(t)
+
     @property
     def line_idx(self):
         return slice(0, self.nline, 1)
@@ -637,9 +683,17 @@ class SnapshotData:
     def Ibus(self):
 
         if self.Ibus_ is None:
-            self.Ibus_ = np.zeros(len(self.bus_data), dtype=complex)
+            self.Ibus_ = self.load_data.get_current_injections_per_bus() / self.Sbase
 
-        return self.Ibus_
+        return self.Ibus_[:, 0]
+
+    @property
+    def YLoadBus(self):
+
+        if self.YloadBus_ is None:
+            self.YloadBus_ = self.load_data.get_admittance_injections_per_bus() / self.Sbase
+
+        return self.YloadBus_[:, 0]
 
     @property
     def Rates(self):
@@ -770,25 +824,13 @@ class SnapshotData:
     def branch_rates(self):
         return self.branch_data.branch_rates[:, 0]
 
-    @property
-    def hvdc_Pf(self):
-        return self.hvdc_data.Pf[:, 0]
-
-    @property
-    def hvdc_Pt(self):
-        return self.hvdc_data.Pt[:, 0]
-
-    @property
-    def hvdc_loading(self):
-        return self.hvdc_data.get_loading()[:, 0]
-
-    @property
-    def hvdc_losses(self):
-        return self.hvdc_data.get_losses()[:, 0]
 
     @property
     def ac_indices(self):
-
+        """
+        Array of indices of the AC branches
+        :return: array of indices
+        """
         if self.ac_ is None:
             self.ac_ = self.branch_data.get_ac_indices()
 
@@ -796,6 +838,10 @@ class SnapshotData:
 
     @property
     def dc_indices(self):
+        """
+        Array of indices of the DC branches
+        :return: array of indices
+        """
         if self.dc_ is None:
             self.dc_ = self.branch_data.get_dc_indices()
 
@@ -803,7 +849,10 @@ class SnapshotData:
 
     @property
     def Cf(self):
-
+        """
+        Connectivity matrix of the "from" nodes
+        :return: CSC matrix
+        """
         # compute on demand and store
         if self.Cf_ is None:
             self.Cf_, self.Ct_ = ycalc.compute_connectivity(branch_active=self.branch_data.branch_active[:, 0],
@@ -817,7 +866,10 @@ class SnapshotData:
 
     @property
     def Ct(self):
-
+        """
+        Connectivity matrix of the "to" nodes
+        :return: CSC matrix
+        """
         # compute on demand and store
         if self.Ct_ is None:
             self.Cf_, self.Ct_ = ycalc.compute_connectivity(branch_active=self.branch_data.branch_active[:, 0],
@@ -829,8 +881,24 @@ class SnapshotData:
 
         return self.Ct_
 
+    @property
+    def A(self):
+        """
+        Connectivity matrix
+        :return: CSC matrix
+        """
+
+        if self.A_ is None:
+            self.A_ = (self.Cf - self.Ct).tocsc()
+
+        return self.A_
+
     @property
     def Ybus(self):
+        """
+        Admittance matrix
+        :return: CSC matrix
+        """
 
         # compute admittances on demand
         if self.Admittances is None:
@@ -840,10 +908,10 @@ class SnapshotData:
                                                          G=self.branch_data.G,
                                                          B=self.branch_data.B,
                                                          k=self.branch_data.k,
-                                                         m=self.branch_data.m[:, 0],
-                                                         mf=self.branch_data.tap_f,
-                                                         mt=self.branch_data.tap_t,
-                                                         theta=self.branch_data.theta[:, 0],
+                                                         tap_module=self.branch_data.m[:, 0],
+                                                         vtap_f=self.branch_data.tap_f,
+                                                         vtap_t=self.branch_data.tap_t,
+                                                         tap_angle=self.branch_data.theta[:, 0],
                                                          Beq=self.branch_data.Beq[:, 0],
                                                          Cf=self.Cf,
                                                          Ct=self.Ct,
@@ -857,7 +925,10 @@ class SnapshotData:
 
     @property
     def Yf(self):
-
+        """
+        Admittance matrix of the "from" nodes with the branches
+        :return: CSC matrix
+        """
         if self.Admittances is None:
             x = self.Ybus  # call the constructor of Yf
 
@@ -865,7 +936,10 @@ class SnapshotData:
 
     @property
     def Yt(self):
-
+        """
+        Admittance matrix of the "to" nodes with the branches
+        :return: CSC matrix
+        """
         if self.Admittances is None:
             x = self.Ybus  # call the constructor of Yt
 
@@ -873,7 +947,10 @@ class SnapshotData:
 
     @property
     def Yseries(self):
-
+        """
+        Admittance matrix of the series elements of the pi model of the branches
+        :return: CSC matrix
+        """
         # compute admittances on demand
         if self.Yseries_ is None:
 
@@ -899,7 +976,10 @@ class SnapshotData:
 
     @property
     def Yshunt(self):
-
+        """
+        Array of shunt admittances of the pi model of the branches (used in HELM mostly)
+        :return: Array of complex values
+        """
         if self.Yshunt_ is None:
             x = self.Yseries  # call the constructor of Yshunt
 
@@ -911,7 +991,10 @@ class SnapshotData:
 
     @property
     def B1(self):
-
+        """
+        B' matrix of the fast decoupled method
+        :return:
+        """
         if self.B1_ is None:
 
             self.B1_, self.B2_ = ycalc.compute_fast_decoupled_admittances(X=self.branch_data.X,
@@ -925,7 +1008,10 @@ class SnapshotData:
 
     @property
     def B2(self):
-
+        """
+        B'' matrix of the fast decoupled method
+        :return:
+        """
         if self.B2_ is None:
             x = self.B1  # call the constructor of B2
 
@@ -933,30 +1019,44 @@ class SnapshotData:
 
     @property
     def Bbus(self):
-
+        """
+        Susceptance matrix for the linear methods
+        :return:
+        """
         if self.Bbus_ is None:
-            self.Bbus_, self.Bf_ = ycalc.compute_linear_admittances(nbr=self.nbr,
-                                                                    X=self.branch_data.X,
-                                                                    R=self.branch_data.R,
-                                                                    m=self.branch_data.m[:, 0],
-                                                                    active=self.branch_data.branch_active[:, 0],
-                                                                    Cf=self.Cf,
-                                                                    Ct=self.Ct,
-                                                                    ac=self.ac_indices,
-                                                                    dc=self.dc_indices)
-            self.Bpqpv_ = self.Bbus_[np.ix_(self.pqpv, self.pqpv)]
-            self.Bref_ = self.Bbus_[np.ix_(self.pqpv, self.vd)]
+            self.Bbus_, self.Bf_, self.Btheta_ = ycalc.compute_linear_admittances(nbr=self.nbr,
+                                                                                  X=self.branch_data.X,
+                                                                                  R=self.branch_data.R,
+                                                                                  m=self.branch_data.m[:, 0],
+                                                                                  active=self.branch_data.branch_active[:, 0],
+                                                                                  Cf=self.Cf,
+                                                                                  Ct=self.Ct,
+                                                                                  ac=self.ac_indices,
+                                                                                  dc=self.dc_indices)
+            self.Bpqpv_ = self.Bbus_[np.ix_(self.pqpv, self.pqpv)].tocsc()
+            self.Bref_ = self.Bbus_[np.ix_(self.pqpv, self.vd)].tocsc()
 
         return self.Bbus_
 
     @property
     def Bf(self):
-
+        """
+        Susceptance matrix of the "from" nodes to the branches
+        :return:
+        """
         if self.Bf_ is None:
             x = self.Bbus  # call the constructor of Bf
 
         return self.Bf_
 
+    @property
+    def Btheta(self):
+
+        if self.Bf_ is None:
+            x = self.Bbus  # call the constructor of Bf
+
+        return self.Btheta_
+
     @property
     def Bpqpv(self):
 
@@ -1047,6 +1147,56 @@ class SnapshotData:
 
         return Bmax, Bmin
 
+    def get_inter_areas_branches(self, buses_areas_1, buses_areas_2):
+        """
+        Get the branches that join two areas
+        :param buses_areas_1: Area from
+        :param buses_areas_2: Area to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        return get_inter_areas_branch(self.branch_data.F, self.branch_data.T, buses_areas_1, buses_areas_2)
+
+    def get_inter_areas_hvdc(self, buses_areas_1, buses_areas_2):
+        """
+        Get the branches that join two areas
+        :param buses_areas_1: Area from
+        :param buses_areas_2: Area to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        F = self.hvdc_data.get_bus_indices_f()
+        T = self.hvdc_data.get_bus_indices_t()
+        return get_inter_areas_branch(F, T, buses_areas_1, buses_areas_2)
+
+    def get_generators_per_areas(self, buses_in_a1, buses_in_a2):
+        """
+        Get the generators that belong to the Area 1, Area 2 and the rest of areas
+        :param buses_in_a1: List of bus indices of the area 1
+        :param buses_in_a2: List of bus indices of the area 2
+        :return: Tree lists: (gens_in_a1, gens_in_a2, gens_out)
+                 each of the lists contains (bus index, generator index) tuples
+        """
+        if isinstance(self.generator_data.C_bus_gen, sp.csc_matrix):
+            Cgen = self.generator_data.C_bus_gen
+        else:
+            Cgen = self.generator_data.C_bus_gen.tocsc()
+
+        return get_devices_per_areas(Cgen, buses_in_a1, buses_in_a2)
+
+    def get_batteries_per_areas(self, buses_in_a1, buses_in_a2):
+        """
+        Get the batteries that belong to the Area 1, Area 2 and the rest of areas
+        :param buses_in_a1: List of bus indices of the area 1
+        :param buses_in_a2: List of bus indices of the area 2
+        :return: Tree lists: (batteries_in_a1, batteries_in_a2, batteries_out)
+                 each of the lists contains (bus index, generator index) tuples
+        """
+        if isinstance(self.battery_data.C_bus_batt, sp.csc_matrix):
+            Cgen = self.battery_data.C_bus_batt
+        else:
+            Cgen = self.battery_data.C_bus_batt.tocsc()
+
+        return get_devices_per_areas(Cgen, buses_in_a1, buses_in_a2)
+
     def get_structure(self, structure_type) -> pd.DataFrame:
         """
         Get a DataFrame with the input.
@@ -1071,11 +1221,31 @@ class SnapshotData:
         elif structure_type == 'Ibus':
             df = pd.DataFrame(data=self.Ibus, columns=['Current (p.u.)'], index=self.bus_data.bus_names)
 
+        elif structure_type == 'tap_f':
+            df = pd.DataFrame(data=self.branch_data.tap_f,
+                              columns=['Virtual tap from (p.u.)'],
+                              index=self.branch_data.branch_names)
+
+        elif structure_type == 'tap_t':
+            df = pd.DataFrame(data=self.branch_data.tap_t,
+                              columns=['Virtual tap to (p.u.)'],
+                              index=self.branch_data.branch_names)
+
         elif structure_type == 'Ybus':
             df = pd.DataFrame(data=self.Ybus.toarray(),
                               columns=self.bus_data.bus_names,
                               index=self.bus_data.bus_names)
 
+        elif structure_type == 'G':
+            df = pd.DataFrame(data=self.Ybus.real.toarray(),
+                              columns=self.bus_data.bus_names,
+                              index=self.bus_data.bus_names)
+
+        elif structure_type == 'B':
+            df = pd.DataFrame(data=self.Ybus.imag.toarray(),
+                              columns=self.bus_data.bus_names,
+                              index=self.bus_data.bus_names)
+
         elif structure_type == 'Yf':
             df = pd.DataFrame(data=self.Yf.toarray(),
                               columns=self.bus_data.bus_names,
@@ -1183,7 +1353,7 @@ class SnapshotData:
             npq = len(self.pq)
             npv = len(self.pv)
             npqpv = npq + npv
-            cols = ['dS/dVa'] * npqpv + ['dS/dVm'] * npq
+            cols = ['dS/dVa {0}'.format(i) for i in range(npqpv)] + ['dS/dVm {0}'.format(i) for i in range(npq)]
             rows = cols
             df = pd.DataFrame(data=J.toarray(), columns=cols, index=rows)
 
@@ -1230,6 +1400,11 @@ class SnapshotData:
         :return: SnapshotData
         """
 
+        # if the island is the same as the original bus indices, no slicing is needed
+        if len(bus_idx) == len(self.original_bus_idx):
+            if np.all(bus_idx == self.original_bus_idx):
+                return self
+
         # find the indices of the devices of the island
         line_idx = self.line_data.get_island(bus_idx)
         dc_line_idx = self.dc_line_data.get_island(bus_idx)
@@ -1288,12 +1463,13 @@ class SnapshotData:
         nc.generator_data = self.generator_data.slice(gen_idx, bus_idx, time_idx)
         nc.shunt_data = self.shunt_data.slice(shunt_idx, bus_idx, time_idx)
 
+        nc.determine_control_indices()
+
         return nc
 
     def split_into_islands(self, ignore_single_node_islands=False) -> List["SnapshotData"]:
         """
         Split circuit into islands
-        :param numeric_circuit: NumericCircuit instance
         :param ignore_single_node_islands: ignore islands composed of only one bus
         :return: List[NumericCircuit]
         """
@@ -1305,42 +1481,36 @@ class SnapshotData:
                                     bus_active=self.bus_data.bus_active[:, 0])
 
         # find the matching islands
-        idx_islands = tp.find_islands(A)
+        idx_islands = tp.find_islands(A, active=self.bus_data.bus_active[:, 0])
 
-        if len(idx_islands) == 1:
-            # numeric_circuit.compute_all()  # compute the internal magnitudes
-            return [self]
+        circuit_islands = list()  # type: List[SnapshotData]
 
-        else:
+        for bus_idx in idx_islands:
 
-            circuit_islands = list()  # type: List[SnapshotData]
+            if ignore_single_node_islands:
 
-            for bus_idx in idx_islands:
-
-                if ignore_single_node_islands:
-
-                    if len(bus_idx) > 1:
-                        island = self.get_island(bus_idx)
-                        # island.compute_all()  # compute the internal magnitudes
-                        circuit_islands.append(island)
-
-                else:
+                if len(bus_idx) > 1:
                     island = self.get_island(bus_idx)
-                    # island.compute_all()  # compute the internal magnitudes
                     circuit_islands.append(island)
 
-            return circuit_islands
+            else:
+                island = self.get_island(bus_idx)
+                circuit_islands.append(island)
+
+        return circuit_islands
 
 
 def compile_snapshot_circuit(circuit: MultiCircuit, apply_temperature=False,
                              branch_tolerance_mode=BranchImpedanceMode.Specified,
-                             opf_results=None) -> SnapshotData:
+                             opf_results=None,
+                             use_stored_guess=False) -> SnapshotData:
     """
 
     :param circuit:
     :param apply_temperature:
     :param branch_tolerance_mode:
     :param opf_results:
+    :param use_stored_guess:
     :return:
     """
 
@@ -1363,24 +1533,67 @@ def compile_snapshot_circuit(circuit: MultiCircuit, apply_temperature=False,
 
     bus_dict = {bus: i for i, bus in enumerate(circuit.buses)}
 
-    nc.bus_data = ds.circuit_to_data.get_bus_data(circuit)
-    nc.load_data = ds.circuit_to_data.get_load_data(circuit, bus_dict, opf_results)
-    nc.static_generator_data = ds.circuit_to_data.get_static_generator_data(circuit, bus_dict)
-    nc.generator_data = ds.circuit_to_data.get_generator_data(circuit, bus_dict, nc.bus_data.Vbus, logger, opf_results)
-    nc.battery_data = ds.circuit_to_data.get_battery_data(circuit, bus_dict, nc.bus_data.Vbus, logger, opf_results)
-    nc.shunt_data = ds.circuit_to_data.get_shunt_data(circuit, bus_dict, nc.bus_data.Vbus, logger)
-    nc.line_data = ds.circuit_to_data.get_line_data(circuit, bus_dict, apply_temperature, branch_tolerance_mode)
-    nc.transformer_data = ds.circuit_to_data.get_transformer_data(circuit, bus_dict)
-    nc.vsc_data = ds.circuit_to_data.get_vsc_data(circuit, bus_dict)
-    nc.upfc_data = ds.circuit_to_data.get_upfc_data(circuit, bus_dict)
-    nc.dc_line_data = ds.circuit_to_data.get_dc_line_data(circuit, bus_dict, apply_temperature, branch_tolerance_mode)
-    nc.upfc_data = ds.circuit_to_data.get_upfc_data(circuit, bus_dict)
+    nc.bus_data = gc_compiler.get_bus_data(circuit=circuit, use_stored_guess=use_stored_guess)
 
-    nc.branch_data = ds.circuit_to_data.get_branch_data(circuit, bus_dict, nc.bus_data.Vbus,
-                                                        apply_temperature, branch_tolerance_mode)
+    nc.load_data = gc_compiler.get_load_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             opf_results=opf_results)
 
-    nc.hvdc_data = ds.circuit_to_data.get_hvdc_data(circuit, bus_dict, nc.bus_data.bus_types)
+    nc.static_generator_data = gc_compiler.get_static_generator_data(circuit=circuit,
+                                                                     bus_dict=bus_dict)
 
-    nc.consolidate_information()
+    nc.generator_data = gc_compiler.get_generator_data(circuit=circuit,
+                                                       bus_dict=bus_dict,
+                                                       Vbus=nc.bus_data.Vbus,
+                                                       logger=logger,
+                                                       opf_results=opf_results,
+                                                       use_stored_guess=use_stored_guess)
+
+    nc.battery_data = gc_compiler.get_battery_data(circuit=circuit,
+                                                   bus_dict=bus_dict,
+                                                   Vbus=nc.bus_data.Vbus,
+                                                   logger=logger,
+                                                   opf_results=opf_results,
+                                                   use_stored_guess=use_stored_guess)
+
+    nc.shunt_data = gc_compiler.get_shunt_data(circuit=circuit,
+                                               bus_dict=bus_dict,
+                                               Vbus=nc.bus_data.Vbus,
+                                               logger=logger,
+                                               use_stored_guess=use_stored_guess)
+
+    nc.line_data = gc_compiler.get_line_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             apply_temperature=apply_temperature,
+                                             branch_tolerance_mode=branch_tolerance_mode)
+
+    nc.transformer_data = gc_compiler.get_transformer_data(circuit=circuit,
+                                                           bus_dict=bus_dict)
+
+    nc.vsc_data = gc_compiler.get_vsc_data(circuit=circuit,
+                                           bus_dict=bus_dict)
+
+    nc.upfc_data = gc_compiler.get_upfc_data(circuit=circuit,
+                                             bus_dict=bus_dict)
+
+    nc.dc_line_data = gc_compiler.get_dc_line_data(circuit=circuit,
+                                                   bus_dict=bus_dict,
+                                                   apply_temperature=apply_temperature,
+                                                   branch_tolerance_mode=branch_tolerance_mode)
+
+    nc.branch_data = gc_compiler.get_branch_data(circuit=circuit,
+                                                 bus_dict=bus_dict,
+                                                 Vbus=nc.bus_data.Vbus,
+                                                 apply_temperature=apply_temperature,
+                                                 branch_tolerance_mode=branch_tolerance_mode,
+                                                 opf_results=opf_results,
+                                                 use_stored_guess=use_stored_guess)
+
+    nc.hvdc_data = gc_compiler.get_hvdc_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             bus_types=nc.bus_data.bus_types,
+                                             opf_results=opf_results)
+
+    nc.consolidate_information(use_stored_guess=use_stored_guess)
 
     return nc
