@@ -1,35 +1,32 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
-import numba as nb
-import pandas as pd
-import scipy.sparse as sp
-from typing import List, Dict
+from typing import List
 
 from GridCal.Engine.basic_structures import Logger
 import GridCal.Engine.Core.topology as tp
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.Core.snapshot_pf_data import SnapshotData
 from GridCal.Engine.basic_structures import BranchImpedanceMode
-from GridCal.Engine.basic_structures import BusMode
-from GridCal.Engine.Simulations.PowerFlow.jacobian_based_power_flow import Jacobian
 from GridCal.Engine.Core.common_functions import compile_types, find_different_states
 from GridCal.Engine.Simulations.sparse_solve import get_sparse_type
 # from GridCal.Engine.Simulations.OPF.opf_ts_results import OptimalPowerFlowTimeSeriesResults
-import GridCal.Engine.Core.DataStructures as ds
+import GridCal.Engine.Core.Compilers.circuit_to_data as gc_compiler
 
 
 sparse_type = get_sparse_type()
@@ -92,6 +89,14 @@ class TimeCircuit(SnapshotData):
 
         return self.Ibus_
 
+    @property
+    def YLoadBus(self):
+
+        if self.YloadBus_ is None:
+            self.YloadBus_ = self.load_data.get_admittance_injections_per_bus() / self.Sbase
+
+        return self.YloadBus_
+
     @property
     def Rates(self):
         return self.branch_data.branch_rates
@@ -120,7 +125,8 @@ class TimeCircuit(SnapshotData):
         self.pqpv_prof_ = list()
 
         for t in range(self.ntime):
-            vd, pq, pv, pqpv = compile_types(Sbus=self.Sbus[:, t], types=self.bus_data.bus_types_prof[:, t])
+            vd, pq, pv, pqpv = compile_types(Sbus=self.Sbus[:, t],
+                                             types=self.bus_data.bus_types_prof[:, t])
             self.vd_prof_.append(vd)
             self.pq_prof_.append(pq)
             self.pv_prof_.append(pv)
@@ -163,7 +169,7 @@ class TimeCircuit(SnapshotData):
 
     @property
     def hvdc_Pf(self):
-        return self.hvdc_data.Pf
+        return self.hvdc_data.Pset
 
     @property
     def hvdc_Pt(self):
@@ -185,6 +191,13 @@ class TimeCircuit(SnapshotData):
         :return: SnapshotData
         """
 
+        # if the island is the same as the original bus indices, no slicing is needed
+        if len(bus_idx) == len(self.original_bus_idx):
+            if np.all(bus_idx == self.original_bus_idx):
+                if len(time_idx) == len(self.original_time_idx):
+                    if np.all(time_idx == self.original_time_idx):
+                        return self
+
         # find the indices of the devices of the island
         line_idx = self.line_data.get_island(bus_idx)
         dc_line_idx = self.dc_line_data.get_island(bus_idx)
@@ -269,27 +282,27 @@ class TimeCircuit(SnapshotData):
                                         bus_active=self.bus_data.bus_active[:, 0])
 
             # find the matching islands
-            idx_islands = tp.find_islands(A)
+            idx_islands = tp.find_islands(A, active=self.bus_data.bus_active[:, 0])
 
-            if len(idx_islands) == 1:  # only one state and only one island -> just copy the data
+            # if len(idx_islands) == 1:  # only one state and only one island -> just copy the data
+            #
+            #     return [self]
+            #
+            # else:  # one state, many islands -> split by bus index, keep the time
 
-                return [self]
+            for bus_idx in idx_islands:
 
-            else:  # one state, many islands -> split by bus index, keep the time
+                if ignore_single_node_islands:
 
-                for bus_idx in idx_islands:
-
-                    if ignore_single_node_islands:
-
-                        if len(bus_idx) > 1:
-                            island = self.get_island(bus_idx, all_time)
-                            circuit_islands.append(island)
-
-                    else:
+                    if len(bus_idx) > 1:
                         island = self.get_island(bus_idx, all_time)
                         circuit_islands.append(island)
 
-                return circuit_islands
+                else:
+                    island = self.get_island(bus_idx, all_time)
+                    circuit_islands.append(island)
+
+            return circuit_islands
 
         else:  # ------------------------------------------------------------------------------------------------------
 
@@ -302,40 +315,44 @@ class TimeCircuit(SnapshotData):
                                             bus_active=self.bus_data.bus_active[:, t_array])
 
                 # find the matching islands
-                idx_islands = tp.find_islands(A)
+                idx_islands = tp.find_islands(A, active=self.bus_data.bus_active[:, t_array])
 
-                if len(idx_islands) == 1:  # many time states, one island -> slice only by time ------------------------
+                # if len(idx_islands) == 1:  # many time states, one island -> slice only by time ------------------------
+                #
+                #     island = self.get_island(all_buses, t_array)  # convert the circuit to an island
+                #
+                #     circuit_islands.append(island)
+                #
+                # else:
 
-                    island = self.get_island(all_buses, t_array)  # convert the circuit to an island
+                # any time states, many islands -> slice by both time and bus index -----------------------------
 
-                    circuit_islands.append(island)
+                for bus_idx in idx_islands:
 
-                else:  # any time states, many islands -> slice by both time and bus index -----------------------------
+                    if ignore_single_node_islands:
 
-                    for bus_idx in idx_islands:
-
-                        if ignore_single_node_islands:
-
-                            if len(bus_idx) > 1:
-                                island = self.get_island(bus_idx, t_array)
-                                circuit_islands.append(island)
-
-                        else:
+                        if len(bus_idx) > 1:
                             island = self.get_island(bus_idx, t_array)
                             circuit_islands.append(island)
 
+                    else:
+                        island = self.get_island(bus_idx, t_array)
+                        circuit_islands.append(island)
+
             return circuit_islands
 
 
 def compile_time_circuit(circuit: MultiCircuit, apply_temperature=False,
                          branch_tolerance_mode=BranchImpedanceMode.Specified,
-                         opf_results=None) -> TimeCircuit:
+                         opf_results=None,
+                         use_stored_guess=True) -> TimeCircuit:
     """
     Compile the information of a circuit and generate the pertinent power flow islands
     :param circuit: Circuit instance
     :param apply_temperature:
     :param branch_tolerance_mode:
     :param opf_results: OptimalPowerFlowTimeSeriesResults instance
+    :param use_stored_guess:
     :return: list of NumericIslands
     """
 
@@ -359,42 +376,97 @@ def compile_time_circuit(circuit: MultiCircuit, apply_temperature=False,
 
     bus_dict = {bus: i for i, bus in enumerate(circuit.buses)}
 
-    nc.bus_data = ds.circuit_to_data.get_bus_data(circuit, time_series=True, ntime=ntime)
+    nc.bus_data = gc_compiler.get_bus_data(circuit=circuit,
+                                           time_series=True,
+                                           ntime=ntime,
+                                           use_stored_guess=use_stored_guess)
 
-    nc.load_data = ds.circuit_to_data.get_load_data(circuit, bus_dict, opf_results, time_series=True, ntime=ntime)
+    nc.load_data = gc_compiler.get_load_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             opf_results=opf_results,
+                                             time_series=True,
+                                             ntime=ntime)
 
-    nc.static_generator_data = ds.circuit_to_data.get_static_generator_data(circuit, bus_dict,
-                                                                            time_series=True, ntime=ntime)
+    nc.static_generator_data = gc_compiler.get_static_generator_data(circuit=circuit,
+                                                                     bus_dict=bus_dict,
+                                                                     time_series=True,
+                                                                     ntime=ntime)
 
-    nc.generator_data = ds.circuit_to_data.get_generator_data(circuit, bus_dict, nc.bus_data.Vbus, logger, opf_results,
-                                                              time_series=True, ntime=ntime)
+    nc.generator_data = gc_compiler.get_generator_data(circuit=circuit,
+                                                       bus_dict=bus_dict,
+                                                       Vbus=nc.bus_data.Vbus,
+                                                       logger=logger,
+                                                       opf_results=opf_results,
+                                                       time_series=True,
+                                                       ntime=ntime,
+                                                       use_stored_guess=use_stored_guess)
 
-    nc.battery_data = ds.circuit_to_data.get_battery_data(circuit, bus_dict, nc.bus_data.Vbus, logger, opf_results,
-                                                          time_series=True, ntime=ntime)
+    nc.battery_data = gc_compiler.get_battery_data(circuit=circuit,
+                                                   bus_dict=bus_dict,
+                                                   Vbus=nc.bus_data.Vbus,
+                                                   logger=logger,
+                                                   opf_results=opf_results,
+                                                   time_series=True,
+                                                   ntime=ntime,
+                                                   use_stored_guess=use_stored_guess)
 
-    nc.shunt_data = ds.circuit_to_data.get_shunt_data(circuit, bus_dict, nc.bus_data.Vbus, logger,
-                                                      time_series=True, ntime=ntime)
+    nc.shunt_data = gc_compiler.get_shunt_data(circuit=circuit,
+                                               bus_dict=bus_dict,
+                                               Vbus=nc.bus_data.Vbus,
+                                               logger=logger,
+                                               time_series=True,
+                                               ntime=ntime,
+                                               use_stored_guess=use_stored_guess)
 
-    nc.line_data = ds.circuit_to_data.get_line_data(circuit, bus_dict, apply_temperature, branch_tolerance_mode,
-                                                    time_series=True, ntime=ntime)
+    nc.line_data = gc_compiler.get_line_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             apply_temperature=apply_temperature,
+                                             branch_tolerance_mode=branch_tolerance_mode,
+                                             time_series=True,
+                                             ntime=ntime)
 
-    nc.transformer_data = ds.circuit_to_data.get_transformer_data(circuit, bus_dict, time_series=True, ntime=ntime)
+    nc.transformer_data = gc_compiler.get_transformer_data(circuit=circuit,
+                                                           bus_dict=bus_dict,
+                                                           time_series=True,
+                                                           ntime=ntime)
 
-    nc.vsc_data = ds.circuit_to_data.get_vsc_data(circuit, bus_dict, time_series=True, ntime=ntime)
+    nc.vsc_data = gc_compiler.get_vsc_data(circuit=circuit,
+                                           bus_dict=bus_dict,
+                                           time_series=True,
+                                           ntime=ntime)
 
-    nc.dc_line_data = ds.circuit_to_data.get_dc_line_data(circuit, bus_dict, apply_temperature, branch_tolerance_mode,
-                                                          time_series=True, ntime=ntime)
+    nc.dc_line_data = gc_compiler.get_dc_line_data(circuit=circuit,
+                                                   bus_dict=bus_dict,
+                                                   apply_temperature=apply_temperature,
+                                                   branch_tolerance_mode=branch_tolerance_mode,
+                                                   time_series=True,
+                                                   ntime=ntime)
 
-    nc.upfc_data = ds.circuit_to_data.get_upfc_data(circuit, bus_dict, time_series=True, ntime=ntime)
+    nc.upfc_data = gc_compiler.get_upfc_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             time_series=True,
+                                             ntime=ntime)
 
-    nc.branch_data = ds.circuit_to_data.get_branch_data(circuit, bus_dict, nc.bus_data.Vbus,
-                                                        apply_temperature, branch_tolerance_mode,
-                                                        time_series=True, ntime=ntime)
+    nc.branch_data = gc_compiler.get_branch_data(circuit=circuit,
+                                                 bus_dict=bus_dict,
+                                                 Vbus=nc.bus_data.Vbus,
+                                                 apply_temperature=apply_temperature,
+                                                 branch_tolerance_mode=branch_tolerance_mode,
+                                                 time_series=True,
+                                                 ntime=ntime,
+                                                 opf_results=opf_results,
+                                                 use_stored_guess=use_stored_guess)
 
-    nc.hvdc_data = ds.circuit_to_data.get_hvdc_data(circuit, bus_dict, nc.bus_data.bus_types,
-                                                    time_series=True, ntime=ntime)
+    nc.hvdc_data = gc_compiler.get_hvdc_data(circuit=circuit,
+                                             bus_dict=bus_dict,
+                                             bus_types=nc.bus_data.bus_types,
+                                             time_series=True,
+                                             ntime=ntime,
+                                             opf_results=opf_results)
 
     nc.consolidate_information()
 
     return nc
 
+
+
