@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from typing import List
 import numpy as np
@@ -19,20 +21,20 @@ import numba as nb
 from scipy.sparse import csc_matrix, diags
 
 
-def find_islands(adj: csc_matrix):
+# @nb.jit(cache=True)
+def find_islands_numba(node_number, indptr, indices, active: np.ndarray):
     """
     Method to get the islands of a graph
     This is the non-recursive version
     :return: list of islands, where each element is a list of the node indices of the island
     """
 
-    node_number = adj.shape[0]
-
     # Mark all the vertices as not visited
-    visited = np.zeros(node_number, dtype=bool)
+    visited = np.zeros(node_number, dtype=int)
 
     # storage structure for the islands (list of lists)
     islands = list()  # type: List[List[int]]
+    # islands = nb.typeof([[0]])
 
     # set the island index
     island_idx = 0
@@ -41,7 +43,7 @@ def find_islands(adj: csc_matrix):
     for node in range(node_number):
 
         # if the node has not been visited...
-        if not visited[node]:
+        if not visited[node] and active[node]:
 
             # add new island, because the recursive process has already visited all the island connected to v
             # if island_idx >= len(islands):
@@ -63,17 +65,17 @@ def find_islands(adj: csc_matrix):
                 if not visited[v]:
 
                     # mark as visited
-                    visited[v] = True
+                    visited[v] = 1
 
                     # add element to the island
                     islands[island_idx].append(v)
 
                     # Add the neighbours of v to the stack
-                    start = adj.indptr[v]
-                    end = adj.indptr[v + 1]
+                    start = indptr[v]
+                    end = indptr[v + 1]
                     for i in range(start, end):
-                        k = adj.indices[i]  # get the row index in the CSC scheme
-                        if not visited[k]:
+                        k = indices[i]  # get the row index in the CSC scheme
+                        if not visited[k] and active[k]:
                             stack.append(k)
             # ------------------------------------------------------------------------------------------------------
 
@@ -87,7 +89,53 @@ def find_islands(adj: csc_matrix):
     return islands
 
 
-def get_elements_of_the_island(C_element_bus, island):
+@nb.njit(cache=True)
+def get_elements_of_the_island_numba(n_rows, indptr, indices, island, active):
+    """
+    Get the element indices of the island
+    :param n_rows: Number of rows of the connectivity matrix
+    :param indptr: CSC index pointers of the element-node connectivity matrix
+    :param indices: CSC indices of the element-node connectivity matrix
+    :param island: island node indices
+    :return: array of indices of the elements that match that island
+    """
+
+    visited = np.zeros(n_rows, dtype=nb.int32)
+    elm_idx = np.zeros(n_rows, dtype=nb.int32)
+    n_visited = 0
+
+    for k in range(len(island)):
+
+        j = island[k]  # column index
+
+        for l in range(indptr[j], indptr[j + 1]):
+
+            i = indices[l]  # row index
+
+            if not visited[i] and active[i]:
+                visited[i] = 1
+                elm_idx[n_visited] = i
+                n_visited += 1
+
+    # resize vector
+    elm_idx = elm_idx[:n_visited]
+
+    return elm_idx
+
+
+def find_islands(adj: csc_matrix, active: np.ndarray):
+    """
+    Method to get the islands of a graph
+    This is the non-recursive version
+    :return: list of islands, where each element is a list of the node indices of the island
+    """
+    return find_islands_numba(node_number=adj.shape[0],
+                              indptr=adj.indptr,
+                              indices=adj.indices,
+                              active=active)
+
+
+def get_elements_of_the_island(C_element_bus, island, active):
     """
     Get the branch indices of the island
     :param C_element_bus: CSC elements-buses connectivity matrix with the dimensions: elements x buses
@@ -99,28 +147,11 @@ def get_elements_of_the_island(C_element_bus, island):
         C_element_bus = C_element_bus.tocsc()
 
     # faster method
-    n_rows = C_element_bus.shape[0]
-    visited = np.zeros(n_rows, dtype=bool)
-    elm_idx = np.zeros(n_rows, dtype=int)
-    n_visited = 0
-
-    for k in range(len(island)):
-
-        j = island[k]  # column index
-
-        for l in range(C_element_bus.indptr[j], C_element_bus.indptr[j + 1]):
-
-            i = C_element_bus.indices[l]  # row index
-
-            if not visited[i]:
-                visited[i] = True
-                elm_idx[n_visited] = i
-                n_visited += 1
-
-    # resize vector
-    elm_idx = elm_idx[:n_visited]
-
-    return elm_idx
+    return get_elements_of_the_island_numba(n_rows=C_element_bus.shape[0],
+                                            indptr=C_element_bus.indptr,
+                                            indices=C_element_bus.indices,
+                                            island=np.array(island, dtype=int),
+                                            active=active)
 
 
 def get_adjacency_matrix(C_branch_bus_f, C_branch_bus_t, branch_active, bus_active):
@@ -147,40 +178,3 @@ def get_adjacency_matrix(C_branch_bus_f, C_branch_bus_t, branch_active, bus_acti
 
 
 
-
-
-class Graph:
-
-    def __init__(self, C_bus_bus, C_branch_bus, bus_states):
-        """
-        Graph adapted to work with CSC sparse matrices
-        see: http://www.scipy-lectures.org/advanced/scipy_sparse/csc_matrix.html
-        :param C_bus_bus: Adjacency matrix in lil format
-        :param C_branch_bus: Connectivity of the branches and the buses
-        :param bus_states: states of the branches
-        """
-        self.node_number = C_bus_bus.shape[0]
-
-        self.adj = C_bus_bus
-
-        self.C_branch_bus = C_branch_bus
-
-        self.bus_states = bus_states
-
-    def find_islands(self):
-        """
-        Method to get the islands of a graph
-        This is the non-recursive version
-        :return: List of islands where each element is a list of the node indices of the island
-        """
-
-        return find_islands(self.adj)
-
-    def get_branches_of_the_island(self, island):
-        """
-        Get the branch indices of the island
-        :param island: array of bus indices of the island
-        :return: array of indices of the branches that belong to the island
-        """
-
-        return get_elements_of_the_island(self.C_branch_bus, island)
