@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from GridCal.Engine.Devices.battery import *
 from GridCal.Engine.Devices.branch import *
@@ -20,6 +22,7 @@ from GridCal.Engine.Devices.vsc import *
 from GridCal.Engine.Devices.bus import *
 from GridCal.Engine.Devices.generator import *
 from GridCal.Engine.Devices.load import *
+from GridCal.Engine.Devices.external_grid import *
 from GridCal.Engine.Devices.line import *
 from GridCal.Engine.Devices.switch import *
 from GridCal.Engine.Devices.hvdc_line import *
@@ -28,8 +31,10 @@ from GridCal.Engine.Devices.shunt import *
 from GridCal.Engine.Devices.static_generator import *
 from GridCal.Engine.Devices.tower import *
 from GridCal.Engine.Devices.transformer import *
+from GridCal.Engine.Devices.transformer3w import *
 from GridCal.Engine.Devices.underground_line import *
 from GridCal.Engine.Devices.wire import *
 from GridCal.Engine.Devices.measurement import *
 from GridCal.Engine.Devices.groupings import *
 from GridCal.Engine.Devices.templates import *
+from GridCal.Engine.Devices.enumerations import *
