@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
 from warnings import warn
@@ -244,42 +246,71 @@ class Battery(Generator):
 
         return batt
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Get json dictionary
         :return: json-compatible dictionary
         """
+        if version == 2:
+            return {'id': self.idtag,
+                    'type': 'battery',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus': self.bus.idtag,
+                    'active': self.active,
 
-        data = {'id': self.idtag,
-                'type': 'battery',
-                'phases': 'ps',
-                'name': self.name,
-                'name_code': self.code,
-                'bus': self.bus.idtag,
-                'active': self.active,
-
-                'p': self.P,
-                'vset': self.Vset,
-                'pf': self.Pf,
-                'snom': self.Snom,
-                'enom': self.Enom,
-                'qmin': self.Qmin,
-                'qmax': self.Qmax,
-                'pmin': self.Pmin,
-                'pmax': self.Pmax,
-                'cost': self.Cost,
-                'charge_efficiency': self.charge_efficiency,
-                'discharge_efficiency': self.discharge_efficiency,
-                'min_soc': self.min_soc,
-                'max_soc': self.max_soc,
-                'soc_0': self.soc_0,
-                'min_soc_charge': self.min_soc_charge,
-                'charge_per_cycle': self.charge_per_cycle,
-                'discharge_per_cycle': self.discharge_per_cycle,
-                'technology': ""
-                }
-
-        return data
+                    'p': self.P,
+                    'vset': self.Vset,
+                    'pf': self.Pf,
+                    'snom': self.Snom,
+                    'enom': self.Enom,
+                    'qmin': self.Qmin,
+                    'qmax': self.Qmax,
+                    'pmin': self.Pmin,
+                    'pmax': self.Pmax,
+                    'cost': self.Cost,
+                    'charge_efficiency': self.charge_efficiency,
+                    'discharge_efficiency': self.discharge_efficiency,
+                    'min_soc': self.min_soc,
+                    'max_soc': self.max_soc,
+                    'soc_0': self.soc_0,
+                    'min_soc_charge': self.min_soc_charge,
+                    'charge_per_cycle': self.charge_per_cycle,
+                    'discharge_per_cycle': self.discharge_per_cycle,
+                    'technology': ""
+                    }
+        elif version == 3:
+            return {'id': self.idtag,
+                    'type': 'battery',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus': self.bus.idtag,
+                    'active': self.active,
+                    'is_controlled': self.is_controlled,
+                    'p': self.P,
+                    'vset': self.Vset,
+                    'pf': self.Pf,
+                    'snom': self.Snom,
+                    'enom': self.Enom,
+                    'qmin': self.Qmin,
+                    'qmax': self.Qmax,
+                    'pmin': self.Pmin,
+                    'pmax': self.Pmax,
+                    'cost': self.Cost,
+                    'charge_efficiency': self.charge_efficiency,
+                    'discharge_efficiency': self.discharge_efficiency,
+                    'min_soc': self.min_soc,
+                    'max_soc': self.max_soc,
+                    'soc_0': self.soc_0,
+                    'min_soc_charge': self.min_soc_charge,
+                    'charge_per_cycle': self.charge_per_cycle,
+                    'discharge_per_cycle': self.discharge_per_cycle,
+                    'technology': ""
+                    }
+        else:
+            return dict()
 
     def get_profiles_dict(self):
         """
