@@ -1,23 +1,25 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
-from GridCal.Engine.basic_structures import BusMode
+from GridCal.Engine.basic_structures import BusMode, ExternalGridMode
 from GridCal.Engine.Devices.editable_device import EditableDevice, DeviceType, GCProp
 from GridCal.Engine.Devices.groupings import Area, Substation, Zone, Country
 
@@ -80,9 +82,11 @@ class Bus(EditableDevice):
 
     """
 
-    def __init__(self, name="Bus", idtag=None, code='', vnom=10, vmin=0.9, vmax=1.1, r_fault=0.0, x_fault=0.0,
+    def __init__(self, name="Bus", idtag=None, code='', vnom=10, vmin=0.9, vmax=1.1,
+                 angle_min=-6.28, angle_max=6.28, r_fault=0.0, x_fault=0.0,
                  xpos=0, ypos=0, height=0, width=0, active=True, is_slack=False, is_dc=False,
-                 area=None, zone=None, substation=None, country=None, longitude=0.0, latitude=0.0):
+                 area=None, zone=None, substation=None, country=None, longitude=0.0, latitude=0.0,
+                 Vm0=1, Va0=0):
 
         EditableDevice.__init__(self,
                                 name=name,
@@ -99,10 +103,16 @@ class Bus(EditableDevice):
                                                   'is_dc': GCProp('', bool, 'Is this bus of DC type?.'),
                                                   'Vnom': GCProp('kV', float,
                                                                  'Nominal line voltage of the bus.'),
+                                                  'Vm0': GCProp('p.u.', float, 'Voltage module guess.'),
                                                   'Vmin': GCProp('p.u.', float,
-                                                                 'Lower range of allowed voltage.'),
+                                                                 'Lower range of allowed voltage module.'),
                                                   'Vmax': GCProp('p.u.', float,
-                                                                 'Higher range of allowed range.'),
+                                                                 'Higher range of allowed voltage module.'),
+                                                  'Va0': GCProp('rad.', float, 'Voltage angle guess.'),
+                                                  'angle_min': GCProp('rad.', float,
+                                                                      'Lower range of allowed voltage angle.'),
+                                                  'angle_max': GCProp('rad.', float,
+                                                                      'Higher range of allowed voltage angle.'),
                                                   'r_fault': GCProp('p.u.', float,
                                                                     'Resistance of the fault.\n'
                                                                     'This is used for short circuit studies.'),
@@ -130,6 +140,14 @@ class Bus(EditableDevice):
         # maximum voltage limit
         self.Vmax = vmax
 
+        self.Vm0 = Vm0
+
+        self.Va0 = Va0
+
+        self.angle_min = angle_min
+
+        self.angle_max = angle_max
+
         # summation of lower reactive power limits connected
         self.Qmin_sum = 0
 
@@ -159,6 +177,9 @@ class Bus(EditableDevice):
         # List of Controlled generators attached to this bus
         self.controlled_generators = list()
 
+        # List of External Grids
+        self.external_grids = list()
+
         # List of shunt s attached to this bus
         self.shunts = list()
 
@@ -168,9 +189,6 @@ class Bus(EditableDevice):
         # List of static generators attached tot this bus
         self.static_generators = list()
 
-        # List of External grid devices
-        self.external_grids = list()
-
         # List of measurements
         self.measurements = list()
 
@@ -195,6 +213,16 @@ class Bus(EditableDevice):
         self.longitude = longitude
         self.latitude = latitude
 
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, val: str):
+        self._name = val
+        if self.graphic_obj is not None:
+            self.graphic_obj.set_label(self._name)
+
     def delete_children(self):
         """
         Delete all the children
@@ -202,6 +230,7 @@ class Bus(EditableDevice):
         self.batteries.clear()
         self.shunts.clear()
         self.static_generators.clear()
+        self.external_grids.clear()
         self.loads.clear()
         self.controlled_generators.clear()
 
@@ -231,28 +260,31 @@ class Bus(EditableDevice):
         else:
             raise Exception('Device type not understood:' + str(device.device_type))
 
-    def determine_bus_type(self):
+    def determine_bus_type(self) -> BusMode:
         """
         Infer the bus type from the devices attached to it
         @return: self.type
         """
+        if not self.active:
+            return BusMode.PQ
+
         if self.is_slack:
             # if it is set as slack, set the bus as slack and exit
             self.type = BusMode.Slack
-            return self.type
+            return BusMode.Slack
 
         elif len(self.external_grids) > 0:  # there are devices setting this as a slack bus
 
             # count the number of active external grids
             ext_on = 0
             for elm in self.external_grids:
-                if elm.active:
+                if elm.active and elm.mode == ExternalGridMode.VD:
                     ext_on += 1
 
             # if there ar any active external grids, set as slack and exit
             if ext_on > 0:
                 self.type = BusMode.Slack
-                return self.type
+                return BusMode.Slack
 
         # if we got here, determine what to do...
 
@@ -275,15 +307,77 @@ class Bus(EditableDevice):
 
         if (gen_on + batt_on + shunt_on) > 0:
             self.type = BusMode.PV
-
-        # elif shunt_on > 0:
-        #     self.type = BusMode.PVB
+            return BusMode.PV
 
         else:
             # Nothing special; set it as PQ
             self.type = BusMode.PQ
+            return BusMode.PQ
 
-        return self.type
+    def determine_bus_type_at(self, t) -> BusMode:
+        """
+        Infer the bus type from the devices attached to it
+        :param t: time index
+        @return: self.type
+        """
+        if not self.active_prof[t]:
+            return BusMode.PQ
+
+        if self.is_slack:
+            # if it is set as slack, set the bus as slack and exit
+            return BusMode.Slack
+
+        elif len(self.external_grids) > 0:  # there are devices setting this as a slack bus
+
+            # count the number of active external grids
+            ext_on = 0
+            for elm in self.external_grids:
+                if elm.active_prof[t] and elm.mode == ExternalGridMode.VD:
+                    ext_on += 1
+
+            # if there ar any active external grids, set as slack and exit
+            if ext_on > 0:
+                return BusMode.Slack
+
+        # if we got here, determine what to do...
+
+        # count the active and controlled generators
+        gen_on = 0
+        for elm in self.controlled_generators:
+            if elm.active_prof[t] and elm.is_controlled:
+                gen_on += 1
+
+        # count the active and controlled batteries
+        batt_on = 0
+        for elm in self.batteries:
+            if elm.active_prof[t] and elm.is_controlled:
+                batt_on += 1
+
+        shunt_on = 0
+        for elm in self.shunts:
+            if elm.active_prof[t] and elm.is_controlled:
+                shunt_on += 1
+
+        if (gen_on + batt_on + shunt_on) > 0:
+            return BusMode.PV
+
+        else:
+            # Nothing special; set it as PQ
+            return BusMode.PQ
+
+    def determine_bus_type_prof(self):
+        """
+        Array of bus types according to the profile
+        :return: array of bus type numbers
+        """
+        if self.active_prof is not None:
+            nt = self.active_prof.shape[0]
+            values = np.zeros(nt, dtype=int)
+            for t in range(nt):
+                values[t] = self.determine_bus_type_at(t).value
+            return values
+        else:
+            raise Exception('Asked the profile types with no profile!')
 
     def get_reactive_power_limits(self):
         """
@@ -308,13 +402,30 @@ class Bus(EditableDevice):
 
         return Qmin, Qmax
 
-    def initialize_lp_profiles(self):
+    def get_voltage_guess(self, logger=None, use_stored_guess=False):
         """
-        Dimension the LP var profiles
-        :return: Nothing
+        Determine the voltage initial guess
+        :param logger: Logger object
+        :param use_stored_guess: use the stored guess or get one from the devices
+        :return: voltage guess
         """
-        for elm in (self.controlled_generators + self.batteries):
-            elm.initialize_lp_vars()
+        if use_stored_guess:
+            return self.Vm0 * np.exp(1j * self.Va0)
+        else:
+            vm = 1.0
+            va = 0.0
+            v = complex(1, 0)
+
+            for lst in [self.controlled_generators, self.batteries]:
+                for elm in lst:
+                    if vm == 1.0:
+                        v = complex(elm.Vset, 0)
+                        vm = elm.Vset
+                    elif elm.Vset != vm:
+                        if logger is not None:
+                            logger.append('Different set points at ' + self.name + ': ' + str(elm.Vset) + ' !=' + str(v))
+
+            return v
 
     def plot_profiles(self, time_profile, ax_load=None, ax_voltage=None, time_series_driver=None, my_index=0):
         """
@@ -430,6 +541,10 @@ class Bus(EditableDevice):
         for g in self.static_generators:
             bus.static_generators.append(g.copy())
 
+        # List of static generators attached tot this bus
+        for g in self.external_grids:
+            bus.external_grids.append(g.copy())
+
         # Bus type
         bus.type = self.type
 
@@ -460,40 +575,40 @@ class Bus(EditableDevice):
 
         return bus
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Return Json-like dictionary
         :return: Dictionary
         """
+        if version in [2, 3]:
+            return {'id': self.idtag,
+                    'type': self.determine_bus_type().value,
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'active': self.active,
+                    'is_slack': self.is_slack,
+                    'vnom': self.Vnom,
+                    'vmin': self.Vmin,
+                    'vmax': self.Vmax,
+                    'rf': self.r_fault,
+                    'xf': self.x_fault,
+                    'x': self.x,
+                    'y': self.y,
+                    'h': self.h,
+                    'w': self.w,
+                    'lat': self.latitude,
+                    'lon': self.longitude,
+                    'alt': 0.0,
+                    'country': self.country.idtag if self.country is not None else "",
+                    'area': self.area.idtag if self.area is not None else "",
+                    'zone': self.zone.idtag if self.zone is not None else "",
+                    'substation': self.substation.idtag if self.substation is not None else ""
+                    }
+        else:
+            return dict()
 
-        d = {'id': self.idtag,
-             'type': self.determine_bus_type().value,
-             'phases': 'ps',
-             'name': self.name,
-             'name_code': self.code,
-             'active': self.active,
-             'is_slack': self.is_slack,
-             'vnom': self.Vnom,
-             'vmin': self.Vmin,
-             'vmax': self.Vmax,
-             'rf': self.r_fault,
-             'xf': self.x_fault,
-             'x': self.x,
-             'y': self.y,
-             'h': self.h,
-             'w': self.w,
-             'lat': self.latitude,
-             'lon': self.longitude,
-             'alt': 0.0,
-             'country': self.country.idtag,
-             'area': self.area.idtag,
-             'zone': self.zone.idtag,
-             'substation': self.substation.idtag
-             }
-
-        return d
-
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
         """
 
         :return:
@@ -506,7 +621,7 @@ class Bus(EditableDevice):
         return {'id': self.idtag,
                 'active': active_profile}
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         """
         Get units of the values
         """
@@ -529,29 +644,26 @@ class Bus(EditableDevice):
         :param t: index of the profile
         :return: Nothing
         """
+
+        self.set_profile_values(t)
+
         for elm in self.loads:
-            elm.P = elm.P_prof[t]
-            elm.Q = elm.Q_prof[t]
-            elm.Ir = elm.Ir_prof[t]
-            elm.Ii = elm.Ii_prof[t]
-            elm.G = elm.G_prof[t]
-            elm.B = elm.B_prof[t]
+            elm.set_profile_values(t)
 
         for elm in self.static_generators:
-            elm.P = elm.P_prof[t]
-            elm.Q = elm.Q_prof[t]
+            elm.set_profile_values(t)
+
+        for elm in self.external_grids:
+            elm.set_profile_values(t)
 
         for elm in self.batteries:
-            elm.P = elm.P_prof[t]
-            elm.Vset = elm.Vset_prof[t]
+            elm.set_profile_values(t)
 
         for elm in self.controlled_generators:
-            elm.P = elm.P_prof[t]
-            elm.Vset = elm.Vset_prof[t]
+            elm.set_profile_values(t)
 
         for elm in self.shunts:
-            elm.G = elm.G_prof[t]
-            elm.B = elm.B_prof[t]
+            elm.set_profile_values(t)
 
     def retrieve_graphic_position(self):
         """
@@ -559,8 +671,8 @@ class Bus(EditableDevice):
         :return: Nothing
         """
         if self.graphic_obj is not None:
-            self.x = self.graphic_obj.pos().x()
-            self.y = self.graphic_obj.pos().y()
+            self.x = int(self.graphic_obj.pos().x())
+            self.y = int(self.graphic_obj.pos().y())
             self.w, self.h = self.graphic_obj.rect().getCoords()[2:4]
 
     def delete_profiles(self):
@@ -573,6 +685,9 @@ class Bus(EditableDevice):
         for elm in self.static_generators:
             elm.delete_profiles()
 
+        for elm in self.external_grids:
+            elm.delete_profiles()
+
         for elm in self.batteries:
             elm.delete_profiles()
 
@@ -596,6 +711,9 @@ class Bus(EditableDevice):
         for elm in self.static_generators:
             elm.create_profiles(index)
 
+        for elm in self.external_grids:
+            elm.create_profiles(index)
+
         for elm in self.batteries:
             elm.create_profiles(index)
 
@@ -610,6 +728,7 @@ class Bus(EditableDevice):
         Set the default values from the profiles at time index t
         :param t: profile time index
         """
+        super().set_profile_values(t)
 
         for elm in self.loads:
             elm.set_profile_values(t)
@@ -617,6 +736,9 @@ class Bus(EditableDevice):
         for elm in self.static_generators:
             elm.set_profile_values(t)
 
+        for elm in self.external_grids:
+            elm.set_profile_values(t)
+
         for elm in self.batteries:
             elm.set_profile_values(t)
 
@@ -626,14 +748,7 @@ class Bus(EditableDevice):
         for elm in self.shunts:
             elm.set_profile_values(t)
 
-    def apply_lp_profiles(self, Sbase):
-        """
-        Sets the lp solution to the regular generators profile
-        """
-        for elm in self.batteries + self.controlled_generators:
-            elm.apply_lp_profile(Sbase)
-
-    def merge(self, other_bus):
+    def merge(self, other_bus: "Bus"):
         """
         Add the elements of the "Other bus" to this bus
         :param other_bus: Another instance of Bus
@@ -644,6 +759,8 @@ class Bus(EditableDevice):
         # List of Controlled generators attached to this bus
         self.controlled_generators += other_bus.controlled_generators.copy()
 
+        self.external_grids += other_bus.external_grids.copy()
+
         # List of shunt s attached to this bus
         self.shunts += other_bus.shunts.copy()
 
@@ -674,16 +791,26 @@ class Bus(EditableDevice):
         Return a list of all the connected objects
         :return: list of connected objects
         """
-        return self.loads + self.controlled_generators + self.batteries + self.static_generators + self.shunts
+        return self.loads + \
+                self.controlled_generators + \
+                self.batteries + \
+                self.static_generators + \
+                self.shunts + \
+                self.external_grids
 
     def get_device_number(self):
         """
         Return a list of all the connected objects
         :return: list of connected objects
         """
-        return len(self.loads) + len(self.controlled_generators) + len(self.batteries) + len(self.static_generators) + len(self.shunts)
+        return len(self.loads) + \
+               len(self.controlled_generators) + \
+               len(self.batteries) + \
+               len(self.static_generators) + \
+               len(self.shunts) + \
+               len(self.external_grids)
 
-    def ensure_area_objects(self, circuit):
+    def ensure_area_objects(self, circuit: "MultiCircuit"):
         """
         Ensure that every grouping parameter has an object
         :param circuit: MultiCircuit instance
@@ -700,4 +827,86 @@ class Bus(EditableDevice):
         if self.country is None:
             self.country = circuit.countries[0]
 
+    @staticmethod
+    def get_fused_device_lst(elm_list, property_names: list):
+        """
+        Fuse all the devices of a list by adding their selected properties
+        :param elm_list: list of devices
+        :param property_names: properties to fuse
+        :return: list of one element
+        """
+        if len(elm_list) > 1:
+            # more than a single element, fuse the list
 
+            elm1 = elm_list[0]  # select the main generator
+            act_final = elm1.active
+            act_prof_final = elm1.active_prof
+
+            # set the final active value
+            for i in range(1, len(elm_list)):  # for each of the other generators
+                elm2 = elm_list[i]
+
+                # modify the final status
+                act_final = bool(act_final + elm2.active)  # equivalent to OR
+
+                if act_prof_final is not None:
+                    act_prof_final = (act_prof_final + elm2.active_prof).astype(bool)
+
+            for prop in property_names:  # sum the properties
+
+                # initialize the value with whatever it is inside elm1
+                if 'prof' not in prop:
+                    # is a regular property
+                    val = getattr(elm1, prop) * elm1.active
+                else:
+                    if act_prof_final is not None:
+                        # it is a profile property
+                        val = getattr(elm1, prop) * elm1.active_prof
+                    else:
+                        val = None
+
+                for i in range(1, len(elm_list)):  # for each of the other generators
+                    elm2 = elm_list[i]
+
+                    if 'prof' not in prop:
+                        # is a regular property
+                        val += getattr(elm2, prop) * elm2.active
+                    else:
+                        if act_prof_final is not None:
+                            # it is a profile property
+                            val += getattr(elm2, prop) * elm2.active_prof
+
+                # set the final property value
+                if 'prof' not in prop:
+                    setattr(elm1, prop, val)
+                else:
+                    setattr(elm1, prop, val)
+
+            # set the final active status
+            elm1.active = act_final
+            elm1.active_prof = act_prof_final
+
+            return [elm1]
+
+        elif len(elm_list) == 1:
+            # single element list, return it as it comes
+            return elm_list
+
+        else:
+            # the list is empty
+            return list()
+
+    def fuse_devices(self):
+        """
+        Fuse the devices into one device per type
+        """
+        self.controlled_generators = self.get_fused_device_lst(self.controlled_generators,
+                                                               ['P', 'Pmin', 'Pmax', 'Qmin', 'Qmax', 'Snom', 'P_prof'])
+        self.batteries = self.get_fused_device_lst(self.batteries,
+                                                   ['P', 'Pmin', 'Pmax', 'Qmin', 'Qmax', 'Snom', 'Enom', 'P_prof'])
+
+        self.loads = self.get_fused_device_lst(self.loads, ['P', 'Q', 'Ir', 'Ii', 'G', 'B', 'P_prof', 'Q_prof'])
+        self.static_generators = self.get_fused_device_lst(self.static_generators, ['P', 'Q', 'P_prof', 'Q_prof'])
+
+        self.shunts = self.get_fused_device_lst(self.shunts, ['G', 'B', 'G_prof', 'B_prof'])
+        self.external_grids = self.get_fused_device_lst(self.external_grids, [])
