@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import uuid
 import numpy as np
 from typing import List, Dict, AnyStr, Any, Optional
@@ -43,7 +45,7 @@ class GCProp:
 
 class EditableDevice:
 
-    def __init__(self, name, active, device_type: DeviceType,
+    def __init__(self, name, active: bool, device_type: DeviceType,
                  editable_headers: Dict[str, GCProp],
                  non_editable_attributes: List[str],
                  properties_with_profile: Dict[str, Optional[Any]],
@@ -65,7 +67,7 @@ class EditableDevice:
         else:
             self.idtag = idtag.replace('_', '').replace('-', '')
 
-        self.name = name
+        self._name = name
 
         self.code = code
 
@@ -84,6 +86,9 @@ class EditableDevice:
 
         self.properties_with_profile = properties_with_profile
 
+    def generate_uuid(self):
+        self.idtag = uuid.uuid4().hex
+
     def __str__(self) -> AnyStr:
         """
         Name of the object
@@ -102,7 +107,18 @@ class EditableDevice:
         return self.__hash__() < other.__hash__()
 
     def __eq__(self, other):
-        return self.idtag == other.idtag
+        if hasattr(other, 'idtag'):
+            return self.idtag == other.idtag
+        else:
+            return False
+
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, val: str):
+        self._name = val
 
     def get_save_data(self):
         """
@@ -227,11 +243,11 @@ class EditableDevice:
             profile = getattr(self, self.properties_with_profile[magnitude])
             setattr(self, magnitude, profile[t])
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         return dict()
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         return dict()
 
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
         return dict()
