@@ -1,21 +1,24 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
+from GridCal.Engine.basic_structures import Logger
 from GridCal.Engine.Devices.editable_device import EditableDevice, GCProp
 from GridCal.Engine.Devices.enumerations import DeviceType, GeneratorTechnologyType
 
@@ -447,6 +450,28 @@ class Generator(EditableDevice):
             if show_fig:
                 plt.show()
 
+    def fix_inconsistencies(self, logger: Logger, min_vset=0.98, max_vset=1.02):
+        """
+        Correct the voltage set points
+        :param logger: logger to store the events
+        :param min_vset: minimum voltage set point (p.u.)
+        :param max_vset: maximum voltage set point (p.u.)
+        :return: True if any correction happened
+        """
+        errors = False
+
+        if self.Vset > max_vset:
+            logger.add_warning("Corrected generator set point", self.name, self.Vset, max_vset)
+            self.Vset = max_vset
+            errors = True
+
+        elif self.Vset < min_vset:
+            logger.add_warning("Corrected generator set point", self.name, self.Vset, min_vset)
+            self.Vset = min_vset
+            errors = True
+
+        return errors
+
     @property
     def Qmax(self):
         """
