@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 
 import pandas as pd
@@ -47,6 +49,79 @@ def firing_angles_to_reactive_limits(P, alphamin, alphamax):
     return Qmin, Qmax
 
 
+def getFromAndToPowerAt(Pset, theta_f, theta_t, Vnf, Vnt, v_set_f, v_set_t, Sbase, r1, angle_droop, rate,
+                        free: bool, in_pu: bool = False):
+    """
+    Compute the power and losses
+    :param Pset: set power in MW
+    :param theta_f: angle from (rad)
+    :param theta_t: angle to (rad)
+    :param Vnf: nominal voltage from (kV)
+    :param Vnt: nominal voltage to (kV)
+    :param v_set_f: control voltage from (p.u.)
+    :param v_set_t: control voltage to (p.u.)
+    :param Sbase: base power MVA
+    :param r1: line resistance (ohm)
+    :param angle_droop: angle droop control (MW/deg)
+    :param free: is free to use the angle droop?
+    :param in_pu: return power in per unit? otherwise the power comes in MW
+    :return: Pf, Pt, losses (in MW or p.u. depending on `in_pu`)
+    """
+
+    if not free:
+
+        # simply copy the set power value
+        Pcalc = Pset
+
+    elif free:
+
+        # compute the angular difference in degrees (0.017453292f -> pi/180)
+        # theta_f and theta_t are in rad
+        # for the control not to be oscillatory, the angle difference must be the opposite (to - from)
+        dtheta = np.rad2deg(theta_t - theta_f)
+
+        # compute the desired control power flow
+        Pcalc = Pset + angle_droop * dtheta  # this is in MW
+
+        # rate truncation
+        if Pcalc > rate:
+            Pcalc = rate
+
+        elif Pcalc < -rate:
+            Pcalc = -rate
+
+    else:
+        Pcalc = 0
+
+    # depending on the value of Pcalc, assign the from and to values
+    if Pcalc > 0:
+        # from ->  to
+        I = Pcalc / (Vnf * v_set_f)  # current in kA
+        loss = r1 * I * I  # losses in MW
+        Pf = - Pcalc
+        Pt = Pcalc - loss
+
+    elif Pcalc < 0:
+        # to -> from
+        I = Pcalc / (Vnt * v_set_t)  # current in kA
+        loss = r1 * I * I  # losses in MW
+        Pf = - Pcalc - loss
+        Pt = Pcalc  # is negative
+
+    else:
+        Pf = 0
+        Pt = 0
+        loss = 0
+
+    # convert to p.u.
+    if in_pu:
+        Pf /= Sbase
+        Pt /= Sbase
+        loss /= Sbase
+
+    return Pf, Pt, loss
+
+
 class HvdcLine(EditableDevice):
     """
     The **Line** class represents the connections between nodes (i.e.
@@ -155,12 +230,13 @@ class HvdcLine(EditableDevice):
         **template** (BranchTemplate, BranchTemplate()): Basic branch template
     """
 
-    def __init__(self, bus_from: Bus = None, bus_to: Bus = None, name='HVDC Line', idtag=None, active=True,
+    def __init__(self, bus_from: Bus = None, bus_to: Bus = None, name='HVDC Line', idtag=None, active=True, code='',
                  rate=1.0, Pset=0.0, r=1e-20, loss_factor=0.0, Vset_f=1.0, Vset_t=1.0, length=1.0, mttf=0.0, mttr=0.0,
                  overload_cost=1000.0,   min_firing_angle_f=-1.0, max_firing_angle_f=1.0, min_firing_angle_t=-1.0,
-                 max_firing_angle_t=1.0, active_prof=np.ones(0, dtype=bool), rate_prof=np.zeros(0),
+                 max_firing_angle_t=1.0,  active_prof=np.ones(0, dtype=bool), rate_prof=np.zeros(0),
                  Pset_prof=np.zeros(0), Vset_f_prof=np.ones(0), Vset_t_prof=np.ones(0), overload_cost_prof=np.zeros(0),
-                 contingency_factor=1.0, control_mode: HvdcControlType=HvdcControlType.type_1_Pset):
+                 contingency_factor=1.0, control_mode: HvdcControlType=HvdcControlType.type_1_Pset,
+                 dispatchable=False, angle_droop=0, angle_droop_prof=np.ones(0), contingency_factor_prof=None):
         """
         HVDC Line model
         :param bus_from: Bus from
@@ -192,6 +268,7 @@ class HvdcLine(EditableDevice):
         EditableDevice.__init__(self,
                                 name=name,
                                 idtag=idtag,
+                                code=code,
                                 active=active,
                                 device_type=DeviceType.HVDCLineDevice,
                                 editable_headers={'name': GCProp('', str, 'Name of the line.'),
@@ -202,6 +279,8 @@ class HvdcLine(EditableDevice):
                                                                    'Name of the bus at the "to" side of the line.'),
                                                   'active': GCProp('', bool, 'Is the line active?'),
 
+                                                  'dispatchable': GCProp('', bool, 'Is the line power optimizable?'),
+
                                                   'rate': GCProp('MVA', float, 'Thermal rating power of the line.'),
 
                                                   'contingency_factor': GCProp('p.u.', float,
@@ -213,9 +292,7 @@ class HvdcLine(EditableDevice):
 
                                                   'r': GCProp('Ohm', float, 'line resistance.'),
 
-                                                  'loss_factor': GCProp('p.u.', float,
-                                                                        'Losses factor.\n'
-                                                                        'The losses are computed as losses=Pfset x Ploss'),
+                                                  'angle_droop': GCProp('MW/deg', float, 'Power/angle rate control'),
 
                                                   'Vset_f': GCProp('p.u.', float, 'Set voltage at the from side'),
                                                   'Vset_t': GCProp('p.u.', float, 'Set voltage at the to side'),
@@ -247,9 +324,11 @@ class HvdcLine(EditableDevice):
                                 non_editable_attributes=['bus_from', 'bus_to', 'idtag'],
                                 properties_with_profile={'active': 'active_prof',
                                                          'rate': 'rate_prof',
+                                                         'contingency_factor': 'contingency_factor_prof',
                                                          'Pset': 'Pset_prof',
                                                          'Vset_f': 'Vset_f_prof',
                                                          'Vset_t': 'Vset_t_prof',
+                                                         'angle_droop': 'angle_droop_prof',
                                                          'overload_cost': 'overload_cost_prof'})
 
         # connectivity
@@ -262,10 +341,14 @@ class HvdcLine(EditableDevice):
         # line length in km
         self.length = length
 
+        self.dispatchable = dispatchable
+
         self.Pset = Pset
 
         self.r = r
 
+        self.angle_droop = angle_droop
+
         self.loss_factor = loss_factor
 
         self.mttf = mttf
@@ -300,33 +383,89 @@ class HvdcLine(EditableDevice):
         self.Vset_f_prof = Vset_f_prof
         self.Vset_t_prof = Vset_t_prof
 
+        self.angle_droop_prof = angle_droop_prof
+
         # branch rating in MVA
         self.rate = rate
         self.contingency_factor = contingency_factor
         self.rate_prof = rate_prof
+        self.contingency_factor_prof = contingency_factor_prof
 
-    def get_from_and_to_power(self):
+    def get_from_and_to_power(self, theta_f, theta_t, Sbase, in_pu=False):
         """
         Get the power set at both ends accounting for meaningful losses
         :return: power from, power to
         """
-        A = int(self.Pset > 0)
-        B = 1 - A
-        Pf = - self.Pset * A + self.Pset * (1 - self.loss_factor) * B
-        Pt = self.Pset * A * (1 - self.loss_factor) - self.Pset * B
+        if self.active:
+            Pf, Pt, losses = getFromAndToPowerAt(Pset=self.Pset,
+                                                 theta_f=theta_f,
+                                                 theta_t=theta_t,
+                                                 Vnf=self.bus_from.Vnom,
+                                                 Vnt=self.bus_to.Vnom,
+                                                 v_set_f=self.Vset_f,
+                                                 v_set_t=self.Vset_t,
+                                                 Sbase=Sbase,
+                                                 r1=self.r,
+                                                 angle_droop=self.angle_droop,
+                                                 rate=self.rate,
+                                                 free=self.control_mode == HvdcControlType.type_0_free,
+                                                 in_pu=in_pu)
 
-        return Pf, Pt
+            return Pf, Pt, losses
+        else:
+            return 0, 0, 0
 
-    def get_from_and_to_power_profiles(self):
+    def get_from_and_to_power_at(self, t, theta_f, theta_t, Sbase, in_pu=False):
         """
         Get the power set at both ends accounting for meaningful losses
         :return: power from, power to
         """
-        A = (self.Pset_prof > 0).astype(int)
-        B = 1 - A
+        if self.active_prof[t]:
+            Pf, Pt, losses = getFromAndToPowerAt(Pset=self.Pset_prof[t],
+                                                 theta_f=theta_f,
+                                                 theta_t=theta_t,
+                                                 Vnf=self.bus_from.Vnom,
+                                                 Vnt=self.bus_to.Vnom,
+                                                 v_set_f=self.Vset_f_prof[t],
+                                                 v_set_t=self.Vset_t_prof[t],
+                                                 Sbase=Sbase,
+                                                 r1=self.r,
+                                                 angle_droop=self.angle_droop,
+                                                 rate=self.rate_prof[t],
+                                                 free=self.control_mode == HvdcControlType.type_0_free,
+                                                 in_pu=in_pu)
 
-        Pf = - self.Pset_prof * A + self.Pset_prof * (1 - self.loss_factor) * B
-        Pt = self.Pset_prof * A * (1 - self.loss_factor) - self.Pset_prof * B
+            return Pf, Pt, losses
+        else:
+            return 0, 0, 0
+
+    def get_from_and_to_power_profiles(self, theta_f, theta_t, Sbase):
+        """
+        Get the power set at both ends accounting for meaningful losses
+        :return: power from, power to
+        """
+        # A = (self.Pset_prof > 0).astype(int)
+        # B = 1 - A
+        #
+        # Pf = - self.Pset_prof * A + self.Pset_prof * (1 - self.loss_factor) * B
+        # Pt = self.Pset_prof * A * (1 - self.loss_factor) - self.Pset_prof * B
+
+        Pf = np.zeros_like(self.Pset_prof)
+        Pt = np.zeros_like(self.Pset_prof)
+        losses = np.zeros_like(self.Pset_prof)
+        for t in range(len(self.Pset_prof)):
+            Pf[t], Pt[t], losses[t] = getFromAndToPowerAt(Pset=self.Pset_prof[t],
+                                                          theta_f=theta_f[t],
+                                                          theta_t=theta_t[t],
+                                                          Vnf=self.bus_from.Vnom,
+                                                          Vnt=self.bus_to.Vnom,
+                                                          v_set_f=self.Vset_f_prof[t],
+                                                          v_set_t=self.Vset_t_prof[t],
+                                                          Sbase=Sbase,
+                                                          r1=self.r,
+                                                          angle_droop=self.angle_droop,
+                                                          rate=self.rate_prof[t],
+                                                          free=self.control_mode == HvdcControlType.type_0_free)
 
         return Pf, Pt
 
@@ -423,62 +562,68 @@ class HvdcLine(EditableDevice):
         :return:
         """
         if version == 2:
-            return {'id': self.idtag,
-                    'type': 'hvdc',
-                    'phases': 'ps',
-                    'name': self.name,
-                    'name_code': self.code,
-                    'bus_from': self.bus_from.idtag,
-                    'bus_to': self.bus_to.idtag,
-                    'active': self.active,
-                    'rate': self.rate,
-                    'r': 0,
-                    'length': self.length,
-                    'loss_factor': self.loss_factor,
-                    'vset_from': self.Vset_f,
-                    'vset_to': self.Vset_t,
-                    'Pset': self.Pset,
-                    'min_firing_angle_f': self.min_firing_angle_f,
-                    'max_firing_angle_f': self.max_firing_angle_f,
-                    'min_firing_angle_t': self.min_firing_angle_t,
-                    'max_firing_angle_t': self.max_firing_angle_t,
-                    'overload_cost': self.overload_cost,
-                    'base_temperature': 20,
-                    'operational_temperature': 20,
-                    'alpha': 0.00330,
-                    'locations': []
-                    }
+            d = {'id': self.idtag,
+                 'type': 'hvdc',
+                 'phases': 'ps',
+                 'name': self.name,
+                 'name_code': self.code,
+                 'bus_from': self.bus_from.idtag,
+                 'bus_to': self.bus_to.idtag,
+                 'active': self.active,
+                 'rate': self.rate,
+                 'control_mode': self.control_mode.value,
+                 'r': self.r,
+                 'length': self.length,
+                 'loss_factor': self.loss_factor,
+                 'angle_droop': self.angle_droop,
+                 'vset_from': self.Vset_f,
+                 'vset_to': self.Vset_t,
+                 'Pset': self.Pset,
+                 'min_firing_angle_f': self.min_firing_angle_f,
+                 'max_firing_angle_f': self.max_firing_angle_f,
+                 'min_firing_angle_t': self.min_firing_angle_t,
+                 'max_firing_angle_t': self.max_firing_angle_t,
+                 'overload_cost': self.overload_cost,
+                 'base_temperature': 20,
+                 'operational_temperature': 20,
+                 'alpha': 0.00330,
+                 'locations': []
+                 }
         elif version == 3:
-            return {'id': self.idtag,
-                    'type': 'hvdc',
-                    'phases': 'ps',
-                    'name': self.name,
-                    'name_code': self.code,
-                    'bus_from': self.bus_from.idtag,
-                    'bus_to': self.bus_to.idtag,
-                    'active': self.active,
-                    'rate': self.rate,
-                    'contingency_factor1': self.contingency_factor,
-                    'contingency_factor2': self.contingency_factor,
-                    'contingency_factor3': self.contingency_factor,
-                    'r': 0,
-                    'length': self.length,
-                    'loss_factor': self.loss_factor,
-                    'vset_from': self.Vset_f,
-                    'vset_to': self.Vset_t,
-                    'Pset': self.Pset,
-                    'min_firing_angle_f': self.min_firing_angle_f,
-                    'max_firing_angle_f': self.max_firing_angle_f,
-                    'min_firing_angle_t': self.min_firing_angle_t,
-                    'max_firing_angle_t': self.max_firing_angle_t,
-                    'overload_cost': self.overload_cost,
-                    'base_temperature': 20,
-                    'operational_temperature': 20,
-                    'alpha': 0.00330,
-                    'locations': []
-                    }
+            d = {'id': self.idtag,
+                 'type': 'hvdc',
+                 'phases': 'ps',
+                 'name': self.name,
+                 'name_code': self.code,
+                 'bus_from': self.bus_from.idtag,
+                 'bus_to': self.bus_to.idtag,
+                 'active': self.active,
+                 'rate': self.rate,
+                 'control_mode': self.control_mode.value,
+                 'contingency_factor1': self.contingency_factor,
+                 'contingency_factor2': self.contingency_factor,
+                 'contingency_factor3': self.contingency_factor,
+                 'r': self.r,
+                 'length': self.length,
+                 'loss_factor': self.loss_factor,
+                 'angle_droop': self.angle_droop,
+                 'vset_from': self.Vset_f,
+                 'vset_to': self.Vset_t,
+                 'Pset': self.Pset,
+                 'min_firing_angle_f': self.min_firing_angle_f,
+                 'max_firing_angle_f': self.max_firing_angle_f,
+                 'min_firing_angle_t': self.min_firing_angle_t,
+                 'max_firing_angle_t': self.max_firing_angle_t,
+                 'overload_cost': self.overload_cost,
+                 'base_temperature': 20,
+                 'operational_temperature': 20,
+                 'alpha': 0.00330,
+                 'locations': []
+                 }
         else:
-            return dict()
+            d = dict()
+
+        return d
 
     def get_profiles_dict(self, version=3):
         """
@@ -490,8 +635,8 @@ class HvdcLine(EditableDevice):
             active_prof = self.active_prof.tolist()
             rate_prof = self.rate_prof.tolist()
             pset_prof = self.Pset_prof.tolist()
-            vset_prof_f = self.Vset_f_prof
-            vset_prof_t = self.Vset_t_prof
+            vset_prof_f = self.Vset_f_prof.tolist()
+            vset_prof_t = self.Vset_t_prof.tolist()
             cost_prof = self.overload_cost_prof.tolist()
         else:
             active_prof = list()
