@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import pandas as pd
 from matplotlib import pyplot as plt
 from GridCal.Engine.Devices.editable_device import EditableDevice, DeviceType, GCProp
@@ -175,7 +177,7 @@ class Load(EditableDevice):
                     'name': self.name,
                     'name_code': self.code,
                     'bus': self.bus.idtag,
-                    'active': self.active,
+                    'active': bool(self.active),
                     'g': self.G,
                     'b': self.B,
                     'ir': self.Ir,
@@ -200,6 +202,7 @@ class Load(EditableDevice):
             Ii_prof = self.Ii_prof.tolist()
             G_prof = self.G_prof.tolist()
             B_prof = self.B_prof.tolist()
+
         else:
             active_profile = list()
             P_prof = list()
