@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import os
 from numpy import sqrt
@@ -424,8 +426,10 @@ class Transformer2W(EditableDevice):
                  temp_base=20, temp_oper=20, alpha=0.00330,
                  control_mode: TransformerControlType = TransformerControlType.fixed,
                  template: TransformerType = None,
-                 rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None, contingency_factor=1.0,
-                 contingency_enabled=True, monitor_loading=True):
+                 rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None,
+                 tap_module_prof=None, angle_prof=None,
+                 contingency_factor=1.0,
+                 contingency_enabled=True, monitor_loading=True, contingency_factor_prof=None):
 
         EditableDevice.__init__(self,
                                 name=name,
@@ -498,6 +502,9 @@ class Transformer2W(EditableDevice):
                                 non_editable_attributes=['bus_from', 'bus_to', 'template'],
                                 properties_with_profile={'active': 'active_prof',
                                                          'rate': 'rate_prof',
+                                                         'contingency_factor': 'contingency_factor_prof',
+                                                         'tap_module': 'tap_module_prof',
+                                                         'angle': 'angle_prof',
                                                          'temp_oper': 'temp_oper_prof',
                                                          'Cost': 'Cost_prof'})
 
@@ -505,22 +512,10 @@ class Transformer2W(EditableDevice):
         self.bus_from = bus_from
         self.bus_to = bus_to
 
-        if bus_from is not None:
-            vh = max(bus_from.Vnom, bus_to.Vnom)
-            vl = min(bus_from.Vnom, bus_to.Vnom)
-        else:
-            vh = 1.0
-            vl = 1.0
-
-        if HV is None:
-            self.HV = vh
-        else:
-            self.HV = HV
-
-        if LV is None:
-            self.LV = vl
-        else:
-            self.LV = LV
+        # set the high and low voltage values
+        self.HV = 0
+        self.LV = 0
+        self.set_hv_and_lv(HV, LV)
 
         # List of measurements
         self.measurements = list()
@@ -563,8 +558,11 @@ class Transformer2W(EditableDevice):
         else:
             self.tap_module = self.tap_changer.get_tap()
 
+        self.tap_module_prof = tap_module_prof
+
         # Tap angle
         self.angle = shift_angle
+        self.angle_prof = angle_prof
 
         self.tap_module_max = tap_module_max
         self.tap_module_min = tap_module_min
@@ -577,6 +575,7 @@ class Transformer2W(EditableDevice):
         self.contingency_enabled: bool = contingency_enabled
         self.monitor_loading: bool = monitor_loading
         self.rate_prof = rate_prof
+        self.contingency_factor_prof = contingency_factor_prof
 
         # branch type: Line, Transformer, etc...
         self.branch_type = BranchType.Transformer
@@ -604,6 +603,29 @@ class Transformer2W(EditableDevice):
 
         self.inv_conv = {val: key for key, val in self.conv.items()}
 
+    def set_hv_and_lv(self, HV, LV):
+        """
+        set the high and low voltage values
+        :param HV: higher voltage value (kV)
+        :param LV: lower voltage value (kV)
+        """
+        if self.bus_from is not None:
+            vh = max(self.bus_from.Vnom, self.bus_to.Vnom)
+            vl = min(self.bus_from.Vnom, self.bus_to.Vnom)
+        else:
+            vh = 1.0
+            vl = 1.0
+
+        if HV is None:
+            self.HV = vh
+        else:
+            self.HV = HV
+
+        if LV is None:
+            self.LV = vl
+        else:
+            self.LV = LV
+
     @property
     def R_corrected(self):
         """
@@ -614,6 +636,15 @@ class Transformer2W(EditableDevice):
         """
         return self.R * (1 + self.alpha * (self.temp_oper - self.temp_base))
 
+    def change_base(self, Sbase_old, Sbase_new):
+
+        b = Sbase_new / Sbase_old
+
+        self.R *= b
+        self.X *= b
+        self.G *= b
+        self.B *= b
+
     def get_weight(self):
         return np.sqrt(self.R * self.R + self.X * self.X)
 
@@ -862,6 +893,8 @@ class Transformer2W(EditableDevice):
                  'rate': self.rate,
                  'Vnomf': v_from,
                  'Vnomt': v_to,
+                 'hv': self.HV,
+                 'lv': self.LV,
                  'r': self.R,
                  'x': self.X,
                  'g': self.G,
@@ -910,6 +943,8 @@ class Transformer2W(EditableDevice):
 
                  'Vnomf': v_from,
                  'Vnomt': v_to,
+                 'hv': self.HV,
+                 'lv': self.LV,
                  'r': self.R,
                  'x': self.X,
                  'g': self.G,
@@ -1016,3 +1051,42 @@ class Transformer2W(EditableDevice):
         Get the branch defining coordinates
         """
         return [self.bus_from.get_coordinates(), self.bus_to.get_coordinates()]
+
+    def delete_virtual_taps(self):
+        """
+        Set the HV and LV parameters such that any virtual tap is null
+        """
+        self.HV = max(self.bus_from.Vnom, self.bus_to.Vnom)
+        self.LV = min(self.bus_from.Vnom, self.bus_to.Vnom)
+
+    def fix_inconsistencies(self, logger: Logger, maximum_difference=0.1):
+        """
+        Fix the voltage inconsistencies
+        :param logger:
+        :param maximum_difference: proportion to be under or above (i.e. Transformer HV=41.9, bus HV=45 41.9/45 = 0.93 -> 0.9 <= 0.93 <= 1.1, so its ok
+        :return:
+        """
+        errors = False
+        HV = max(self.bus_from.Vnom, self.bus_to.Vnom)
+        LV = min(self.bus_from.Vnom, self.bus_to.Vnom)
+
+        if self.LV > self.HV:
+            logger.add_warning("HV > LV", self.name, self.HV, HV)
+            self.HV, self.LV = self.LV, self.HV
+            errors = True
+
+        rHV = self.HV / HV
+        rLV = self.LV / LV
+        LB = 1 - maximum_difference
+        UB = 1 + maximum_difference
+        if not (LB <= rHV <= UB):
+            logger.add_warning("Corrected transformer HV", self.name, self.HV, HV)
+            self.HV = HV
+            errors = True
+
+        if not (LB <= rLV <= UB):
+            logger.add_warning("Corrected transformer LV", self.name, self.LV, LV)
+            self.LV = LV
+            errors = True
+
+        return errors
