@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import pandas as pd
 import numpy as np
@@ -35,7 +37,7 @@ class VSC(EditableDevice):
                  Pfset = 0.0, Qfset=0.0, Vac_set=1.0, Vdc_set=1.0,
                  alpha1=0.0001, alpha2=0.015, alpha3=0.2,
                  mttf=0, mttr=0, cost=100, cost_prof=None, rate_prof=None, active_prof=None, contingency_factor=1.0,
-                 contingency_enabled=True, monitor_loading=True):
+                 contingency_enabled=True, monitor_loading=True, contingency_factor_prof=None):
         """
         Voltage source converter (VSC)
         :param bus_from:
@@ -138,6 +140,7 @@ class VSC(EditableDevice):
                                 non_editable_attributes=['bus_from', 'bus_to', 'idtag'],
                                 properties_with_profile={'active': 'active_prof',
                                                          'rate': 'rate_prof',
+                                                         'contingency_factor': 'contingency_factor_prof',
                                                          'Cost': 'Cost_prof'})
 
         # the VSC must only connect from an DC to a AC bus
@@ -206,6 +209,7 @@ class VSC(EditableDevice):
         self.contingency_enabled: bool = contingency_enabled
         self.monitor_loading: bool = monitor_loading
         self.rate_prof = rate_prof
+        self.contingency_factor_prof = contingency_factor_prof
 
         # branch type: Line, Transformer, etc...
         self.branch_type = BranchType.VSC
@@ -213,6 +217,28 @@ class VSC(EditableDevice):
     def get_weight(self):
         return np.sqrt(self.R1 * self.R1 + self.X1 * self.X1)
 
+    @property
+    def R(self):
+        return self.R1
+
+    @property
+    def X(self):
+        return self.X1
+
+    def change_base(self, Sbase_old, Sbase_new):
+        b = Sbase_new / Sbase_old
+
+        self.R1 *= b
+        self.X1 *= b
+        self.G0 *= b
+        self.Beq *= b
+
+    def get_coordinates(self):
+        """
+        Get the line defining coordinates
+        """
+        return [self.bus_from.get_coordinates(), self.bus_to.get_coordinates()]
+
     def correct_buses_connection(self):
         """
         Fix the buses connection (from: DC, To: AC)
@@ -257,7 +283,9 @@ class VSC(EditableDevice):
                  ConverterControlType.type_II_4: 4,
                  ConverterControlType.type_II_5: 5,
                  ConverterControlType.type_III_6: 6,
-                 ConverterControlType.type_III_7: 7}
+                 ConverterControlType.type_III_7: 7,
+                 ConverterControlType.type_IV_I: 8,
+                 ConverterControlType.type_IV_II: 9}
         if version == 2:
             d = {'id': self.idtag,
                  'type': 'vsc',
@@ -340,7 +368,7 @@ class VSC(EditableDevice):
                  'vac_set': self.Vac_set,
                  'vdc_set': self.Vdc_set,
 
-                 'mode': modes[self.control_mode]
+                 'control_mode': modes[self.control_mode]
                  }
         else:
             d = dict()
