@@ -7,4 +7,4 @@ from GridCal.Engine.IO.json_parser import *
 from GridCal.Engine.IO.matpower.matpower_parser import parse_matpower_file
 from GridCal.Engine.IO.raw_parser import PSSeParser
 from GridCal.Engine.IO.excel_interface import *
-from GridCal.Engine.IO.file_handler import *
+from GridCal.Engine.IO.file_handler import FileOpen, FileSave
