@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import os
 import chardet
 import pandas as pd
@@ -166,7 +168,7 @@ class CIMExport:
         text_file.write('<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:cim="http://iec.ch/TC57/2009/CIM-schema-cim14#">\n')
 
         # Model
-        model = cimdev.GeneralContainer(rfid=self.circuit.name, tpe='Model')
+        model = cimdev.GeneralContainer(rfid=self.circuit.idtag, tpe='Model')
         model.properties['name'] = self.circuit.name
         model.properties['version'] = 1
         model.properties['description'] = self.circuit.comments
@@ -217,7 +219,7 @@ class CIMExport:
 
         for substation in substation_bus.keys():
 
-            substation_id = substation + '_' + str(substation_idx)
+            substation_id = substation.idtag
             substation_idx += 1
 
             model = cimdev.GeneralContainer(rfid=substation_id, tpe='Substation', resources=['Location', 'SubGeographicalRegion'])
@@ -262,10 +264,12 @@ class CIMExport:
 
                     # generate model
                     model = cimdev.GeneralContainer(rfid=conn_node_id, tpe='ConnectivityNode',
-                                             resources=['BaseVoltage', 'VoltageLevel', 'ConnectivityNodeContainer'],
-                                             class_replacements={'name': 'IdentifiedObject',
-                                                                 'aliasName': 'IdentifiedObject'}
-                                             )
+                                                    resources=['BaseVoltage',
+                                                               'VoltageLevel',
+                                                               'ConnectivityNodeContainer'],
+                                                    class_replacements={'name': 'IdentifiedObject',
+                                                                        'aliasName': 'IdentifiedObject'}
+                                                    )
                     model.properties['name'] = bus.name
                     model.properties['aliasName'] = bus.name
                     model.properties['BaseVoltage'] = base_voltage
@@ -275,15 +279,16 @@ class CIMExport:
 
                     for il, elm in enumerate(bus.loads):
 
-                        id2 = conn_node_id + '_LOAD_' + str(il)
+                        id2 = elm.idtag + '_LOAD_' + str(il)
                         id3 = id2 + '_LRC'
 
                         model = cimdev.GeneralContainer(rfid=id2, tpe='ConformLoad',
-                                                 resources=['BaseVoltage', 'LoadResponse', 'VoltageLevel'],
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject',
-                                                                     'EquipmentContainer': 'Equipment'}
-                                                 )
+                                                        resources=['BaseVoltage',
+                                                                   'LoadResponse',
+                                                                   'VoltageLevel'],
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject',
+                                                                            'EquipmentContainer': 'Equipment'})
                         model.properties['name'] = elm.name
                         model.properties['aliasName'] = elm.name
                         model.properties['BaseVoltage'] = base_voltage
@@ -311,9 +316,9 @@ class CIMExport:
 
                         # Terminal 1 (from)
                         model = cimdev.GeneralContainer(rfid=id2 + '_T', tpe='Terminal',
-                                                 resources=terminal_resources,
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'})
+                                                        resources=terminal_resources,
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'})
                         model.properties['name'] = elm.name
                         model.properties['TopologicalNode'] = bus_id_dict[bus]
                         model.properties['ConductingEquipment'] = id2
@@ -323,14 +328,14 @@ class CIMExport:
 
                     for il, elm in enumerate(bus.static_generators):
 
-                        id2 = conn_node_id + '_StatGen_' + str(il)
+                        id2 = elm.idtag + '_StatGen_' + str(il)
 
                         model = cimdev.GeneralContainer(rfid=id2, tpe='ConformLoad',
-                                                 resources=['BaseVoltage', 'LoadResponse', 'VoltageLevel'],
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject',
-                                                                     'EquipmentContainer': 'Equipment'}
-                                                 )
+                                                        resources=['BaseVoltage', 'LoadResponse', 'VoltageLevel'],
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject',
+                                                                            'EquipmentContainer': 'Equipment'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['aliasName'] = elm.name
                         model.properties['BaseVoltage'] = base_voltage
@@ -342,10 +347,10 @@ class CIMExport:
 
                         # Terminal 1 (from)
                         model = cimdev.GeneralContainer(rfid=id2 + '_T', tpe='Terminal',
-                                                 resources=terminal_resources,
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'}
-                                                 )
+                                                        resources=terminal_resources,
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['TopologicalNode'] = bus_id_dict[bus]
                         model.properties['ConductingEquipment'] = id2
@@ -355,17 +360,17 @@ class CIMExport:
 
                     for il, elm in enumerate(bus.controlled_generators):
 
-                        id2 = conn_node_id + '_SyncGen_' + str(il)
+                        id2 = elm.idtag + '_SyncGen_' + str(il)
                         id3 = id2 + '_GU'
                         id4 = id2 + '_RC'
 
                         model = cimdev.GeneralContainer(rfid=id2, tpe='SynchronousMachine',
-                                                 resources=['BaseVoltage', 'RegulatingControl',
-                                                            'GeneratingUnit', 'VoltageLevel'],
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject',
-                                                                     'EquipmentContainer': 'Equipment'}
-                                                 )
+                                                        resources=['BaseVoltage', 'RegulatingControl',
+                                                                   'GeneratingUnit', 'VoltageLevel'],
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject',
+                                                                            'EquipmentContainer': 'Equipment'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['aliasName'] = elm.name
                         model.properties['BaseVoltage'] = base_voltage
@@ -390,10 +395,10 @@ class CIMExport:
 
                         # Terminal 1 (from)
                         model = cimdev.GeneralContainer(rfid=id2 + '_T', tpe='Terminal',
-                                                 resources=terminal_resources,
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'}
-                                                 )
+                                                        resources=terminal_resources,
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['TopologicalNode'] = bus_id_dict[bus]
                         model.properties['ConductingEquipment'] = id2
@@ -403,14 +408,14 @@ class CIMExport:
 
                     for il, elm in enumerate(bus.shunts):
 
-                        id2 = conn_node_id + '_Shunt_' + str(il)
+                        id2 = elm.idtag + '_Shunt_' + str(il)
 
                         model = cimdev.GeneralContainer(rfid=id2, tpe='ShuntCompensator',
-                                                 resources=['BaseVoltage', 'VoltageLevel'],
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject',
-                                                                     'EquipmentContainer': 'Equipment'}
-                                                 )
+                                                        resources=['BaseVoltage', 'VoltageLevel'],
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject',
+                                                                            'EquipmentContainer': 'Equipment'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['aliasName'] = elm.name
                         model.properties['BaseVoltage'] = base_voltage
@@ -424,10 +429,10 @@ class CIMExport:
 
                         # Terminal 1 (from)
                         model = cimdev.GeneralContainer(rfid=id2 + '_T', tpe='Terminal',
-                                                 resources=terminal_resources,
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'}
-                                                 )
+                                                        resources=terminal_resources,
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'}
+                                                        )
                         model.properties['name'] = elm.name
                         model.properties['TopologicalNode'] = bus_id_dict[bus]
                         model.properties['ConductingEquipment'] = id2
@@ -439,9 +444,9 @@ class CIMExport:
                         equivalent_network_id = conn_node_id + '_EqNetwork'
 
                         model = cimdev.GeneralContainer(rfid=equivalent_network_id, tpe='EquivalentNetwork',
-                                                 resources=['BaseVoltage', 'VoltageLevel'],
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'})
+                                                        resources=['BaseVoltage', 'VoltageLevel'],
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'})
                         model.properties['name'] = bus.name + '_Slack'
                         model.properties['aliasName'] = bus.name + '_Slack'
                         model.properties['BaseVoltage'] = base_voltage
@@ -450,10 +455,10 @@ class CIMExport:
 
                         # Terminal 1 (from)
                         model = cimdev.GeneralContainer(rfid=equivalent_network_id + '_T', tpe='Terminal',
-                                                 resources=terminal_resources,
-                                                 class_replacements={'name': 'IdentifiedObject',
-                                                                     'aliasName': 'IdentifiedObject'}
-                                                 )
+                                                        resources=terminal_resources,
+                                                        class_replacements={'name': 'IdentifiedObject',
+                                                                            'aliasName': 'IdentifiedObject'}
+                                                        )
                         model.properties['name'] = equivalent_network_id + '_T'
                         model.properties['TopologicalNode'] = bus_id_dict[bus]
                         model.properties['ConductingEquipment'] = equivalent_network_id
@@ -469,14 +474,12 @@ class CIMExport:
         tap_changer_resources = ['TransformerWinding']
         for i, branch in enumerate(self.circuit.transformers2w):
 
-            conn_node_id = 'Transformer_' + str(i)
-
-            model = cimdev.GeneralContainer(rfid=conn_node_id, tpe='PowerTransformer',
-                                     resources=[],
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject',
-                                                         'EquipmentContainer': 'Equipment'}
-                                     )
+            model = cimdev.GeneralContainer(rfid=branch.idtag, tpe='PowerTransformer',
+                                            resources=[],
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject',
+                                                                'EquipmentContainer': 'Equipment'}
+                                            )
             model.properties['name'] = branch.name
             model.properties['aliasName'] = branch.name
             text_file.write(model.get_xml(1))
@@ -498,14 +501,14 @@ class CIMExport:
             winding_power_rate = branch.rate / 2
             Zbase = (branch.bus_from.Vnom ** 2) / winding_power_rate
             Ybase = 1 / Zbase
-            model = cimdev.GeneralContainer(rfid=conn_node_id + "_W1", tpe='PowerTransformerEnd',
-                                     resources=winding_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject',
-                                                         'BaseVoltage': 'ConductingEquipment'}
-                                     )
+            model = cimdev.GeneralContainer(rfid=branch.idtag + "_W1", tpe='PowerTransformerEnd',
+                                            resources=winding_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject',
+                                                                'BaseVoltage': 'ConductingEquipment'}
+                                            )
             model.properties['name'] = branch.name
-            model.properties['PowerTransformer'] = conn_node_id
+            model.properties['PowerTransformer'] = branch.idtag
             model.properties['BaseVoltage'] = base_voltages_dict[branch.bus_from.Vnom]
             model.properties['r'] = branch.R / 2 * Zbase
             model.properties['x'] = branch.X / 2 * Zbase
@@ -526,14 +529,13 @@ class CIMExport:
             # W2 (To)
             Zbase = (branch.bus_to.Vnom ** 2) / winding_power_rate
             Ybase = 1 / Zbase
-            model = cimdev.GeneralContainer(rfid=conn_node_id + "_W2", tpe='PowerTransformerEnd',
-                                     resources=winding_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject',
-                                                         'BaseVoltage': 'ConductingEquipment'}
-                                     )
+            model = cimdev.GeneralContainer(rfid=branch.idtag + "_W2", tpe='PowerTransformerEnd',
+                                            resources=winding_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject',
+                                                                'BaseVoltage': 'ConductingEquipment'})
             model.properties['name'] = branch.name
-            model.properties['PowerTransformer'] = conn_node_id
+            model.properties['PowerTransformer'] = branch.idtag
             model.properties['BaseVoltage'] = base_voltages_dict[branch.bus_to.Vnom]
             model.properties['r'] = branch.R / 2 * Zbase
             model.properties['x'] = branch.X / 2 * Zbase
@@ -557,8 +559,8 @@ class CIMExport:
                 Vnom = branch.bus_to.Vnom
                 SVI = (Vnom - Vnom * branch.tap_module) * 100.0 / Vnom
 
-                model = cimdev.GeneralContainer(rfid=conn_node_id + 'Tap_2', tpe='RatioTapChanger', resources=tap_changer_resources)
-                model.properties['TransformerWinding'] = conn_node_id + "_W2"
+                model = cimdev.GeneralContainer(rfid=branch.idtag + 'Tap_2', tpe='RatioTapChanger', resources=tap_changer_resources)
+                model.properties['TransformerWinding'] = branch.idtag + "_W2"
                 model.properties['name'] = branch.name + 'tap changer'
                 model.properties['neutralU'] = Vnom
                 model.properties['stepVoltageIncrement'] = SVI
@@ -568,30 +570,28 @@ class CIMExport:
                 model.properties['subsequentDelay'] = branch.angle
                 text_file.write(model.get_xml(1))
 
-
-
             # Terminal 1 (from)
-            model = cimdev.GeneralContainer(rfid=conn_node_id + '_T1', tpe='Terminal',
-                                     resources=terminal_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject'}
-                                     )
-            model.properties['name'] = bus.name + '_' + branch.name + '_T1'
+            model = cimdev.GeneralContainer(rfid=branch.idtag + '_T1', tpe='Terminal',
+                                            resources=terminal_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject'}
+                                            )
+            model.properties['name'] = branch.name + '_T1'
             model.properties['TopologicalNode'] = bus_id_dict[branch.bus_from]
-            model.properties['ConductingEquipment'] = conn_node_id
+            model.properties['ConductingEquipment'] = branch.idtag
             model.properties['connected'] = 'true'
             model.properties['sequenceNumber'] = '1'
             text_file.write(model.get_xml(1))
 
             # Terminal 2 (to)
-            model = cimdev.GeneralContainer(rfid=conn_node_id + '_T2', tpe='Terminal',
-                                     resources=terminal_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject'}
-                                     )
-            model.properties['name'] = bus.name + '_' + branch.name + '_T2'
+            model = cimdev.GeneralContainer(rfid=branch.idtag + '_T2', tpe='Terminal',
+                                            resources=terminal_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject'}
+                                            )
+            model.properties['name'] = branch.name + '_T2'
             model.properties['TopologicalNode'] = bus_id_dict[branch.bus_to]
-            model.properties['ConductingEquipment'] = conn_node_id
+            model.properties['ConductingEquipment'] = branch.idtag
             model.properties['connected'] = 'true'
             model.properties['sequenceNumber'] = '1'
             text_file.write(model.get_xml(1))
@@ -600,7 +600,6 @@ class CIMExport:
 
             if branch.branch_type == gcdev.BranchType.Line or branch.branch_type == gcdev.BranchType.Branch:
 
-                conn_node_id = 'Branch_' + str(i)
                 Zbase = (branch.bus_from.Vnom ** 2) / self.circuit.Sbase
 
                 if branch.bus_from.Vnom <= 0.0:
@@ -608,19 +607,19 @@ class CIMExport:
                 else:
                     Ybase = 1 / Zbase
 
-                model = cimdev.GeneralContainer(rfid=conn_node_id, tpe='ACLineSegment',
-                                         resources=['BaseVoltage'],
-                                         class_replacements={'name': 'IdentifiedObject',
-                                                             'aliasName': 'IdentifiedObject',
-                                                             'BaseVoltage': 'ConductingEquipment',
-                                                             'value': 'CurrentLimit'}
-                                         )
+                model = cimdev.GeneralContainer(rfid=branch.idtag, tpe='ACLineSegment',
+                                                resources=['BaseVoltage'],
+                                                class_replacements={'name': 'IdentifiedObject',
+                                                                    'aliasName': 'IdentifiedObject',
+                                                                    'BaseVoltage': 'ConductingEquipment',
+                                                                    'value': 'CurrentLimit'}
+                                                )
                 model.properties['name'] = branch.name
                 model.properties['aliasName'] = branch.name
                 model.properties['BaseVoltage'] = base_voltages_dict[branch.bus_from.Vnom]
                 model.properties['r'] = branch.R * Zbase
                 model.properties['x'] = branch.X * Zbase
-                model.properties['gch'] = branch.G * Ybase
+                model.properties['gch'] = 0
                 model.properties['bch'] = branch.B * Ybase
                 model.properties['r0'] = 0.0
                 model.properties['x0'] = 0.0
@@ -632,8 +631,7 @@ class CIMExport:
 
             elif branch.branch_type == gcdev.BranchType.Switch:
 
-                conn_node_id = 'Switch_' + str(i)
-                model = cimdev.GeneralContainer(rfid=conn_node_id, tpe='Switch', resources=['BaseVoltage'])
+                model = cimdev.GeneralContainer(rfid=branch.idtag, tpe='Switch', resources=['BaseVoltage'])
                 model.properties['name'] = branch.name
                 model.properties['aliasName'] = branch.name
                 model.properties['BaseVoltage'] = base_voltages_dict[branch.bus_from.Vnom]
@@ -652,7 +650,7 @@ class CIMExport:
                 else:
                     Ybase = 1 / Zbase
 
-                model = cimdev.GeneralContainer(rfid=conn_node_id, tpe='ACLineSegment', resources=['BaseVoltage'])
+                model = cimdev.GeneralContainer(rfid=branch.idtag, tpe='ACLineSegment', resources=['BaseVoltage'])
                 model.properties['name'] = branch.name
                 model.properties['aliasName'] = branch.name
                 model.properties['BaseVoltage'] = base_voltages_dict[branch.bus_from.Vnom]
@@ -668,27 +666,27 @@ class CIMExport:
                 text_file.write(model.get_xml(1))
 
             # Terminal 1 (from)
-            model = cimdev.GeneralContainer(rfid=conn_node_id + '_T1', tpe='Terminal',
-                                     resources=terminal_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject'}
-                                     )
+            model = cimdev.GeneralContainer(rfid=branch.idtag + '_T1', tpe='Terminal',
+                                            resources=terminal_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject'}
+                                            )
             model.properties['name'] = bus.name + '_' + branch.name + '_T1'
             model.properties['TopologicalNode'] = bus_id_dict[branch.bus_from]
-            model.properties['ConductingEquipment'] = conn_node_id
+            model.properties['ConductingEquipment'] = branch.idtag
             model.properties['connected'] = 'true'
             model.properties['sequenceNumber'] = '1'
             text_file.write(model.get_xml(1))
 
             # Terminal 2 (to)
-            model = cimdev.GeneralContainer(rfid=conn_node_id + '_T2', tpe='Terminal',
-                                     resources=terminal_resources,
-                                     class_replacements={'name': 'IdentifiedObject',
-                                                         'aliasName': 'IdentifiedObject'}
-                                     )
+            model = cimdev.GeneralContainer(rfid=branch.idtag + '_T2', tpe='Terminal',
+                                            resources=terminal_resources,
+                                            class_replacements={'name': 'IdentifiedObject',
+                                                                'aliasName': 'IdentifiedObject'}
+                                            )
             model.properties['name'] = bus.name + '_' + branch.name + '_T2'
             model.properties['TopologicalNode'] = bus_id_dict[branch.bus_to]
-            model.properties['ConductingEquipment'] = conn_node_id
+            model.properties['ConductingEquipment'] = branch.idtag
             model.properties['connected'] = 'true'
             model.properties['sequenceNumber'] = '1'
             text_file.write(model.get_xml(1))
@@ -727,7 +725,8 @@ class CIMImport:
         if self.progress_func is not None:
             self.progress_func(val)
 
-    def parse_model(self, cim: CIMCircuit, circuit: MultiCircuit):
+    @staticmethod
+    def parse_model(cim: CIMCircuit, circuit: MultiCircuit):
         """
 
         :param cim:
@@ -979,7 +978,6 @@ class CIMImport:
         for f in lst2:
             name, file_extension = os.path.splitext(f)
             self.emit_text('Parsing xml structure of ' + name)
-
             self.cim.parse_xml_text(text_lines=data[f])
 
         # replace CIM references in the CIM objects
