@@ -1,23 +1,25 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import os
 import json
 
 from GridCal.Engine.basic_structures import Logger
 
-from GridCal.Engine.IO.json_parser import save_json_file
+from GridCal.Engine.IO.json_parser import save_json_file_v3, save_json_file_v4
 from GridCal.Engine.IO.cim.cim_parser import CIMExport
 from GridCal.Engine.IO.excel_interface import save_excel, load_from_xls, interpret_excel_v3, interprete_excel_v2
 from GridCal.Engine.IO.pack_unpack import create_data_frames, data_frames_to_circuit
@@ -28,10 +30,12 @@ from GridCal.Engine.IO.dpx_parser import load_dpx
 from GridCal.Engine.IO.ipa_parser import load_iPA
 from GridCal.Engine.IO.json_parser import parse_json, parse_json_data_v2, parse_json_data_v3
 from GridCal.Engine.IO.raw_parser import PSSeParser
+from GridCal.Engine.IO.power_world_parser import PowerWorldParser
 from GridCal.Engine.IO.cim.cim_parser import CIMImport
 from GridCal.Engine.IO.zip_interface import save_data_frames_to_zip, get_frames_from_zip, get_session_tree, load_session_driver_objects
 from GridCal.Engine.IO.sqlite_interface import save_data_frames_to_sqlite, open_data_frames_from_sqlite
 from GridCal.Engine.IO.h5_interface import save_h5, open_h5
+from GridCal.Engine.IO.rawx_parser import rawx_parse, rawx_writer
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 
 
@@ -167,8 +171,26 @@ class FileOpen:
                     else:
                         self.logger.add_error('Unknown json format')
 
+                elif file_extension.lower() == '.ejson3':
+                    data = json.load(open(self.file_name))
+                    self.circuit = parse_json_data_v3(data, self.logger)
+
+                elif file_extension.lower() == '.ejson4':
+                    data = json.load(open(self.file_name))
+                    # self.circuit = parse_json_data_v4(data, self.logger)
+
                 elif file_extension.lower() == '.raw':
-                    parser = PSSeParser(self.file_name)
+                    parser = PSSeParser(self.file_name, text_func=text_func,  progress_func=progress_func)
+                    self.circuit = parser.circuit
+                    self.logger += parser.logger
+
+                elif file_extension.lower() == '.rawx':
+                    circuit, logger = rawx_parse(self.file_name)
+                    self.circuit = circuit
+                    self.logger += logger
+
+                elif file_extension.lower() == '.epc':
+                    parser = PowerWorldParser(self.file_name)
                     self.circuit = parser.circuit
                     self.logger += parser.logger
 
@@ -222,8 +244,11 @@ class FileSave:
         elif self.file_name.endswith('.sqlite'):
             logger = self.save_sqlite()
 
-        elif self.file_name.endswith('.json'):
-            logger = self.save_json()
+        elif self.file_name.endswith('.ejson3'):
+            logger = self.save_json_v3()
+
+        elif self.file_name.endswith('.ejson4'):
+            logger = self.save_json_v4()
 
         elif self.file_name.endswith('.xml'):
             logger = self.save_cim()
@@ -231,6 +256,9 @@ class FileSave:
         elif self.file_name.endswith('.gch5'):
             logger = self.save_h5()
 
+        elif self.file_name.endswith('.rawx'):
+            logger = self.save_rawx()
+
         else:
             logger = Logger()
             logger.add_error('File path extension not understood', self.file_name)
@@ -282,13 +310,22 @@ class FileSave:
 
         return logger
 
-    def save_json(self):
+    def save_json_v3(self):
         """
         Save the circuit information in json format
         :return:logger with information
         """
 
-        logger = save_json_file(self.file_name, self.circuit, self.simulation_drivers)
+        logger = save_json_file_v3(self.file_name, self.circuit, self.simulation_drivers)
+        return logger
+
+    def save_json_v4(self):
+        """
+        Save the circuit information in json format
+        :return:logger with information
+        """
+
+        logger = save_json_file_v4(self.file_name, self.circuit, self.simulation_drivers)
         return logger
 
     def save_cim(self):
@@ -314,3 +351,12 @@ class FileSave:
 
         return logger
 
+    def save_rawx(self):
+        """
+        Save the circuit information in json format
+        :return:logger with information
+        """
+
+        logger = rawx_writer(self.file_name, self.circuit)
+        return logger
+
