@@ -1,18 +1,20 @@
 """
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 """
 
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
@@ -229,11 +231,15 @@ def parse_generators(circuit: MultiCircuit, data, bus_idx_dict, logger: Logger):
         names = ['gen ' + str(i) for i in range(n)]
     for i in range(len(table)):
         bus_idx = bus_idx_dict[int(table[i, matpower_gen.GEN_BUS])]
+        # TODO: Calculate pf based on reactive_power
         gen = Generator(name=names[i],
                         active_power=table[i, matpower_gen.PG],
                         voltage_module=table[i, matpower_gen.VG],
                         Qmax=table[i, matpower_gen.QMAX],
-                        Qmin=table[i, matpower_gen.QMIN])
+                        Qmin=table[i, matpower_gen.QMIN],
+                        p_min=table[i, matpower_gen.PMIN],
+                        p_max=table[i, matpower_gen.PMAX]
+                        )
 
         # Add the generator to the bus
         gen.bus = circuit.buses[bus_idx]
