@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from typing import Dict
 import pandas as pd
@@ -396,8 +398,11 @@ def data_frames_to_circuit(data: Dict):
 
                             else:
                                 # regular types (int, str, float, etc...)
-                                val = dtype(df[object_property_name].values[i])
-                                setattr(devices[i], object_property_name, val)
+                                try:
+                                    val = dtype(df[object_property_name].values[i])
+                                    setattr(devices[i], object_property_name, val)
+                                except ValueError:
+                                    circuit.logger.add_error('type error', devices[i].name, df[object_property_name].values[i])
 
                         # search the profiles in the data and assign them
                         if object_property_name in template_elm.properties_with_profile.keys():
