@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import chardet
 import re
@@ -223,7 +225,7 @@ class PSSeBus:
 
         bustype = {1: BusMode.PQ, 2: BusMode.PV, 3: BusMode.Slack, 4: BusMode.PQ}
 
-        if version == 33:
+        if version >= 33:
             n = len(data[0])
             dta = np.zeros(13, dtype=object)
             dta[0:n] = data[0]
@@ -235,7 +237,7 @@ class PSSeBus:
             name = self.NAME.replace("'", "")
             self.bus = Bus(name=name,
                            vnom=self.BASKV, code=str(self.I), vmin=self.EVLO, vmax=self.EVHI, xpos=0, ypos=0, active=True,
-                           area=self.AREA, zone=self.ZONE)
+                           area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
         elif version == 32:
 
@@ -244,17 +246,17 @@ class PSSeBus:
             # create bus
             name = self.NAME
             self.bus = Bus(name=name, code=str(self.I), vnom=self.BASKV, vmin=0.9, vmax=1.1, xpos=0, ypos=0,
-                           active=True, area=self.AREA, zone=self.ZONE)
+                           active=True, area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
         elif version in [29, 30]:
-            # I, ’NAME’, BASKV, IDE, GL, BL, AREA, ZONE, VM, VA, OWNER
+            # I, 'NAME', BASKV, IDE, GL, BL, AREA, ZONE, VM, VA, OWNER
             self.I, self.NAME, self.BASKV, self.IDE, self.GL, self.BL, \
             self.AREA, self.ZONE, self.VM, self.VA, self.OWNER = data[0]
 
             # create bus
             name = self.NAME
             self.bus = Bus(name=name, code=str(self.I), vnom=self.BASKV, vmin=0.9, vmax=1.1, xpos=0, ypos=0,
-                           active=True, area=self.AREA, zone=self.ZONE)
+                           active=True, area=self.AREA, zone=self.ZONE, Vm0=self.VM, Va0=np.deg2rad(self.VA))
 
             if self.GL > 0 or self.BL > 0:
                 sh = Shunt(name='Shunt_' + str(self.I),
@@ -342,7 +344,7 @@ class PSSeLoad:
                    idtag=None,
                    code=name,
                    active=bool(self.STATUS),
-                   P=p, Q=q)
+                   P=p, Q=q, Ir=ir, Ii=ii, G=g, B=b)
 
         return elm
 
@@ -398,7 +400,7 @@ class PSSeSwitchedShunt:
         name = name.strip()
 
         # GL and BL come in MW and MVAr
-        # THey must be in siemens
+        # They must be in siemens
         vv = bus.Vnom ** 2.0
 
         if vv == 0:
@@ -682,7 +684,7 @@ class PSSeTwoTerminalDCLine:
             '''
             'NAME',MDC,RDC,SETVL,VSCHD,VCMOD,RCOMP,DELTI,METER,DCVMIN,CCCITMX,CCCACC
             IPR,NBR,ANMXR,ANMNR,RCR,XCR,EBASR,TRR,TAPR,TMXR,TMNR,STPR,ICR,IFR,ITR,IDR,XCAPR
-            IPI,NBI,ANMXI,ANMNI,RCI,XCI,EBASI,TRI,TAPI,TMXI,TMNI,STPI,ICI,IFI,ITI,IDI,XCAPI 
+            IPI,NBI,ANMXI,ANMNI,RCI,XCI,EBASI,TRI,TAPI,TMXI,TMNI,STPI,ICI,IFI,ITI,IDI,XCAPI
             '''
 
             self.NAME, self.MDC, self.RDC, self.SETVL, self.VSCHD, self.VCMOD, self.RCOMP, self.DELTI, self.METER, \
@@ -755,6 +757,7 @@ class PSSeTwoTerminalDCLine:
                        Vset_f=Vset_f,
                        Vset_t=Vset_t,
                        rate=specified_power,
+                       r=r_pu,
                        min_firing_angle_f=np.deg2rad(self.ANMNR),
                        max_firing_angle_f=np.deg2rad(self.ANMXR),
                        min_firing_angle_t=np.deg2rad(self.ANMNI),
@@ -800,7 +803,7 @@ class PSSeVscDCLine:
         elif version == 29:
 
             '''
-            ’NAME’, MDC, RDC, O1, F1, ... O4, F4
+            'NAME', MDC, RDC, O1, F1, ... O4, F4
             IBUS,TYPE,MODE,DCSET,ACSET,ALOSS,BLOSS,MINLOSS,SMAX,IMAX,PWF,MAXQ,MINQ,REMOT,RMPCT
             IBUS,TYPE,MODE,DCSET,ACSET,ALOSS,BLOSS,MINLOSS,SMAX,IMAX,PWF,MAXQ,MINQ,REMOT,RMPCT
             '''
@@ -837,8 +840,8 @@ class PSSeVscDCLine:
         # P = dV^2 / R
         V1 = bus1.Vnom * Vset_f
         V2 = bus2.Vnom * Vset_t
-        dV = (V1-V2) * 1000.0  # in V
-        P = dV * dV / self.RDC  # power in W
+        dV = (V1 - V2) * 1000.0  # in V
+        P = dV * dV / self.RDC if self.RDC != 0 else 0  # power in W
         specified_power = P * 1e-6  # power in MW
 
         obj = HvdcLine(bus_from=bus1,
@@ -853,6 +856,97 @@ class PSSeVscDCLine:
         return obj
 
 
+def get_psse_transformer_impedances(CW, CZ, CM, V1, V2, sbase, logger, code,
+                                    MAG1, MAG2, WINDV1, WINDV2, ANG1, NOMV1, NOMV2,
+                                    R1_2, X1_2, SBASE1_2):
+    """
+
+    CW	Winding I/O code
+    1	Turns ratio (pu on bus base kV)
+    2	Winding voltage kV
+    3	Turns ratio (pu on nominal winding kV)
+
+    CZ	Impedance I/O code
+    1	Z pu (winding kV system MVA)
+    2	Z pu (winding kV winding MVA)
+    3	Load loss (W) & |Z| (pu)
+
+    CM	Admittance I/O code
+    1	Y pu (system base)
+    2	No load loss (W) & Exciting I (pu)
+
+
+    :param CW:
+    :param CZ:
+    :param CM:
+    :return:
+    """
+
+    g = MAG1
+    b = MAG2
+    tap_mod = WINDV1 / WINDV2
+    tap_angle = np.deg2rad(ANG1)
+
+    # if self.CW == 2 or self.CW == 3:
+    #     tap_mod *= bus_to.Vnom / bus_from.Vnom
+    #
+    # if self.CW == 3:
+    #     tap_mod *= self.NOMV1 / self.NOMV2
+
+    """
+    CW	Winding I/O code
+    1	Turns ratio (pu on bus base kV)
+    2	Winding voltage kV
+    3	Turns ratio (pu on nominal winding kV)        
+    """
+
+    if CW == 1:
+        tap_mod = WINDV1 / WINDV2
+
+    elif CW == 2:
+        tap_mod = (WINDV1 / V1) / (WINDV2 / V2)
+
+    elif CW == 3:
+        tap_mod = (WINDV1 / WINDV2) * (NOMV1 / NOMV2)
+
+    """
+    CZ	Impedance I/O code
+    1	Z pu (winding kV system MVA)
+    2	Z pu (winding kV winding MVA)
+    3	Load loss (W) & |Z| (pu)
+    """
+    r = 1e-20
+    x = 1e-20
+    if CZ == 1:
+        # the transformer values are in system base
+        r = R1_2
+        x = X1_2
+
+    elif CZ == 2:
+        # pu on Winding 1 to 2 MVA base (SBASE1-2) and winding voltage base
+        logger.add_warning('Transformer not in system base', code)
+
+        if SBASE1_2 > 0:
+            zb = sbase / SBASE1_2
+            r = R1_2 * zb
+            x = X1_2 * zb
+
+        else:
+            logger.add_error('Transformer SBASE1_2 is zero', code)
+
+    elif CZ == 3:
+        # R1-2 is the load loss in watts, and X1-2 is the impedance magnitude
+        # in pu on Winding 1 to 2 MVA base (SBASE1-2) and winding voltage base
+        r = R1_2 * 1e-6 / SBASE1_2 / sbase
+        x = np.sqrt(X1_2 * X1_2 - r * r)
+        logger.add_warning('Transformer not in system base', code)
+
+    else:
+        raise Exception('Unknown impedance combination CZ=' + str(CZ))
+
+    return r, x, g, b, tap_mod, tap_angle
+
+
 class PSSeTransformer:
 
     def __init__(self, data, version, logger: Logger):
@@ -885,7 +979,7 @@ class PSSeTransformer:
                 self.windings = 2
 
                 '''
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4,VECGRP
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4,VECGRP
                 R1-2,X1-2,SBASE1-2
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD1,CONT1,RMA1,RMI1,VMA1,VMI1,NTP1,TAB1,CR1,CX1,CNXA1
                 WINDV2,NOMV2
@@ -906,7 +1000,7 @@ class PSSeTransformer:
                 self.windings = 3
 
                 '''
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4,VECGRP
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4,VECGRP
                 R1-2,X1-2,SBASE1-2,R2-3,X2-3,SBASE2-3,R3-1,X3-1,SBASE3-1,VMSTAR,ANSTAR
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD1,CONT1,RMA1,RMI1,VMA1,VMI1,NTP1,TAB1,CR1,CX1,CNXA1
                 WINDV2,NOMV2,ANG2,RATA2,RATB2,RATC2,COD2,CONT2,RMA2,RMI2,VMA2,VMI2,NTP2,TAB2,CR2,CX2,CNXA2
@@ -931,7 +1025,7 @@ class PSSeTransformer:
         elif version == 32:
 
             '''
-            I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4
+            I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
 
             R1-2,X1-2,SBASE1-2,R2-3,X2-3,SBASE2-3,R3-1,X3-1,SBASE3-1,VMSTAR,ANSTAR
 
@@ -1024,14 +1118,14 @@ class PSSeTransformer:
 
                 2 windings -> 4 lines
 
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
                 R1-2,X1-2,SBASE1-2
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD,CONT,RMA,RMI,VMA,VMI,NTP,TAB,CR,CX
                 WINDV2,NOMV2
 
                 3 windings -> 5 lines
 
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
                 R1-2,X1-2,SBASE1-2,R2-3,X2-3,SBASE2-3,R3-1,X3-1,SBASE3-1,VMSTAR,ANSTAR
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD,CONT,RMA,RMI,VMA,VMI,NTP,TAB,CR,CX
                 WINDV2,NOMV2,ANG2,RATA2,RATB2,RATC2
@@ -1045,7 +1139,7 @@ class PSSeTransformer:
             if len(data[1]) == 3:
 
                 '''
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
                 R1-2,X1-2,SBASE1-2
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD,CONT,RMA,RMI,VMA,VMI,NTP,TAB,CR,CX
                 WINDV2,NOMV2
@@ -1063,7 +1157,7 @@ class PSSeTransformer:
             else:
 
                 '''
-                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,’NAME’,STAT,O1,F1,...,O4,F4
+                I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
                 R1-2,X1-2,SBASE1-2,R2-3,X2-3,SBASE2-3,R3-1,X3-1,SBASE3-1,VMSTAR,ANSTAR
 
                 WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD,CONT,RMA,RMI,VMA,VMI,NTP,TAB,CR,CX
@@ -1126,12 +1220,6 @@ class PSSeTransformer:
             bus_from = psse_bus_dict[self.I]
             bus_to = psse_bus_dict[self.J]
 
-            # 11000_AGUAYO_400_12004_ABANTO_400_1_CKT
-            # if self.NAME != "":
-            # name = "{0}:{1}_{2}_{3}_{4}_{5}_{6}_{7}".format(self.NAME,
-            #                                                 self.I, bus_from.name, bus_from.Vnom,
-            #                                                 self.J, bus_to.name, bus_to.Vnom, self.CKT)
-            # else:
             name = "{0}_{1}_{2}_{3}_{4}_{5}_{6}".format(self.I, bus_from.name, bus_from.Vnom,
                                                         self.J, bus_to.name, bus_to.Vnom, self.CKT)
 
@@ -1143,33 +1231,6 @@ class PSSeTransformer:
             """            
             PSS/e's randomness:            
             """
-            zbs = bus_from.Vnom * bus_from.Vnom / sbase
-
-            r = self.R1_2
-            x = self.X1_2
-            g = self.MAG1
-            b = self.MAG2
-            tap_mod = self.WINDV1 / self.WINDV2
-            use_winding_base_voltage = True
-
-            if self.CZ == 3:
-                r *= 1e-6 / self.SBASE1_2 / sbase
-                x = np.sqrt(self.X1_2 * self.X1_2 - r * r)
-
-            if self.CZ == 2:
-                if self.SBASE1_2 > 0:
-                    zb = sbase / self.SBASE1_2
-                    r *= zb
-                    x *= zb
-                else:
-                    logger.add_error('Transformer SBASE1_2 is zero', code)
-
-            # adjust tap
-            if self.CW == 2 or self.CW == 3:
-                tap_mod *= bus_to.Vnom / bus_from.Vnom
-
-            if self.CW == 3:
-                tap_mod *= self.NOMV1 / self.NOMV2
 
             if self.NOMV1 == 0:
                 V1 = bus_from.Vnom
@@ -1183,13 +1244,28 @@ class PSSeTransformer:
 
             contingency_factor = self.RATB1 / self.RATA1 if self.RATA1 > 0.0 else 1.0
 
+            r, x, g, b, tap_mod, tap_angle = get_psse_transformer_impedances(self.CW, self.CZ, self.CM,
+                                                                             V1, V2,
+                                                                             sbase, logger, code,
+                                                                             self.MAG1, self.MAG2, self.WINDV1,
+                                                                             self.WINDV2,
+                                                                             self.ANG1, self.NOMV1, self.NOMV2,
+                                                                             self.R1_2, self.X1_2, self.SBASE1_2)
+
+            if V1 >= V2:
+                HV = V1
+                LV = V2
+            else:
+                HV = V2
+                LV = V1
+
             elm = Transformer2W(bus_from=bus_from,
                                 bus_to=bus_to,
                                 idtag=None,
                                 code=code,
                                 name=name,
-                                HV=V1,
-                                LV=V2,
+                                HV=HV,
+                                LV=LV,
                                 r=r,
                                 x=x,
                                 g=g,
@@ -1197,7 +1273,7 @@ class PSSeTransformer:
                                 rate=self.RATA1,
                                 contingency_factor=round(contingency_factor, 6),
                                 tap=tap_mod,
-                                shift_angle=np.deg2rad(self.ANG1),
+                                shift_angle=tap_angle,
                                 active=bool(self.STAT),
                                 mttf=0,
                                 mttr=0)
@@ -1226,8 +1302,8 @@ class PSSeTransformer:
             else:
                 V3 = self.NOMV3
 
-            """            
-            PSS/e's randomness:            
+            """
+            PSS/e's randomness:
             """
 
             # see: https://en.wikipedia.org/wiki/Per-unit_system
@@ -1342,8 +1418,8 @@ class PSSeFACTS:
 
         if version in [32, 33, 34]:
             '''
-            ’NAME’,I,J,MODE,PDES,QDES,VSET,SHMX,TRMX,VTMN,VTMX,VSMX,IMX,LINX,
-            RMPCT,OWNER,SET1,SET2,VSREF,REMOT,’MNAME’
+            'NAME',I,J,MODE,PDES,QDES,VSET,SHMX,TRMX,VTMN,VTMX,VSMX,IMX,LINX,
+            RMPCT,OWNER,SET1,SET2,VSREF,REMOT,'MNAME'
             '''
 
             self.NAME, self.I, self.J, self.MODE, self.PDES, self.QDES, self.VSET, self.SHMX, \
@@ -1352,8 +1428,8 @@ class PSSeFACTS:
 
         elif version == 29:
             '''
-            ’NAME’,I,J,MODE,PDES,QDES,VSET,SHMX,TRMX,VTMN,VTMX,VSMX,IMX,LINX,
-                RMPCT,OWNER,SET1,SET2,VSREF,REMOT,’MNAME’
+            'NAME',I,J,MODE,PDES,QDES,VSET,SHMX,TRMX,VTMN,VTMX,VSMX,IMX,LINX,
+                RMPCT,OWNER,SET1,SET2,VSREF,REMOT,'MNAME'
             '''
 
             self.NAME, self.I, self.J, self.MODE, self.PDES, self.QDES, self.VSET, self.SHMX, \
@@ -1556,7 +1632,7 @@ def interpret_line(line, splitter=','):
 
 class PSSeParser:
 
-    def __init__(self, file_name):
+    def __init__(self, file_name, text_func=print,  progress_func=None):
         """
         Parse PSSe file
         Args:
@@ -1569,7 +1645,7 @@ class PSSeParser:
 
         self.file_name = file_name
 
-        self.pss_grid, logs = self.parse_psse()
+        self.pss_grid, logs = self.parse_psse(text_func=text_func,  progress_func=progress_func)
 
         self.logger += logs
 
@@ -1578,16 +1654,26 @@ class PSSeParser:
         self.circuit.comments = 'Converted from the PSS/e .raw file ' \
                                 + os.path.basename(file_name) + '\n\n' + str(self.logger)
 
-    def read_and_split(self) -> (List[AnyStr], Dict[AnyStr, AnyStr]):
+    def read_and_split(self, text_func=None,  progress_func=None) -> (List[AnyStr], Dict[AnyStr, AnyStr]):
         """
         Read the text file and split it into sections
         :return: list of sections, dictionary of sections by type
         """
 
+        if text_func is not None:
+            text_func("Detecting raw file encoding...")
+
+        if progress_func is not None:
+            progress_func(0)
+
         # make a guess of the file encoding
         detection = chardet.detect(open(self.file_name, "rb").read())
 
         # open the text file into a variable
+
+        if text_func is not None:
+            text_func("Reading raw file...")
+
         txt = ''
         with open(self.file_name, 'r', encoding=detection['encoding']) as my_file:
             for line in my_file:
@@ -1599,9 +1685,12 @@ class PSSeParser:
 
         sections_dict = dict()
 
+        if text_func is not None:
+            text_func("Parsing the raw file information...")
+
         str_a = 'End of'.lower()
         str_b = 'data'.lower()
-
+        n_sec = len(sections)
         for i, sec in enumerate(sections):
             data = sec.split('\n')
             first = data.pop(0).lower()
@@ -1619,9 +1708,12 @@ class PSSeParser:
 
                 sections_dict[name] = data2
 
+            if progress_func is not None:
+                progress_func((i / n_sec) * 100)
+
         return sections, sections_dict
 
-    def parse_psse(self) -> (MultiCircuit, List[AnyStr]):
+    def parse_psse(self, text_func=None,  progress_func=None) -> (MultiCircuit, List[AnyStr]):
         """
         Parser implemented according to:
             - POM section 4.1.1 Power Flow Raw Data File Contents (v.29)
@@ -1633,7 +1725,10 @@ class PSSeParser:
 
         logger = Logger()
 
-        sections, sections_dict = self.read_and_split()
+        if text_func is not None:
+            text_func("Reading file...")
+
+        sections, sections_dict = self.read_and_split(text_func=text_func,  progress_func=progress_func)
 
         # header -> new grid
         grid = PSSeGrid(interpret_line(sections[0]))
@@ -1697,6 +1792,9 @@ class PSSeParser:
             # get the parsers for the declared object type
             objects_list, ObjectT, lines_per_object = values
 
+            if text_func is not None:
+                text_func("Converting {0}...".format(key))
+
             if key in sections_dict.keys():
                 lines = sections_dict[key]
 
@@ -1742,6 +1840,9 @@ class PSSeParser:
                     # add lines
                     l += lines_per_object2
 
+                    if progress_func is not None:
+                        progress_func((l / len(lines)) * 100)
+
             else:
                 pass
 
