@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from io import StringIO, TextIOWrapper, BytesIO
 import os
@@ -19,15 +21,15 @@ import numpy as np
 import chardet
 import pandas as pd
 import zipfile
-from typing import List, Dict
+from typing import List, Dict, Any
 from GridCal.Engine.basic_structures import Logger
 from GridCal.Engine.IO.generic_io_functions import parse_config_df
-from GridCal.Gui.Session.session import SimulationSession
+# from GridCal.Gui.Session.session import SimulationSession
 
 
 def save_data_frames_to_zip(dfs: Dict[str, pd.DataFrame], filename_zip="file.zip",
                             text_func=None, progress_func=None,
-                            sessions: List[SimulationSession] = []):
+                            sessions: List[Any] = []):
     """
     Save a list of DataFrames to a zip file without saving to disk the csv files
     :param dfs: dictionary of pandas dataFrames {name: DataFrame}
@@ -55,13 +57,13 @@ def save_data_frames_to_zip(dfs: Dict[str, pd.DataFrame], filename_zip="file.zip
             if name.endswith('_prof'):
 
                 # compose the csv file name
-                filename = name + ".pkl"
+                filename = name + ".parquet"
 
                 # open a string buffer
-                try:  # try pickle
+                try:  # try parquet file
                     with BytesIO() as buffer:
                         # save the DataFrame to the buffer, protocol4 is to be compatible with python 3.6
-                        df.to_pickle(buffer, protocol=4)
+                        df.to_parquet(buffer)
                         # save the buffer to the zip file
                         f_zip_ptr.writestr(filename, buffer.getvalue())
 
@@ -104,6 +106,10 @@ def save_data_frames_to_zip(dfs: Dict[str, pd.DataFrame], filename_zip="file.zip
                                 text_func('Flushing ' + filename + ' to ' + filename_zip + '...')
 
                             with BytesIO() as buffer:
+                                # np.save(buffer, np.array(arr))  # save the DataFrame to the buffer
+                                # f_zip_ptr.writestr(filename + '.npy', buffer.getvalue())  # save the buffer to the zip file
+
+                                # save the DataFrame to the buffer, protocol4 is to be compatible with python 3.6
                                 np.save(buffer, np.array(arr))  # save the DataFrame to the buffer
                                 f_zip_ptr.writestr(filename + '.npy', buffer.getvalue())  # save the buffer to the zip file
 
@@ -128,6 +134,13 @@ def read_data_frame_from_zip(file_pointer, extension, index_col=None, logger=Log
     try:
         if extension == '.csv':
             return pd.read_csv(file_pointer, index_col=index_col)
+
+        elif extension == '.npy':
+            try:
+                return np.load(file_pointer)
+            except ValueError:
+                return np.load(file_pointer, allow_pickle=True)
+
         elif extension == '.pkl':
             try:
                 return pd.read_pickle(file_pointer)
@@ -137,6 +150,17 @@ def read_data_frame_from_zip(file_pointer, extension, index_col=None, logger=Log
             except AttributeError as e:
                 logger.add_error(str(e) + ' Upgrading pandas might help.', device=file_pointer.name)
                 return None
+
+        elif extension == '.parquet':
+            try:
+                return pd.read_parquet(file_pointer)
+            except ValueError as e:
+                logger.add_error(str(e), device=file_pointer.name)
+                return None
+            except AttributeError as e:
+                logger.add_error(str(e) + ' Upgrading pandas might help.', device=file_pointer.name)
+                return None
+
     except EOFError:
         return None
     except zipfile.BadZipFile:
@@ -249,14 +273,21 @@ def load_session_driver_objects(file_name_zip: str, session_name: str, study_nam
             path = name.split('/')
             if len(path) > 3:
                 if path[0].lower() == 'sessions' and session_name == path[1] and study_name == path[2]:
-                    arr_name = path[3].replace('.npy', '')
+
                     # create a buffer to read the file
                     file_pointer = zip_file_pointer.open(name)
 
-                    try:
-                        data[arr_name] = np.load(file_pointer)
-                    except ValueError:
-                        data[arr_name] = np.load(file_pointer, allow_pickle=True)
+                    # split the file name into name and extension
+                    _, extension = os.path.splitext(name)
+                    arr_name = path[3].replace(extension, '')
+
+                    # read the data
+                    data[arr_name] = read_data_frame_from_zip(file_pointer, extension)
+
+                    # try:
+                    #     data[arr_name] = np.load(file_pointer)
+                    # except ValueError:
+                    #     data[arr_name] = np.load(file_pointer, allow_pickle=True)
 
     return data
 
