@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import time
 import datetime
 import numpy as np
@@ -26,7 +28,7 @@ from GridCal.Engine.Simulations.driver_types import SimulationTypes
 from GridCal.Engine.Simulations.driver_template import TimeSeriesDriverTemplate
 
 
-@jit(nopython=True, parallel=False)
+@jit(nopython=True, parallel=False, cache=True)
 def compute_flows_numba_t(e, c, nt, LODF, Flows, rates, overload_count, max_overload, worst_flows):
     """
     Compute LODF based flows (Sf)
@@ -59,7 +61,7 @@ def compute_flows_numba_t(e, c, nt, LODF, Flows, rates, overload_count, max_over
             worst_flows[t, e] = flow_n_1
 
 
-@jit(nopython=True, parallel=True)
+@jit(nopython=True, parallel=True, cache=True)
 def compute_flows_numba(e, nt, contingency_branch_idx, LODF, Flows, rates, overload_count,
                         max_overload, worst_flows, parallelize_from=500):
     """
@@ -156,9 +158,20 @@ class ContingencyAnalysisTimeSeries(TimeSeriesDriverTemplate):
 
         self.progress_text.emit('Computing branch base loading...')
         Pbus = ts_numeric_circuit.Sbus.real
-        flows = linear_analysis.get_flows_time_series(Pbus)
         rates = ts_numeric_circuit.ContingencyRates.T
 
+        # get flow
+        if self.options.use_provided_flows:
+            flows = self.options.Pf
+
+            if self.options.Pf is None:
+                msg = 'The option to use the provided flows is enabled, but no flows are available'
+                self.logger.add_error(msg)
+                raise Exception(msg)
+        else:
+            # compute the base Sf
+            flows = linear_analysis.get_flows_time_series(Pbus)  # will be converted to MW internally
+
         self.progress_text.emit('Computing N-1 loading...')
 
         for ie, e in enumerate(br_idx):
@@ -199,8 +212,6 @@ class ContingencyAnalysisTimeSeries(TimeSeriesDriverTemplate):
 
         end = time.time()
         self.elapsed = end - start
-        self.progress_text.emit('Done!')
-        self.done_signal.emit()
 
 
 if __name__ == '__main__':
