@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 """
 This file implements a DC-OPF for time series
@@ -259,27 +261,24 @@ def add_battery_discharge_restriction(problem: LpProblem, SoC0, Capacity, Effici
 
 class OpfAcTimeSeries(OpfTimeSeries):
 
-    def __init__(self, numerical_circuit: OpfTimeCircuit, start_idx, end_idx, solver: MIPSolvers = MIPSolvers.CBC,
-                 batteries_energy_0=None):
+    def __init__(self, numerical_circuit: OpfTimeCircuit, start_idx, end_idx, solver_type: MIPSolvers = MIPSolvers.CBC):
         """
         AC time series linear optimal power flow
         :param numerical_circuit: NumericalCircuit instance
         :param start_idx: start index of the time series
         :param end_idx: end index of the time series
-        :param solver: MIP solver_type to use
+        :param solver_type: MIP solver_type to use
         :param batteries_energy_0: initial state of the batteries, if None the default values are taken
         """
 
         OpfTimeSeries.__init__(self, numerical_circuit=numerical_circuit, start_idx=start_idx, end_idx=end_idx,
-                               solver=solver, skip_formulation=True)
+                               solver_type=solver_type, skip_formulation=True)
 
         self.v0 = None
         self.dva = None
         self.dvm = None
         self.Ql = None
 
-        self.problem = self.formulate(batteries_energy_0=batteries_energy_0)
-
     def formulate(self, batteries_energy_0=None):
         """
         Formulate the AC OPF time series in the non-sequential fashion (all to the solver_type at once)
@@ -427,7 +426,7 @@ if __name__ == '__main__':
     pf_options = PowerFlowOptions()
 
     options = OptimalPowerFlowOptions(solver=solver,
-                                      grouping=grouping,
+                                      time_grouping=grouping,
                                       mip_solver=mip_solver,
                                       power_flow_options=pf_options)
 
