@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 """
 This file implements a DC-OPF for time series
@@ -25,18 +27,18 @@ from GridCal.Engine.Simulations.OPF.opf_templates import OpfTimeSeries
 
 class OpfSimpleTimeSeries(OpfTimeSeries):
 
-    def __init__(self, numerical_circuit: OpfTimeCircuit, start_idx, end_idx, solver: MIPSolvers = MIPSolvers.CBC,
-                 batteries_energy_0=None, text_prog=None, prog_func=None):
+    def __init__(self, numerical_circuit: OpfTimeCircuit, start_idx, end_idx, solver_type: MIPSolvers = MIPSolvers.CBC,
+                 text_prog=None, prog_func=None):
         """
         DC time series linear optimal power flow
         :param numerical_circuit: NumericalCircuit instance
         :param start_idx: start index of the time series
         :param end_idx: end index of the time series
-        :param solver: MIP solver_type to use
+        :param solver_type: MIP solver_type to use
         :param batteries_energy_0: initial state of the batteries, if None the default values are taken
         """
         OpfTimeSeries.__init__(self, numerical_circuit=numerical_circuit, start_idx=start_idx, end_idx=end_idx,
-                               solver=solver)
+                               solver_type=solver_type)
 
         self.text_prog = text_prog
 
@@ -112,6 +114,12 @@ class OpfSimpleTimeSeries(OpfTimeSeries):
         self.Pb = Pb
         self.Pl = Pl
         self.E = E
+
+        self.Pinj = self.numerical_circuit.Sbus.transpose().real
+        self.hvdc_flow = np.zeros((nt, self.numerical_circuit.nhvdc))
+        self.hvdc_slacks = np.zeros((nt, self.numerical_circuit.nhvdc))
+        self.phase_shift = np.zeros((nt, m))
+
         self.load_shedding = np.zeros((nt, nl))
         self.s_from = np.zeros((nt, m))
         self.s_to = np.zeros((nt, m))
@@ -140,7 +148,7 @@ class OpfSimpleTimeSeries(OpfTimeSeries):
         """
         return self.s_from / self.rating.T
 
-    def get_branch_power(self):
+    def get_branch_power_from(self):
         """
         return the branch loading (time, device)
         :return: 2D array
@@ -207,7 +215,7 @@ if __name__ == '__main__':
     pf_options = PowerFlowOptions()
 
     options = OptimalPowerFlowOptions(solver=solver,
-                                      grouping=grouping,
+                                      time_grouping=grouping,
                                       mip_solver=mip_solver,
                                       power_flow_options=pf_options)
 
