@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from GridCal.Engine.basic_structures import BranchImpedanceMode, ReactivePowerControlMode, SolverType, TapsControlMode
 
@@ -28,7 +30,7 @@ class PowerFlowOptions:
         **retry_with_other_methods** (bool, True): Use a battery of methods to tackle
         the problem if the main solver_type fails
 
-        **verbose** (bool, False): Print additional details in the logger
+        **verbose** (int, 0): Print additional details in the console (0: no details, 1: some details, 2: all details)
 
         **initialize_with_existing_solution** (bool, True): *To be detailed*
 
@@ -74,7 +76,7 @@ class PowerFlowOptions:
     def __init__(self,
                  solver_type: SolverType = SolverType.NR,
                  retry_with_other_methods=True,
-                 verbose=False,
+                 verbose=0,
                  initialize_with_existing_solution=True,
                  tolerance=1e-6,
                  max_iter=25,
@@ -90,7 +92,8 @@ class PowerFlowOptions:
                  distributed_slack=False,
                  ignore_single_node_islands=False,
                  mu=1.0,
-                 backtracking_parameter=0.5):
+                 backtracking_parameter=0.05,
+                 use_stored_guess=False):
 
         self.solver_type = solver_type
 
@@ -130,5 +133,7 @@ class PowerFlowOptions:
 
         self.backtracking_parameter = backtracking_parameter
 
+        self.use_stored_guess = use_stored_guess
+
     def __str__(self):
         return "PowerFlowOptions"
