@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 import pandas as pd
@@ -23,7 +25,7 @@ from GridCal.Engine.Simulations.results_template import ResultsTemplate
 class NumericPowerFlowResults:
 
     def __init__(self, V, converged, norm_f, Scalc, ma=None, theta=None, Beq=None, Ybus=None, Yf=None, Yt=None,
-                 iterations=0, elapsed=0):
+                 iterations=0, elapsed=0, method=None):
         """
         Object to store the results returned by a numeric power flow routine
         :param V: Voltage vector
@@ -51,11 +53,13 @@ class NumericPowerFlowResults:
         self.Yt = Yt
         self.iterations = iterations
         self.elapsed = elapsed
+        self.method = None
 
 
 class PowerFlowResults(ResultsTemplate):
 
-    def __init__(self, n, m, n_tr, n_hvdc, bus_names, branch_names, transformer_names, hvdc_names, bus_types):
+    def __init__(self, n, m, n_tr, n_hvdc, bus_names, branch_names, transformer_names, hvdc_names, bus_types,
+                 area_names=None):
         """
         A **PowerFlowResults** object is create as an attribute of the
         :ref:`PowerFlowMP<pf_mp>` (as PowerFlowMP.results) when the power flow is run. It
@@ -101,7 +105,8 @@ class PowerFlowResults(ResultsTemplate):
 
                                                     ResultTypes.HvdcLosses,
                                                     ResultTypes.HvdcPowerFrom,
-                                                    ResultTypes.HvdcPowerTo],
+                                                    ResultTypes.HvdcPowerTo,
+                                                    ResultTypes.InterAreaExchange],
                                  data_variables=['bus_types',
                                                  'bus_names',
                                                  'branch_names',
@@ -138,6 +143,14 @@ class PowerFlowResults(ResultsTemplate):
         self.transformer_names = transformer_names
         self.hvdc_names = hvdc_names
 
+        # vars for the inter-area computation
+        self.F = None
+        self.T = None
+        self.hvdc_F = None
+        self.hvdc_T = None
+        self.bus_area_indices = None
+        self.area_names = area_names
+
         self.Sbus = np.zeros(n, dtype=complex)
 
         self.voltage = np.zeros(n, dtype=complex)
@@ -226,7 +239,6 @@ class PowerFlowResults(ResultsTemplate):
         val.loading = self.loading.copy()
         val.transformer_tap_module = self.transformer_tap_module.copy()
         val.losses = self.losses.copy()
-        val.overloads = self.overloads.copy()
 
         return val
 
@@ -286,6 +298,27 @@ class PowerFlowResults(ResultsTemplate):
 
         return df
 
+    def get_inter_area_flows(self):
+
+        na = len(self.area_names)
+        x = np.zeros((na, na), dtype=complex)
+
+        for f, t, flow in zip(self.F, self.T, self.Sf):
+            a1 = self.bus_area_indices[f]
+            a2 = self.bus_area_indices[t]
+            if a1 != a2:
+                x[a1, a2] += flow
+                x[a2, a1] -= flow
+
+        for f, t, flow in zip(self.hvdc_F, self.hvdc_T, self.hvdc_Pf):
+            a1 = self.bus_area_indices[f]
+            a2 = self.bus_area_indices[t]
+            if a1 != a2:
+                x[a1, a2] += flow
+                x[a2, a1] -= flow
+
+        return x
+
     def mdl(self, result_type: ResultTypes) -> "ResultsTable":
         """
 
@@ -293,179 +326,188 @@ class PowerFlowResults(ResultsTemplate):
         :return:
         """
 
+        columns = [result_type.value[0]]
+
         if result_type == ResultTypes.BusVoltageModule:
             labels = self.bus_names
             y = np.abs(self.voltage)
             y_label = '(p.u.)'
-            title = 'Bus voltage '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BusVoltageAngle:
             labels = self.bus_names
             y = np.angle(self.voltage, deg=True)
             y_label = '(deg)'
-            title = 'Bus voltage '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BusActivePower:
             labels = self.bus_names
             y = self.Sbus.real
             y_label = '(MW)'
-            title = 'Bus Active power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BusReactivePower:
             labels = self.bus_names
             y = self.Sbus.imag
             y_label = '(MVAr)'
-            title = 'Bus Reactive power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BusVoltagePolar:
             labels = self.bus_names
             y = self.voltage
             y_label = '(p.u.)'
-            title = 'Bus voltage '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchPower:
             labels = self.branch_names
             y = self.Sf
             y_label = '(MVA)'
-            title = 'Branch power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchActivePowerFrom:
             labels = self.branch_names
             y = self.Sf.real
             y_label = '(MW)'
-            title = 'Branch active power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchReactivePowerFrom:
             labels = self.branch_names
             y = self.Sf.imag
             y_label = '(MVAr)'
-            title = 'Branch reactive power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchActivePowerTo:
             labels = self.branch_names
             y = self.St.real
             y_label = '(MW)'
-            title = 'Branch active power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchReactivePowerTo:
             labels = self.branch_names
             y = self.St.imag
             y_label = '(MVAr)'
-            title = 'Branch reactive power '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.Transformer2WTapModule:
             labels = self.transformer_names
             y = self.transformer_tap_module
             y_label = '(p.u.)'
-            title = 'Transformer tap module '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchCurrent:
             labels = self.branch_names
             y = self.If
             y_label = '(p.u.)'
-            title = 'Branch current '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchActiveCurrentFrom:
             labels = self.branch_names
             y = self.If.real
             y_label = '(p.u.)'
-            title = 'Branch active current '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchReactiveCurrentFrom:
             labels = self.branch_names
             y = self.If.imag
             y_label = '(p.u.)'
-            title = 'Branch reactive current '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchActiveCurrentTo:
             labels = self.branch_names
             y = self.It.real
             y_label = '(p.u.)'
-            title = 'Branch active current '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchReactiveCurrentTo:
             labels = self.branch_names
             y = self.It.imag
             y_label = '(p.u.)'
-            title = 'Branch reactive current '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchLoading:
             labels = self.branch_names
             y = np.abs(self.loading) * 100
             y_label = '(%)'
-            title = 'Branch loading '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchLosses:
             labels = self.branch_names
             y = self.losses
             y_label = '(MVA)'
-            title = 'Branch losses '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchActiveLosses:
             labels = self.branch_names
             y = self.losses.real
             y_label = '(MW)'
-            title = 'Branch active losses '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchReactiveLosses:
             labels = self.branch_names
             y = self.losses.imag
             y_label = '(MVAr)'
-            title = 'Branch reactive losses '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchVoltage:
             labels = self.branch_names
             y = np.abs(self.Vbranch)
             y_label = '(p.u.)'
-            title = 'Branch voltage drop '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchAngles:
             labels = self.branch_names
             y = np.angle(self.Vbranch, deg=True)
             y_label = '(deg)'
-            title = 'Branch voltage angle '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchTapModule:
             labels = self.branch_names
             y = self.ma
             y_label = '(p.u.)'
-            title = 'Branch tap module '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchTapAngle:
             labels = self.branch_names
             y = self.theta
             y_label = '(rad)'
-            title = 'Branch tap angle '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.BranchBeq:
             labels = self.branch_names
             y = self.Beq
             y_label = '(p.u.)'
-            title = 'Branch Beq '
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.HvdcLosses:
             labels = self.hvdc_names
             y = self.hvdc_losses
             y_label = '(MW)'
-            title = result_type.value
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.HvdcPowerFrom:
             labels = self.hvdc_names
             y = self.hvdc_Pf
             y_label = '(MW)'
-            title = result_type.value
+            title = result_type.value[0]
 
         elif result_type == ResultTypes.HvdcPowerTo:
             labels = self.hvdc_names
             y = self.hvdc_Pt
             y_label = '(MW)'
-            title = result_type.value
+            title = result_type.value[0]
+
+        elif result_type == ResultTypes.InterAreaExchange:
+            labels = [a + '->' for a in self.area_names]
+            columns = ['->' + a for a in self.area_names]
+            y = self.get_inter_area_flows().real
+            y_label = '(MW)'
+            title = result_type.value[0]
 
         else:
             raise Exception('Unsupported result type: ' + str(result_type))
 
         # assemble model
-        mdl = ResultsTable(data=y, index=labels, columns=[result_type.value[0]],
+        mdl = ResultsTable(data=y, index=labels, columns=columns,
                            title=title, ylabel=y_label, units=y_label)
         return mdl
 
@@ -484,7 +526,10 @@ class PowerFlowResults(ResultsTemplate):
         vr = self.voltage.real
         vi = self.voltage.imag
         bus_data = np.c_[vr, vi, vm, va]
-        bus_cols = ['Real voltage (p.u.)', 'Imag Voltage (p.u.)', 'Voltage module (p.u.)', 'Voltage angle (rad)']
+        bus_cols = ['Real voltage (p.u.)',
+                    'Imag Voltage (p.u.)',
+                    'Voltage module (p.u.)',
+                    'Voltage angle (rad)']
         df_bus = pd.DataFrame(data=bus_data, columns=bus_cols)
 
         # branch results
@@ -495,11 +540,20 @@ class PowerFlowResults(ResultsTemplate):
         la = self.losses.real
         lr = self.losses.imag
         ls = np.abs(self.losses)
-        tm = self.transformer_tap_module
+        if self.transformer_tap_module.size == 0:
+            tm = [np.nan] * sr.size
+        else:
+            tm = self.transformer_tap_module
 
         branch_data = np.c_[sr, si, sm, ld, la, lr, ls, tm]
-        branch_cols = ['Real power (MW)', 'Imag power (MVAr)', 'Power module (MVA)', 'Loading(%)',
-                       'Losses (MW)', 'Losses (MVAr)', 'Losses (MVA)', 'Tap module']
+        branch_cols = ['Real power (MW)',
+                       'Imag power (MVAr)',
+                       'Power module (MVA)',
+                       'Loading(%)',
+                       'Losses (MW)',
+                       'Losses (MVAr)',
+                       'Losses (MVA)',
+                       'Tap module']
         df_branch = pd.DataFrame(data=branch_data, columns=branch_cols)
 
         return df_bus, df_branch
