@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 from scipy.sparse.linalg import inv
@@ -21,6 +23,7 @@ from GridCal.Engine.Simulations.ShortCircuitStudies.short_circuit import short_c
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.basic_structures import BranchImpedanceMode
 from GridCal.Engine.Simulations.PowerFlow.power_flow_driver import PowerFlowResults, PowerFlowOptions
+from GridCal.Engine.Simulations.PowerFlow.power_flow_worker import power_flow_post_process
 from GridCal.Engine.Core.snapshot_pf_data import SnapshotData
 from GridCal.Engine.Simulations.result_types import ResultTypes
 from GridCal.Engine.Devices import Branch, Bus
@@ -36,29 +39,42 @@ from GridCal.Engine.Simulations.driver_template import DriverTemplate
 
 class ShortCircuitOptions:
 
-    def __init__(self, bus_index=[], branch_index=[], branch_fault_locations=[], branch_fault_impedance=[],
+    def __init__(self, bus_index=None, branch_index=None, branch_fault_locations=None, branch_fault_impedance=None,
                  branch_impedance_tolerance_mode=BranchImpedanceMode.Specified,
                  verbose=False):
         """
 
-        Args:
-            bus_index:
-            branch_index:
-            branch_fault_locations:
-            branch_fault_impedance:
-            verbose:
+        :param bus_index:
+        :param branch_index:
+        :param branch_fault_locations:
+        :param branch_fault_impedance:
+        :param branch_impedance_tolerance_mode:
+        :param verbose:
         """
 
-        assert (len(branch_fault_locations) == len(branch_index))
-        assert (len(branch_fault_impedance) == len(branch_index))
+        if branch_index is not None:
+            assert (len(branch_fault_locations) == len(branch_index))
+            assert (len(branch_fault_impedance) == len(branch_index))
 
-        self.bus_index = bus_index
+        if bus_index is None:
+            self.bus_index = list()
+        else:
+            self.bus_index = bus_index
 
-        self.branch_index = branch_index
+        if branch_index is None:
+            self.branch_index = list()
+        else:
+            self.branch_index = branch_index
 
-        self.branch_fault_locations = branch_fault_locations
+        if branch_fault_locations is None:
+            self.branch_fault_locations = list()
+        else:
+            self.branch_fault_locations = branch_fault_locations
 
-        self.branch_fault_impedance = branch_fault_impedance
+        if branch_fault_impedance is None:
+            self.branch_fault_impedance = list()
+        else:
+            self.branch_fault_impedance = branch_fault_impedance
 
         self.branch_impedance_tolerance_mode = branch_impedance_tolerance_mode
 
@@ -251,7 +267,7 @@ class ShortCircuitDriver(DriverTemplate):
 
     def single_short_circuit(self, calculation_inputs: SnapshotData, Vpf, Zf):
         """
-        Run a power flow simulation for a single circuit
+        Run a short circuit simulation for a single island
         @param calculation_inputs:
         @param Vpf: Power flow voltage vector applicable to the island
         @param Zf: Short circuit impedance vector applicable to the island
@@ -261,7 +277,7 @@ class ShortCircuitDriver(DriverTemplate):
         # is dense, so no need to store it as sparse
         if calculation_inputs.Ybus.shape[0] > 1:
 
-            Zbus = inv(calculation_inputs.Ybus).toarray()
+            Zbus = inv(calculation_inputs.Ybus.tocsc()).toarray()
 
             # Compute the short circuit
             V, SCpower = short_circuit_3p(bus_idx=self.options.bus_index,
@@ -271,7 +287,14 @@ class ShortCircuitDriver(DriverTemplate):
                                           baseMVA=calculation_inputs.Sbase)
 
             # Compute the branches power
-            Sf, If, loading, losses = self.compute_branch_results(calculation_inputs=calculation_inputs, V=V)
+            # Sf, If, loading, losses = self.compute_branch_results(calculation_inputs=calculation_inputs, V=V)
+            Sfb, Stb, If, It, Vbranch, \
+            loading, losses, Sbus = power_flow_post_process(calculation_inputs=calculation_inputs,
+                                                            Sbus=calculation_inputs.Sbus,
+                                                            V=V,
+                                                            branch_rates=calculation_inputs.branch_rates,
+                                                            Yf=calculation_inputs.Yf,
+                                                            Yt=calculation_inputs.Yt)
 
             # voltage, Sf, loading, losses, error, converged, Qpv
             results = ShortCircuitResults(n=calculation_inputs.nbus,
@@ -282,13 +305,22 @@ class ShortCircuitDriver(DriverTemplate):
                                           transformer_names=calculation_inputs.tr_names,
                                           bus_types=calculation_inputs.bus_types)
 
-            results.Sbus = calculation_inputs.Sbus
-            results.voltage = V
-            results.Sf = Sf
-            results.If = If
-            results.losses = losses
             results.SCpower = SCpower
-
+            results.Sbus = calculation_inputs.Sbus * calculation_inputs.Sbase  # MVA
+            results.voltage = V
+            results.Sf = Sfb  # in MVA already
+            results.St = Stb  # in MVA already
+            results.If = If  # in p.u.
+            results.It = It  # in p.u.
+            # results.ma = calculation_inputs.ma
+            # results.theta = calculation_inputs.theta
+            # results.Beq = calculation_inputs.Beq
+            results.Vbranch = Vbranch
+            results.loading = loading
+            results.losses = losses
+            # results.transformer_tap_module = solution.ma[circuit.transformer_idx]
+            # results.convergence_reports.append(report)
+            # results.Qpv = Sbus.imag[circuit.pv]
 
         else:
             nbus = calculation_inputs.Ybus.shape[0]
@@ -312,25 +344,6 @@ class ShortCircuitDriver(DriverTemplate):
 
         return results
 
-    @staticmethod
-    def compute_branch_results(calculation_inputs: SnapshotData, V):
-        """
-        Compute the power Sf trough the branches
-        @param calculation_inputs: instance of Circuit
-        @param V: Voltage solution array for the circuit buses
-        @return: Sf, If, loading, losses
-        """
-        If = calculation_inputs.Yf * V
-        It = calculation_inputs.Yt * V
-        Sf = (calculation_inputs.Cf * V) * np.conj(If)
-        St = (calculation_inputs.Ct * V) * np.conj(It)
-        losses = Sf - St
-        # Ibranch = np.maximum(If, It)
-        # Sbranch = np.maximum(Sf, St)
-        loading = Sf * calculation_inputs.Sbase / (calculation_inputs.branch_rates + 1e-20)
-
-        return Sf, If, loading, losses
-
     def run(self):
         """
         Run a power flow for every circuit
@@ -404,9 +417,5 @@ class ShortCircuitDriver(DriverTemplate):
         self.grid.short_circuit_results = results
         self._is_running = False
 
-        self.progress_signal.emit(0.0)
-        self.progress_text.emit('Done!')
-        self.done_signal.emit()
-
     def isRunning(self):
         return self._is_running
