@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 import numba as nb
 from matplotlib import pyplot as plt
@@ -22,7 +24,7 @@ from GridCal.Engine.Simulations.results_table import ResultsTable
 from GridCal.Engine.Simulations.result_types import ResultTypes
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.Core.snapshot_pf_data import compile_snapshot_circuit
-from GridCal.Engine.Simulations.PowerFlow.helm_power_flow import helm_coefficients_josep, sigma_function
+from GridCal.Engine.Simulations.PowerFlow.NumericalMethods.helm_power_flow import helm_coefficients_josep, sigma_function
 from GridCal.Engine.Simulations.driver_template import DriverTemplate
 from GridCal.Engine.Simulations.driver_types import SimulationTypes
 
@@ -216,17 +218,19 @@ def multi_island_sigma(multi_circuit: MultiCircuit, options: PowerFlowOptions, l
 
             if len(calculation_input.vd) > 0:
                 # V, converged, norm_f, Scalc, iter_, elapsed, Sig_re, Sig_im
-                U, X, Q, iter_ = helm_coefficients_josep(Yseries=calculation_input.Yseries,
-                                                         V0=calculation_input.Vbus,
-                                                         S0=calculation_input.Sbus,
-                                                         Ysh0=calculation_input.Yshunt,
-                                                         pq=calculation_input.pq,
-                                                         pv=calculation_input.pv,
-                                                         sl=calculation_input.vd,
-                                                         pqpv=calculation_input.pqpv,
-                                                         tolerance=options.tolerance,
-                                                         max_coeff=options.max_iter,
-                                                         verbose=False,)
+                U, X, Q, V, iter_ = helm_coefficients_josep(Ybus=calculation_input.Ybus,
+                                                            Yseries=calculation_input.Yseries,
+                                                            V0=calculation_input.Vbus,
+                                                            S0=calculation_input.Sbus,
+                                                            Ysh0=calculation_input.Yshunt,
+                                                            pq=calculation_input.pq,
+                                                            pv=calculation_input.pv,
+                                                            sl=calculation_input.vd,
+                                                            pqpv=calculation_input.pqpv,
+                                                            tolerance=options.tolerance,
+                                                            max_coeff=options.max_iter,
+                                                            verbose=False,
+                                                            logger=logger)
 
                 # compute the sigma values
                 n = calculation_input.nbus
@@ -258,17 +262,19 @@ def multi_island_sigma(multi_circuit: MultiCircuit, options: PowerFlowOptions, l
             # only one island
             calculation_input = calculation_inputs[0]
 
-            U, X, Q, iter_ = helm_coefficients_josep(Yseries=calculation_input.Yseries,
-                                                     V0=calculation_input.Vbus,
-                                                     S0=calculation_input.Sbus,
-                                                     Ysh0=calculation_input.Yshunt,
-                                                     pq=calculation_input.pq,
-                                                     pv=calculation_input.pv,
-                                                     sl=calculation_input.vd,
-                                                     pqpv=calculation_input.pqpv,
-                                                     tolerance=options.tolerance,
-                                                     max_coeff=options.max_iter,
-                                                     verbose=False, )
+            U, X, Q, V, iter_ = helm_coefficients_josep(Ybus=calculation_input.Ybus,
+                                                        Yseries=calculation_input.Yseries,
+                                                        V0=calculation_input.Vbus,
+                                                        S0=calculation_input.Sbus,
+                                                        Ysh0=calculation_input.Yshunt,
+                                                        pq=calculation_input.pq,
+                                                        pv=calculation_input.pv,
+                                                        sl=calculation_input.vd,
+                                                        pqpv=calculation_input.pqpv,
+                                                        tolerance=options.tolerance,
+                                                        max_coeff=options.max_iter,
+                                                        verbose=False,
+                                                        logger=logger)
 
             # compute the sigma values
             n = calculation_input.nbus
@@ -294,7 +300,7 @@ def multi_island_sigma(multi_circuit: MultiCircuit, options: PowerFlowOptions, l
     return results
 
 
-@nb.jit()
+@nb.jit(cache=True)
 def sigma_distance(sigma_real, sigma_imag):
     """
     Distance to the collapse in the sigma space
@@ -400,11 +406,6 @@ class SigmaAnalysisDriver(DriverTemplate):
                                           options=self.options,
                                           logger=self.logger)
 
-        # send the finnish signal
-        self.progress_signal.emit(0.0)
-        self.progress_text.emit('Done!')
-        self.done_signal.emit()
-
     def cancel(self):
         self.__cancel__ = True
 
