@@ -1,23 +1,27 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 
+import GridCal.Engine.basic_structures as bs
 from GridCal.Engine.Simulations.StateEstimation.state_estimation import solve_se_lm
 from GridCal.Engine.Simulations.PowerFlow.power_flow_worker import PowerFlowResults, power_flow_post_process
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
+from GridCal.Engine.Core.snapshot_pf_data import compile_snapshot_circuit
 from GridCal.Engine.Devices.measurement import MeasurementType
 from GridCal.Engine.Simulations.driver_template import DriverTemplate
 
@@ -140,7 +144,7 @@ class StateEstimation(DriverTemplate):
 
         DriverTemplate.__init__(self, grid=circuit)
 
-        self.se_results = None
+        self.results = None
 
     @staticmethod
     def collect_measurements(circuit: MultiCircuit, bus_idx, branch_idx):
@@ -204,13 +208,19 @@ class StateEstimation(DriverTemplate):
         """
         n = len(self.grid.buses)
         m = self.grid.get_branch_number()
-        self.se_results = StateEstimationResults()
-        self.se_results.initialize(n, m)
 
-        numerical_circuit = self.grid.compile_snapshot()
-        islands = numerical_circuit.compute()
+        numerical_circuit = compile_snapshot_circuit(self.grid)
+        self.results = StateEstimationResults(n=n, m=m,
+                                              n_tr=numerical_circuit.ntr,
+                                              bus_names=numerical_circuit.bus_names,
+                                              branch_names=numerical_circuit.branch_names,
+                                              transformer_names=numerical_circuit.transformer_data.tr_names,
+                                              bus_types=numerical_circuit.bus_types)
+        # self.se_results.initialize(n, m)
 
-        self.se_results.bus_types = numerical_circuit.bus_types
+        islands = numerical_circuit.split_into_islands()
+
+        self.results.bus_types = numerical_circuit.bus_types
 
         for island in islands:
 
@@ -220,34 +230,49 @@ class StateEstimation(DriverTemplate):
                                                  branch_idx=island.original_branch_idx)
 
             # run solver
-            v_sol, err, converged = solve_se_lm(Ybus=island.Ybus,
-                                                Yf=island.Yf,
-                                                Yt=island.Yt,
-                                                f=island.F,
-                                                t=island.T,
-                                                se_input=se_input,
-                                                ref=island.ref,
-                                                pq=island.pq,
-                                                pv=island.pv)
+            report = bs.ConvergenceReport()
+            solution = solve_se_lm(Ybus=island.Ybus,
+                                   Yf=island.Yf,
+                                   Yt=island.Yt,
+                                   f=island.F,
+                                   t=island.T,
+                                   se_input=se_input,
+                                   ref=island.vd,
+                                   pq=island.pq,
+                                   pv=island.pv)
+
+            report.add(method=bs.SolverType.LM,
+                       converged=solution.converged,
+                       error=solution.norm_f,
+                       elapsed=solution.elapsed,
+                       iterations=solution.iterations)
 
             # Compute the branches power and the slack buses power
-            Sfb, Stb, If, It, Vbrnach, loading, \
-             losses, Sbus = power_flow_post_process(calculation_inputs=island, V=v_sol)
+            Sfb, Stb, If, It, Vbranch, loading, losses, Sbus = power_flow_post_process(calculation_inputs=island,
+                                                                                       Sbus=island.Sbus,
+                                                                                       V=solution.V,
+                                                                                       branch_rates=island.branch_rates,
+                                                                                       Yf=None, Yt=None)
 
             # pack results into a SE results object
-            results = StateEstimationResults(Sbus=Sbus,
-                                             voltage=v_sol,
-                                             Sbranch=Sfb,
-                                             Ibranch=If,
-                                             loading=loading,
-                                             losses=losses,
-                                             error=[err],
-                                             converged=[converged],
-                                             Qpv=None)
+            results = StateEstimationResults(n=island.nbus,
+                                             m=island.nbr,
+                                             n_tr=island.ntr,
+                                             bus_names=island.bus_names,
+                                             branch_names=island.branch_names,
+                                             transformer_names=island.transformer_data.tr_names,
+                                             bus_types=island.bus_types)
+            results.Sbus = Sbus
+            results.Sf = Sfb
+            results.voltage = solution.V
+            results.losses = losses
+            results.loading = loading
+            results.convergence_reports.append(report)
 
-            self.se_results.apply_from_island(results,
-                                              island.original_bus_idx,
-                                              island.original_branch_idx)
+            self.results.apply_from_island(results,
+                                           island.original_bus_idx,
+                                           island.original_branch_idx,
+                                           island.original_tr_idx)
 
 
 if __name__ == '__main__':
