@@ -1,17 +1,19 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from enum import Enum
 from GridCal.Engine.Devices import DeviceType
@@ -51,6 +53,7 @@ class ResultTypes(Enum):
 
     HvdcLosses = 'HVDC losses', DeviceType.HVDCLineDevice
     HvdcPowerFrom = 'HVDC power "from"', DeviceType.HVDCLineDevice
+    HvdcLoading = 'HVDC loading', DeviceType.HVDCLineDevice
     HvdcPowerTo = 'HVDC power "to"', DeviceType.HVDCLineDevice
 
     # StochasticPowerFlowDriver
@@ -72,18 +75,21 @@ class ResultTypes(Enum):
     BusVoltageModule = 'Bus voltage module', DeviceType.BusDevice
     BusVoltageAngle = 'Bus voltage angle', DeviceType.BusDevice
     BusPower = 'Bus power', DeviceType.BusDevice
-    ShadowPrices = 'Bus shadow prices', DeviceType.BusDevice
+    BusShadowPrices = 'Bus shadow prices', DeviceType.BusDevice
     BranchOverloads = 'Branch overloads', DeviceType.BranchDevice
     LoadShedding = 'Load shedding', DeviceType.LoadDevice
-    ControlledGeneratorShedding = 'Generator shedding', DeviceType.GeneratorDevice
-    ControlledGeneratorPower = 'Generator power', DeviceType.GeneratorDevice
+    GeneratorShedding = 'Generator shedding', DeviceType.GeneratorDevice
+    GeneratorPower = 'Generator power', DeviceType.GeneratorDevice
 
     # OPF-NTC
     HvdcOverloads = 'HVDC overloads', DeviceType.HVDCLineDevice
     NodeSlacks = 'Nodal slacks', DeviceType.BusDevice
     GenerationDelta = 'Generation deltas', DeviceType.GeneratorDevice
+    GenerationDeltaSlacks = 'Generation delta slacks', DeviceType.GeneratorDevice
     InterAreaExchange = 'Inter-Area exchange', DeviceType.NoDevice
 
+    OptimalNetTransferCapacityTimeSeriesReport = 'Optimal net transfer capacity series report', DeviceType.NoDevice
+
     # Short-circuit
     BusShortCircuitPower = 'Bus short circuit power', DeviceType.BusDevice
 
@@ -94,6 +100,8 @@ class ResultTypes(Enum):
     OTDF = 'Outage transfer distribution factors', DeviceType.BranchDevice
 
     MaxOverloads = 'Maximum contingency flow', DeviceType.BranchDevice
+    ContingencyFlows = 'Contingency flow', DeviceType.BranchDevice
+    ContingencyLoading = 'Contingency loading', DeviceType.BranchDevice
     WorstContingencyFlows = 'Worst contingency Sf', DeviceType.BranchDevice
     WorstContingencyLoading = 'Worst contingency loading', DeviceType.BranchDevice
     ContingencyFrequency = 'Contingency frequency', DeviceType.BranchDevice
@@ -118,6 +126,16 @@ class ResultTypes(Enum):
     NetTransferCapacity = 'Net transfer capacity', DeviceType.BranchDevice
     AvailableTransferCapacityReport = 'ATC Report', DeviceType.NoDevice
 
+    ContingencyFlowsReport = 'Contingency Report', DeviceType.NoDevice
+    ContingencyFlowsBranchReport = 'Contingency Branch Report', DeviceType.NoDevice
+    ContingencyFlowsGenerationReport = 'Contingency Generation Report', DeviceType.NoDevice
+    ContingencyFlowsHvdcReport = 'Contingency Hvdc Report', DeviceType.NoDevice
+
+    # inputs analysis
+    ZoneAnalysis = 'Zone analysis', DeviceType.NoDevice
+    CountryAnalysis = 'Country analysis', DeviceType.NoDevice
+    AreaAnalysis = 'Area analysis', DeviceType.NoDevice
+
     def __str__(self):
         return self.value
 
