@@ -1,18 +1,20 @@
 # -*- coding: utf-8 -*-
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 from enum import Enum
 from scipy.sparse import csr_matrix, csc_matrix
@@ -66,7 +68,7 @@ except ImportError:
     # print(SparseSolver.Pardiso.value + ' failed')
 
 try:
-    from scikits.umfpack import spsolve, splu
+    from scikits.umfpack import spsolve_umfpack
 
     available_sparse_solvers.append(SparseSolver.UMFPACK)
 except ImportError:
@@ -153,7 +155,7 @@ def umfpack_linsolve(A, b):
     :param b:
     :return:
     """
-    return spsolve(A, b)
+    return spsolve_umfpack(A, b)
 
 
 def get_linear_solver(solver_type: SparseSolver = preferred_type):
