@@ -1,18 +1,20 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
-
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+import numpy as np
 from PySide2.QtWidgets import *
 from PySide2.QtCore import *
 from PySide2.QtGui import *
@@ -125,6 +127,14 @@ class BusGraphicItem(QGraphicsRectItem):
         # Update size:
         self.change_size(self.w, self.h)
 
+    def set_label(self, val: str):
+        """
+        Set the label content
+        :param val:
+        :return:
+        """
+        self.label.setPlainText(val)
+
     def mouseMoveEvent(self, event: 'QGraphicsSceneMouseEvent'):
         """
         On mouse move of this object...
@@ -162,8 +172,11 @@ class BusGraphicItem(QGraphicsRectItem):
         :param x: x in pixels
         :param y: y in pixels
         """
-        # self.setPos(self.editor.diagramView.mapToScene(QPoint(x, y)))
-        self.setPos(QPoint(x, y))
+        if np.isnan(x):
+            x = 0
+        if np.isnan(y):
+            y = 0
+        self.setPos(QPoint(int(x), int(y)))
 
     def set_tile_color(self, brush):
         """
@@ -219,7 +232,7 @@ class BusGraphicItem(QGraphicsRectItem):
         y0 = h + self.offset
         x0 = 0
         self.terminal.setPos(x0, y0)
-        self.terminal.setRect(0.0, 0.0, w, 10)
+        self.terminal.setRect(0, 0, w, 10)
 
         # Set text
         if self.api_object is not None:
@@ -284,18 +297,6 @@ class BusGraphicItem(QGraphicsRectItem):
         pe.setChecked(self.api_object.active)
         pe.triggered.connect(self.enable_disable_toggle)
 
-        pl = menu.addAction('Plot profiles')
-        plot_icon = QIcon()
-        plot_icon.addPixmap(QPixmap(":/Icons/icons/plot.svg"))
-        pl.setIcon(plot_icon)
-        pl.triggered.connect(self.plot_profiles)
-
-        arr = menu.addAction('Arrange')
-        arr_icon = QIcon()
-        arr_icon.addPixmap(QPixmap(":/Icons/icons/automatic_layout.svg"))
-        arr.setIcon(arr_icon)
-        arr.triggered.connect(self.arrange_children)
-
         sc = menu.addAction('Short circuit')
         sc_icon = QIcon()
         sc_icon.addPixmap(QPixmap(":/Icons/icons/short_circuit.svg"))
@@ -312,6 +313,25 @@ class BusGraphicItem(QGraphicsRectItem):
         dc.setChecked(self.api_object.is_dc)
         dc.triggered.connect(self.enable_disable_dc)
 
+        pl = menu.addAction('Plot profiles')
+        plot_icon = QIcon()
+        plot_icon.addPixmap(QPixmap(":/Icons/icons/plot.svg"))
+        pl.setIcon(plot_icon)
+        pl.triggered.connect(self.plot_profiles)
+
+        arr = menu.addAction('Arrange')
+        arr_icon = QIcon()
+        arr_icon.addPixmap(QPixmap(":/Icons/icons/automatic_layout.svg"))
+        arr.setIcon(arr_icon)
+        arr.triggered.connect(self.arrange_children)
+
+        ra5 = menu.addAction('Assign active state to profile')
+        ra5_icon = QIcon()
+        ra5_icon.addPixmap(QPixmap(":/Icons/icons/assign_to_profile.svg"))
+        ra5.setIcon(ra5_icon)
+        ra5.triggered.connect(self.assign_status_to_profile)
+
+
         ra3 = menu.addAction('Delete all the connections')
         del2_icon = QIcon()
         del2_icon.addPixmap(QPixmap(":/Icons/icons/delete_conn.svg"))
@@ -364,6 +384,12 @@ class BusGraphicItem(QGraphicsRectItem):
 
         menu.exec_(event.screenPos())
 
+    def assign_status_to_profile(self):
+        """
+        Assign the snapshot rate to the profile
+        """
+        self.diagramScene.set_active_status_to_profile(self.api_object)
+
     def delete_all_connections(self):
         """
         Delete all bus connections
@@ -399,27 +425,41 @@ class BusGraphicItem(QGraphicsRectItem):
             self.diagramScene.removeItem(self)
             self.diagramScene.circuit.delete_bus(self.api_object, ask)
 
+    def update_color(self):
+        if self.api_object.active:
+            self.set_tile_color(QBrush(ACTIVE['color']))
+        else:
+            self.set_tile_color(QBrush(DEACTIVATED['color']))
+
     def enable_disable_toggle(self):
         """
         Toggle bus element state
         @return:
         """
         if self.api_object is not None:
+
+            # change the bus state (snapshot)
             self.api_object.active = not self.api_object.active
 
-            if self.api_object.active:
+            # change the branches state (snapshot)
+            for host in self.terminal.hosting_connections:
+                if host.api_object is not None:
+                    host.set_enable(val=self.api_object.active)
 
-                self.set_tile_color(QBrush(ACTIVE['color']))
+            self.update_color()
 
-                for host in self.terminal.hosting_connections:
-                    if host.api_object is not None:
-                        host.set_enable(val=True)
-            else:
-                self.set_tile_color(QBrush(DEACTIVATED['color']))
+            if self.diagramScene.circuit.has_time_series:
+                ok = yes_no_question('Do you want to update the time series active status accordingly?',
+                                     'Update time series active status')
 
-                for host in self.terminal.hosting_connections:
-                    if host.api_object is not None:
-                        host.set_enable(val=False)
+                if ok:
+                    # change the bus state (time series)
+                    self.diagramScene.set_active_status_to_profile(self.api_object, override_question=True)
+
+                    # change the branches state (time series)
+                    for host in self.terminal.hosting_connections:
+                        if host.api_object is not None:
+                            self.diagramScene.set_active_status_to_profile(host.api_object, override_question=True)
 
     def enable_disable_sc(self):
         """
