@@ -1,24 +1,28 @@
-# This file is part of GridCal.
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
 #
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
 #
-# GridCal is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import sys
 import os
 import numpy as np
 import pandas as pd
-
+from typing import List
 import networkx as nx
 from warnings import warn
+
+
 from PySide2.QtWidgets import *
 from PySide2.QtCore import *
 from PySide2.QtGui import *
@@ -31,6 +35,7 @@ from GridCal.Engine.Devices.transformer import Transformer2W
 from GridCal.Engine.Devices.vsc import VSC
 from GridCal.Engine.Devices.upfc import UPFC
 from GridCal.Engine.Devices.hvdc_line import HvdcLine
+from GridCal.Engine.Devices.transformer3w import Transformer3W
 from GridCal.Gui.GridEditorWidget.terminal_item import TerminalItem
 from GridCal.Gui.GridEditorWidget.bus_graphics import BusGraphicItem
 from GridCal.Gui.GridEditorWidget.line_graphics import LineGraphicItem
@@ -39,6 +44,7 @@ from GridCal.Gui.GridEditorWidget.transformer2w_graphics import TransformerGraph
 from GridCal.Gui.GridEditorWidget.hvdc_graphics import HvdcGraphicItem
 from GridCal.Gui.GridEditorWidget.vsc_graphics import VscGraphicItem
 from GridCal.Gui.GridEditorWidget.upfc_graphics import UpfcGraphicItem
+from GridCal.Gui.GridEditorWidget.transformer3w_graphics import Transformer3WGraphicItem
 from GridCal.Engine.Simulations.driver_types import SimulationTypes
 from matplotlib import pyplot as plt
 
@@ -60,6 +66,12 @@ The graphic objects need to call the API objects and functions inside the MultiC
 To do this the graphic objects call "parent.circuit.<function or object>"
 '''
 
+def toQBytesArray(val: str):
+    data = QByteArray()
+    stream = QDataStream(data, QIODevice.WriteOnly)
+    stream.writeQString(val)
+    return data
+
 
 class EditorGraphicsView(QGraphicsView):
 
@@ -71,7 +83,7 @@ class EditorGraphicsView(QGraphicsView):
         @param editor:
         """
         QGraphicsView.__init__(self, scene, parent)
-
+        self._zoom = 0
         self.setDragMode(QGraphicsView.RubberBandDrag)
         self.setRubberBandSelectionMode(Qt.IntersectsItemShape)
         self.setMouseTracking(True)
@@ -114,10 +126,10 @@ class EditorGraphicsView(QGraphicsView):
         if event.mimeData().hasFormat('component/name'):
             obj_type = event.mimeData().data('component/name')
             elm = None
-            data = QByteArray()
-            stream = QDataStream(data, QIODevice.WriteOnly)
-            stream.writeQString('Bus')
-            if obj_type == data:
+            bus_data = toQBytesArray('Bus')
+            tr3w_data = toQBytesArray('3W-Transformer')
+
+            if bus_data == obj_type:
                 name = 'Bus ' + str(len(self.scene_.circuit.buses))
 
                 obj = Bus(name=name,
@@ -130,6 +142,14 @@ class EditorGraphicsView(QGraphicsView):
                 obj.graphic_obj = elm
                 self.scene_.circuit.add_bus(obj)  # weird but it's the only way to have graphical-API communication
 
+            elif tr3w_data == obj_type:
+                name = "Transformer 3-windings" + str(len(self.scene_.circuit.transformers3w))
+                obj = Transformer3W(name=name)
+                elm = Transformer3WGraphicItem(diagramScene=self.scene(), editor=self.editor, elm=obj)
+                obj.graphic_obj = elm
+                self.scene_.circuit.add_transformer3w(obj)  # weird but it's the only way to have graphical-API communication
+                print('3w transformer dropped')
+
             if elm is not None:
                 elm.setPos(self.mapToScene(event.pos()))
                 self.scene_.addItem(elm)
@@ -171,6 +191,7 @@ class EditorGraphicsView(QGraphicsView):
 class LibraryModel(QStandardItemModel):
     """
     Items model to host the draggable icons
+    This is the list of draggable items
     """
 
     def __init__(self, parent=None):
@@ -278,7 +299,6 @@ class DiagramScene(QGraphicsScene):
         """
         fig = plt.figure(figsize=(12, 8))
         ax_1 = fig.add_subplot(211)
-        # ax_2 = fig.add_subplot(212, sharex=ax_1)
         ax_2 = fig.add_subplot(212)
 
         # set time
@@ -313,10 +333,10 @@ class DiagramScene(QGraphicsScene):
                                 power_data[key.value] = driver.results.Sf.real[:, i]
                                 loading_data[key.value] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
 
-                            elif key == SimulationTypes.NetTransferCapacityTS_run:
-                                power_data[key.value] = driver.results.atc[:, i]
-                                atc_perc = driver.results.atc[:, i] / (api_object.rate_prof + 1e-9)
-                                loading_data[key.value] = np.sort(np.abs(atc_perc * 100.0))
+                            # elif key == SimulationTypes.NetTransferCapacityTS_run:
+                            #     power_data[key.value] = driver.results.atc[:, i]
+                            #     atc_perc = driver.results.atc[:, i] / (api_object.rate_prof + 1e-9)
+                            #     loading_data[key.value] = np.sort(np.abs(atc_perc * 100.0))
 
                             elif key == SimulationTypes.ContingencyAnalysisTS_run:
                                 power_data[key.value] = driver.results.worst_flows.real[:, i]
@@ -381,38 +401,103 @@ class DiagramScene(QGraphicsScene):
                 # plot the profiles
                 plt.show()
 
-    def plot_hvdc_branch(self, api_obj):
+    def plot_hvdc_branch(self, i, api_object):
         """
 
-        :param api_obj:
+        :param api_object:
         :return:
         """
         fig = plt.figure(figsize=(12, 8))
-
         ax_1 = fig.add_subplot(211)
-        ax_2 = fig.add_subplot(212, sharex=ax_1)
+        # ax_2 = fig.add_subplot(212, sharex=ax_1)
+        ax_2 = fig.add_subplot(212)
 
+        # set time
         x = self.circuit.time_profile
+        x_cl = x
 
         if x is not None:
             if len(x) > 0:
-                # loading
-                y = api_obj.Pset_prof / (api_obj.rate_prof + 1e-9) * 100.0
-                df = pd.DataFrame(data=y, index=x, columns=[api_obj.name])
-                ax_1.set_title('Loading', fontsize=14)
-                ax_1.set_ylabel('Loading [%]', fontsize=11)
-                df.plot(ax=ax_1)
 
-                # losses
-                y = api_obj.Pset_prof * api_obj.loss_factor
-                df = pd.DataFrame(data=y, index=x, columns=[api_obj.name])
-                ax_2.set_title('Losses', fontsize=14)
-                ax_2.set_ylabel('Losses [MVA]', fontsize=11)
-                df.plot(ax=ax_2)
+                p = np.arange(len(x)).astype(float) / len(x)
+
+                # search available results
+                power_data = dict()
+                loading_data = dict()
+                loading_st_data = None
+                loading_clustering_data = None
+                power_clustering_data = None
+
+                for key, driver in self.results_dictionary.items():
+                    if hasattr(driver, 'results'):
+                        if driver.results is not None:
+                            if key == SimulationTypes.TimeSeries_run:
+                                power_data[key.value] = driver.results.hvdc_Pf[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.hvdc_loading[:, i] * 100.0))
+
+                            elif key == SimulationTypes.ClusteringTimeSeries_run:
+                                x_cl = x[driver.sampled_time_idx]
+                                power_clustering_data = driver.results.hvdc_Pf[:, i]
+                                loading_clustering_data = np.sort(np.abs(driver.results.hvdc_loading[:, i] * 100.0))
+
+                            elif key == SimulationTypes.LinearAnalysis_TS_run:
+                                power_data[key.value] = driver.results.hvdc_Pf[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.hvdc_loading[:, i] * 100.0))
+
+                            elif key == SimulationTypes.OPFTimeSeries_run:
+                                power_data[key.value] = driver.results.hvdc_Pf[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.hvdc_loading[:, i] * 100.0))
+
+                # add the rating
+                # power_data['Rates+'] = api_object.rate_prof
+                # power_data['Rates-'] = -api_object.rate_prof
+
+                # loading
+                if len(loading_data.keys()):
+                    df = pd.DataFrame(data=loading_data, index=p)
+                    ax_1.set_title('Probability x < value', fontsize=14)
+                    ax_1.set_ylabel('Loading [%]', fontsize=11)
+                    df.plot(ax=ax_1)
+
+                if loading_clustering_data is not None:
+                    p_st = np.arange(len(loading_clustering_data)).astype(float) / len(loading_clustering_data)
+                    df = pd.DataFrame(data=loading_clustering_data,
+                                      index=p_st,
+                                      columns=[SimulationTypes.ClusteringTimeSeries_run.value])
+                    ax_1.set_title('Probability x < value', fontsize=14)
+                    ax_1.set_ylabel('Loading [%]', fontsize=11)
+                    df.plot(ax=ax_1)
+
+                if loading_st_data is not None:
+                    p_st = np.arange(len(loading_st_data)).astype(float) / len(loading_st_data)
+                    df = pd.DataFrame(data=loading_st_data,
+                                      index=p_st,
+                                      columns=[SimulationTypes.StochasticPowerFlow.value])
+                    ax_1.set_title('Probability x < value', fontsize=14)
+                    ax_1.set_ylabel('Loading [%]', fontsize=11)
+                    df.plot(ax=ax_1)
+
+                # power
+                if len(power_data.keys()):
+                    df = pd.DataFrame(data=power_data, index=x)
+                    ax_2.set_title('Power', fontsize=14)
+                    ax_2.set_ylabel('Power [MW]', fontsize=11)
+                    df.plot(ax=ax_2)
+                    ax_2.plot(x, api_object.rate_prof, c='gray', linestyle='dashed', linewidth=1)
+                    ax_2.plot(x, -api_object.rate_prof, c='gray', linestyle='dashed', linewidth=1)
+
+                if power_clustering_data is not None:
+                    df = pd.DataFrame(data=power_clustering_data,
+                                      index=x_cl,
+                                      columns=[SimulationTypes.ClusteringTimeSeries_run.value])
+                    ax_2.set_title('Power', fontsize=14)
+                    ax_2.set_ylabel('Power [MW]', fontsize=11)
+                    df.plot(ax=ax_2)
 
                 plt.legend()
-                fig.suptitle(api_obj.name, fontsize=20)
+                fig.suptitle(api_object.name, fontsize=20)
 
+                # plot the profiles
                 plt.show()
 
     def set_rate_to_profile(self, api_object):
@@ -422,83 +507,61 @@ class DiagramScene(QGraphicsScene):
         """
         if api_object is not None:
             if api_object.rate_prof is not None:
-                quit_msg = "Are you sure that you want to overwrite the rates profile with the snapshot value?"
-                reply = QMessageBox.question(self.parent_, 'Overwrite the profile', quit_msg, QMessageBox.Yes, QMessageBox.No)
+                quit_msg = str(api_object.name) + \
+                           "\nAre you sure that you want to overwrite the rates profile with the snapshot value?"
+                reply = QMessageBox.question(self.parent_, 'Overwrite the profile', quit_msg,
+                                             QMessageBox.Yes, QMessageBox.No)
 
                 if reply == QMessageBox.Yes:
                     api_object.rate_prof *= 0
                     api_object.rate_prof += api_object.rate
 
-    def set_active_status_to_profile(self, api_object):
+    def set_active_status_to_profile(self, api_object, override_question=False):
         """
 
         :param api_object:
+        :param override_question:
+        :return:
         """
         if api_object is not None:
             if api_object.active_prof is not None:
-                quit_msg = "Are you sure that you want to overwrite the rates profile with the snapshot value?"
-                reply = QMessageBox.question(self.parent_, 'Overwrite the profile', quit_msg, QMessageBox.Yes, QMessageBox.No)
+                if not override_question:
+                    quit_msg = str(api_object.name) + \
+                               "\nAre you sure that you want to overwrite the active profile with the snapshot value?"
+                    reply = QMessageBox.question(self.parent_, 'Overwrite the active profile', quit_msg,
+                                                 QMessageBox.Yes, QMessageBox.No)
+                    ok = reply == QMessageBox.Yes
+                else:
+                    ok = True
 
-                if reply == QMessageBox.Yes:
+                if ok:
                     shape = api_object.active_prof.shape
                     if api_object.active:
                         api_object.active_prof = np.ones(shape, dtype=bool)
                     else:
                         api_object.active_prof = np.zeros(shape, dtype=bool)
 
-    def mouseMoveEvent(self, mouseEvent):
+    def mouseMoveEvent(self, event):
         """
 
-        @param mouseEvent:
+        @param event:
         @return:
         """
-        self.parent_.scene_mouse_move_event(mouseEvent)
+        self.parent_.scene_mouse_move_event(event)
 
         # call the parent event
-        super(DiagramScene, self).mouseMoveEvent(mouseEvent)
+        super(DiagramScene, self).mouseMoveEvent(event)
 
-    def mouseReleaseEvent(self, mouseEvent):
+    def mouseReleaseEvent(self, event):
         """
 
-        @param mouseEvent:
+        @param event:
         @return:
         """
-        self.parent_.scene_mouse_release_event(mouseEvent)
+        self.parent_.scene_mouse_release_event(event)
 
         # call mouseReleaseEvent on "me" (continue with the rest of the actions)
-        super(DiagramScene, self).mouseReleaseEvent(mouseEvent)
-
-
-class ObjectFactory(object):
-
-    def get_box(self):
-        """
-
-        @return:
-        """
-        pixmap = QPixmap(40, 40)
-        pixmap.fill()
-        painter = QPainter(pixmap)
-        painter.fillRect(0, 0, 40, 40, Qt.black)
-        painter.end()
-
-        return QIcon(pixmap)
-
-    def get_circle(self):
-        """
-
-        @return:
-        """
-        pixmap = QPixmap(40, 40)
-        pixmap.fill()
-        painter = QPainter(pixmap)
-
-        painter.setBrush(Qt.red)
-        painter.drawEllipse(0, 0, 40, 40)
-
-        painter.end()
-
-        return QIcon(pixmap)
+        super(DiagramScene, self).mouseReleaseEvent(event)
 
 
 class GridEditor(QSplitter):
@@ -524,12 +587,23 @@ class GridEditor(QSplitter):
         self.libraryModel = LibraryModel(self)
         self.libraryModel.setColumnCount(1)
 
-        # Create an icon with an icon:
-        object_factory = ObjectFactory()
-
         # initialize library of items
         self.libItems = list()
-        self.libItems.append(QStandardItem(object_factory.get_box(), 'Bus'))
+
+        # add bus to the drag&drop
+        bus_icon = QIcon()
+        bus_icon.addPixmap(QPixmap(":/Icons/icons/bus_icon.svg"))
+        item = QStandardItem(bus_icon, "Bus")
+        item.setToolTip("Drag & drop this into the schematic")
+        self.libItems.append(item)
+
+        # add transformer3w to the drag&drop
+        # t3w_icon = QIcon()
+        # t3w_icon.addPixmap(QPixmap(":/Icons/icons/transformer3w.svg"))
+        # item = QStandardItem(t3w_icon, "3W-Transformer")
+        # item.setToolTip("Drag & drop this into the schematic")
+        # self.libItems.append(item)
+
         for i in self.libItems:
             self.libraryModel.appendRow(i)
 
@@ -558,6 +632,7 @@ class GridEditor(QSplitter):
 
         self.name_label = QLineEdit()
         self.name_label.setText(str(self.circuit.name))
+        self.name_label.setToolTip('Name of the model')
         self.name_layout.addWidget(self.name_label)
         self.name_editor_frame.setLayout(self.name_layout)
 
@@ -622,68 +697,110 @@ class GridEditor(QSplitter):
                         item.hosting_connections.append(self.started_branch)
                         self.started_branch.bus_to = item.parent
 
-                        if self.started_branch.bus_from.api_object.is_dc != self.started_branch.bus_to.api_object.is_dc:
-                            # different DC status -> VSC
+                        if isinstance(self.started_branch.bus_from.api_object, Bus) and \
+                           isinstance(self.started_branch.bus_to.api_object, Bus):
 
-                            name = 'VSC ' + str(len(self.circuit.vsc_devices) + 1)
-                            obj = VSC(bus_from=self.started_branch.bus_from.api_object,
-                                      bus_to=self.started_branch.bus_to.api_object,
-                                      name=name)
+                            if self.started_branch.bus_from.api_object.is_dc != self.started_branch.bus_to.api_object.is_dc:
+                                # different DC status -> VSC
 
-                            obj.graphic_obj = VscGraphicItem(fromPort=self.started_branch.fromPort,
-                                                             toPort=self.started_branch.toPort,
-                                                             diagramScene=self.diagramScene,
-                                                             branch=obj)
+                                name = 'VSC ' + str(len(self.circuit.vsc_devices) + 1)
+                                obj = VSC(bus_from=self.started_branch.bus_from.api_object,
+                                          bus_to=self.started_branch.bus_to.api_object,
+                                          name=name)
 
-                        elif self.started_branch.bus_from.api_object.is_dc and self.started_branch.bus_to.api_object.is_dc:
-                            # both buses are DC
+                                obj.graphic_obj = VscGraphicItem(fromPort=self.started_branch.fromPort,
+                                                                 toPort=self.started_branch.toPort,
+                                                                 diagramScene=self.diagramScene,
+                                                                 branch=obj)
 
-                            name = 'Dc line ' + str(len(self.circuit.dc_lines) + 1)
-                            obj = DcLine(bus_from=self.started_branch.bus_from.api_object,
-                                         bus_to=self.started_branch.bus_to.api_object,
-                                         name=name)
+                            elif self.started_branch.bus_from.api_object.is_dc and self.started_branch.bus_to.api_object.is_dc:
+                                # both buses are DC
 
-                            obj.graphic_obj = DcLineGraphicItem(fromPort=self.started_branch.fromPort,
-                                                                toPort=self.started_branch.toPort,
-                                                                diagramScene=self.diagramScene,
-                                                                branch=obj)
+                                name = 'Dc line ' + str(len(self.circuit.dc_lines) + 1)
+                                obj = DcLine(bus_from=self.started_branch.bus_from.api_object,
+                                             bus_to=self.started_branch.bus_to.api_object,
+                                             name=name)
 
-                        else:
-                            # Same DC status -> line / trafo
-                            v1 = self.started_branch.bus_from.api_object.Vnom
-                            v2 = self.started_branch.bus_to.api_object.Vnom
-
-                            if abs(v1 - v2) > 1.0:
-                                name = 'Transformer ' + str(len(self.circuit.transformers2w) + 1)
-                                obj = Transformer2W(bus_from=self.started_branch.bus_from.api_object,
-                                                    bus_to=self.started_branch.bus_to.api_object,
-                                                    name=name)
-
-                                obj.graphic_obj = TransformerGraphicItem(fromPort=self.started_branch.fromPort,
-                                                                         toPort=self.started_branch.toPort,
-                                                                         diagramScene=self.diagramScene,
-                                                                         branch=obj)
+                                obj.graphic_obj = DcLineGraphicItem(fromPort=self.started_branch.fromPort,
+                                                                    toPort=self.started_branch.toPort,
+                                                                    diagramScene=self.diagramScene,
+                                                                    branch=obj)
 
                             else:
-                                name = 'Line ' + str(len(self.circuit.lines) + 1)
-                                obj = Line(bus_from=self.started_branch.bus_from.api_object,
-                                           bus_to=self.started_branch.bus_to.api_object,
-                                           name=name)
+                                # Same DC status -> line / trafo
+                                v1 = self.started_branch.bus_from.api_object.Vnom
+                                v2 = self.started_branch.bus_to.api_object.Vnom
 
-                                obj.graphic_obj = LineGraphicItem(fromPort=self.started_branch.fromPort,
-                                                                  toPort=self.started_branch.toPort,
-                                                                  diagramScene=self.diagramScene,
-                                                                  branch=obj)
+                                if abs(v1 - v2) > 1.0:
+                                    name = 'Transformer ' + str(len(self.circuit.transformers2w) + 1)
+                                    obj = Transformer2W(bus_from=self.started_branch.bus_from.api_object,
+                                                        bus_to=self.started_branch.bus_to.api_object,
+                                                        name=name)
 
-                        # add the new object to the circuit
-                        self.circuit.add_branch(obj)
+                                    obj.graphic_obj = TransformerGraphicItem(fromPort=self.started_branch.fromPort,
+                                                                             toPort=self.started_branch.toPort,
+                                                                             diagramScene=self.diagramScene,
+                                                                             branch=obj)
 
-                        # update the connection placement
-                        obj.graphic_obj.fromPort.update()
-                        obj.graphic_obj.toPort.update()
+                                else:
+                                    name = 'Line ' + str(len(self.circuit.lines) + 1)
+                                    obj = Line(bus_from=self.started_branch.bus_from.api_object,
+                                               bus_to=self.started_branch.bus_to.api_object,
+                                               name=name)
 
-                        # set the connection placement
-                        obj.graphic_obj.setZValue(-1)
+                                    obj.graphic_obj = LineGraphicItem(fromPort=self.started_branch.fromPort,
+                                                                      toPort=self.started_branch.toPort,
+                                                                      diagramScene=self.diagramScene,
+                                                                      branch=obj)
+
+                            # add the new object to the circuit
+                            self.circuit.add_branch(obj)
+
+                            # update the connection placement
+                            obj.graphic_obj.fromPort.update()
+                            obj.graphic_obj.toPort.update()
+
+                            # set the connection placement
+                            obj.graphic_obj.setZValue(-1)
+
+                        elif isinstance(self.started_branch.bus_from.api_object, Transformer3W):
+
+                            obj = self.started_branch.bus_from.api_object
+
+                            obj.graphic_obj = LineGraphicItem(fromPort=self.started_branch.fromPort,
+                                                              toPort=self.started_branch.toPort,
+                                                              diagramScene=self.diagramScene,
+                                                              branch=None)
+
+                            # update the connection placement
+                            obj.graphic_obj.fromPort.update()
+                            obj.graphic_obj.toPort.update()
+
+                            # set the connection placement
+                            obj.graphic_obj.setZValue(-1)
+
+                            print('Hosted tr3w connection FROM')
+
+                        elif isinstance(self.started_branch.bus_to.api_object, Transformer3W):
+
+                            obj = self.started_branch.bus_to.api_object
+
+                            obj.graphic_obj = LineGraphicItem(fromPort=self.started_branch.fromPort,
+                                                              toPort=self.started_branch.toPort,
+                                                              diagramScene=self.diagramScene,
+                                                              branch=None)
+
+                            # update the connection placement
+                            obj.graphic_obj.fromPort.update()
+                            obj.graphic_obj.toPort.update()
+
+                            # set the connection placement
+                            obj.graphic_obj.setZValue(-1)
+
+                            print('Hosted tr3w connection TO')
+
+                        else:
+                            print('unknown connection')
 
             # if self.started_branch.toPort is None:
             self.started_branch.remove_widget()
@@ -1234,7 +1351,7 @@ class GridEditor(QSplitter):
                                        prog_func=prog_func,
                                        text_func=text_func)
 
-    def align_schematic(self):
+    def align_schematic(self, buses: List[Bus] = []):
         """
         Align the scene view to the content
         """
@@ -1244,8 +1361,13 @@ class GridEditor(QSplitter):
         max_x = -sys.maxsize
         max_y = -sys.maxsize
 
+        if len(buses):
+            lst = buses
+        else:
+            lst = self.circuit.buses
+
         # Align lines
-        for bus in self.circuit.buses:
+        for bus in lst:
             bus.graphic_obj.arrange_children()
             # get the item position
             x = bus.graphic_obj.pos().x()
@@ -1265,6 +1387,7 @@ class GridEditor(QSplitter):
 
     def clear(self):
         self.diagramView.scene_.clear()
+        self.name_label.setText("")
 
     def schematic_from_api(self, explode_factor=1.0, prog_func=None, text_func=None):
         """
@@ -1290,3 +1413,10 @@ class GridEditor(QSplitter):
 
 
 
+if __name__ == '__main__':
+    app = QApplication(sys.argv)
+    app.setStyle('Fusion')  # ['Breeze', 'Oxygen', 'QtCurve', 'Windows', 'Fusion']
+    circuit = MultiCircuit()
+    window = GridEditor(circuit)
+    window.show()
+    sys.exit(app.exec_())
\ No newline at end of file
