@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 from PySide2.QtWidgets import *
 from PySide2.QtCore import *
@@ -125,7 +127,7 @@ class GeneratorGraphicItem(QGraphicsItemGroup):
             pos.x() + self.w / 2,
             pos.y() + 0,
             parent.x() + rect.width() / 2,
-            parent.y() + parent.terminal.y() + 5,
+            parent.y() + parent.terminal.y + 5,
         )
         self.setZValue(-1)
         self.nexus.setZValue(-1)
@@ -189,6 +191,14 @@ class GeneratorGraphicItem(QGraphicsItemGroup):
             else:
                 self.set_enable(True)
 
+            if self.diagramScene.circuit.has_time_series:
+                ok = yes_no_question('Do you want to update the time series active status accordingly?',
+                                     'Update time series active status')
+
+                if ok:
+                    # change the bus state (time series)
+                    self.diagramScene.set_active_status_to_profile(self.api_object, override_question=True)
+
     def enable_disable_control_toggle(self):
         """
         Enable / Disable device voltage control
