@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 import numpy as np
 
 from GridCal.Gui.GuiFunctions import get_list_model
@@ -485,13 +487,13 @@ class LineGraphicItem(QGraphicsLineItem):
         :param QGraphicsSceneMouseEvent:
         :return:
         """
+        if self.api_object is not None:
+            mdl = BranchObjectModel([self.api_object], self.api_object.editable_headers,
+                                    parent=self.diagramScene.parent().object_editor_table,
+                                    editable=True, transposed=True,
+                                    non_editable_attributes=self.api_object.non_editable_attributes)
 
-        mdl = BranchObjectModel([self.api_object], self.api_object.editable_headers,
-                                parent=self.diagramScene.parent().object_editor_table,
-                                editable=True, transposed=True,
-                                non_editable_attributes=self.api_object.non_editable_attributes)
-
-        self.diagramScene.parent().object_editor_table.setModel(mdl)
+            self.diagramScene.parent().object_editor_table.setModel(mdl)
 
     def mouseDoubleClickEvent(self, event):
         """
@@ -499,13 +501,13 @@ class LineGraphicItem(QGraphicsLineItem):
         :param event:
         :return:
         """
-
-        if self.api_object.branch_type in [BranchType.Transformer, BranchType.Line]:
-            # trigger the editor
-            self.edit()
-        elif self.api_object.branch_type is BranchType.Switch:
-            # change state
-            self.enable_disable_toggle()
+        if self.api_object is not None:
+            if self.api_object.branch_type in [BranchType.Transformer, BranchType.Line]:
+                # trigger the editor
+                self.edit()
+            elif self.api_object.branch_type is BranchType.Switch:
+                # change state
+                self.enable_disable_toggle()
 
     def remove(self, ask=True):
         """
@@ -587,6 +589,14 @@ class LineGraphicItem(QGraphicsLineItem):
             else:
                 self.set_enable(True)
 
+            if self.diagramScene.circuit.has_time_series:
+                ok = yes_no_question('Do you want to update the time series active status accordingly?',
+                                     'Update time series active status')
+
+                if ok:
+                    # change the bus state (time series)
+                    self.diagramScene.set_active_status_to_profile(self.api_object, override_question=True)
+
     def set_enable(self, val=True):
         """
         Set the enable value, graphically and in the API
