@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import numpy as np
 import numba as nb
@@ -389,6 +391,21 @@ class PandasModel(QtCore.QAbstractTableModel):
         """
         self.data_c[:, col] = self.data_c[row, col]
 
+    def is_complex(self):
+        return self.data_c.dtype == complex
+
+    def is_2d(self):
+        """
+        actually check if the array is 1D or 2D
+        :return: true if it is really a 2D data set
+        """
+
+        is_2d_ = len(self.data_c.shape) == 2
+        if is_2d_:
+            if self.data_c.shape[1] <= 1:
+                is_2d_ = False
+        return is_2d_
+
     def get_data(self, mode=None):
         """
 
@@ -414,7 +431,7 @@ class PandasModel(QtCore.QAbstractTableModel):
             else:
                 names = [val.name for val in self.cols_c]
 
-            if self.data_c.dtype == complex:
+            if self.is_complex():
 
                 if mode == 'real':
                     values = self.data_c.real
@@ -423,7 +440,7 @@ class PandasModel(QtCore.QAbstractTableModel):
                 elif mode == 'abs':
                     values = np.abs(self.data_c)
                 else:
-                    values = np.abs(self.data_c)
+                    values = self.data_c.astype(object)
 
             else:
                 values = self.data_c
@@ -433,9 +450,17 @@ class PandasModel(QtCore.QAbstractTableModel):
             # there are no elements
             return list(), list(), list()
 
+    def get_df(self, mode=None):
+        """
+        Get the data as pandas DataFrame
+        :return: DataFrame
+        """
+        index, columns, data = self.get_data(mode=mode)
+        return pd.DataFrame(data=data, index=index, columns=columns)
+
     def save_to_excel(self, file_name, mode):
         """
-
+        Save the data to excel
         Args:
             file_name:
             mode: 'real', 'imag', 'abs'
@@ -458,16 +483,18 @@ class PandasModel(QtCore.QAbstractTableModel):
 
         if n > 0:
 
-            index, columns, data = self.get_data(mode=mode)
+            if mode == 'numpy':
+                txt = fast_data_to_numpy_text(self.data_c)
+            else:
+                index, columns, data = self.get_data(mode=mode)
 
-            data = data.astype(str)
+                data = data.astype(str)
 
-            # header first
-            txt = '\t' + '\t'.join(columns) + '\n'
+                # header first
+                txt = '\t' + '\t'.join(columns) + '\n'
 
-            # data
-            for t, index_value in enumerate(index):
-                if data[t, :].sum() != 0.0:
+                # data
+                for t, index_value in enumerate(index):
                     txt += str(index_value) + '\t' + '\t'.join(data[t, :]) + '\n'
 
             # copy to clipboard
@@ -475,10 +502,6 @@ class PandasModel(QtCore.QAbstractTableModel):
             cb.clear(mode=cb.Clipboard)
             cb.setText(txt, mode=cb.Clipboard)
 
-        else:
-            # there are no elements
-            pass
-
 
 class ObjectsModel(QtCore.QAbstractTableModel):
     """
@@ -630,6 +653,31 @@ class ObjectsModel(QtCore.QAbstractTableModel):
         else:
             return self.c
 
+    def data_raw(self, r, c):
+        """
+        Get the data to display
+        :param index:
+        :return:
+        """
+
+        if self.transposed:
+            obj_idx = c
+            attr_idx = r
+        else:
+            obj_idx = r
+            attr_idx = c
+
+        attr = self.attributes[attr_idx]
+        tpe = self.attribute_types[attr_idx]
+
+        if tpe is Bus:
+            return getattr(self.objects[obj_idx], attr).name
+        elif tpe is BranchType:
+            # conv = BranchType(None)
+            return BranchType(getattr(self.objects[obj_idx], attr))
+        else:
+            return getattr(self.objects[obj_idx], attr)
+
     def data_with_type(self, index):
         """
         Get the data to display
@@ -781,6 +829,47 @@ class ObjectsModel(QtCore.QAbstractTableModel):
             else:
                 pass  # the column cannot be edited
 
+    def get_data(self):
+        """
+
+        :return:
+        """
+        nrows = self.rowCount()
+        ncols = self.columnCount()
+        data = np.empty((nrows, ncols), dtype=object)
+
+        for j in range(ncols):
+            for i in range(nrows):
+                data[i, j] = self.data_raw(r=i, c=j)
+
+        columns = [self.headerData(i, orientation=QtCore.Qt.Horizontal, role=QtCore.Qt.DisplayRole) for i in range(ncols)]
+        index = [self.headerData(i, orientation=QtCore.Qt.Vertical, role=QtCore.Qt.DisplayRole) for i in range(nrows)]
+
+        return index, columns, data
+
+    def copy_to_clipboard(self):
+        """
+
+        :return:
+        """
+        if self.columnCount() > 0:
+
+            index, columns, data = self.get_data()
+
+            data = data.astype(str)
+
+            # header first
+            txt = '\t' + '\t'.join(columns) + '\n'
+
+            # data
+            for t, index_value in enumerate(index):
+                txt += str(index_value) + '\t' + '\t'.join(data[t, :]) + '\n'
+
+            # copy to clipboard
+            cb = QApplication.clipboard()
+            cb.clear(mode=cb.Clipboard)
+            cb.setText(txt, mode=cb.Clipboard)
+
 
 class BranchObjectModel(ObjectsModel):
 
@@ -1278,4 +1367,27 @@ def get_tree_item_path(item: QtGui.QStandardItem):
         path.append(parent_text)
         item_parent = item_parent.parent()
     path.reverse()
-    return path
\ No newline at end of file
+    return path
+
+def fast_data_to_numpy_text(data):
+
+    if len(data.shape) == 1:
+        txt = '[' + ', '.join(['{0:.6f}'.format(x) for x in data]) + ']'
+
+    elif len(data.shape) == 2:
+
+        if data.shape[1] > 1:
+            # header first
+            txt = '['
+
+            # data
+            for t in range(data.shape[0]):
+                txt += '[' + ', '.join(['{0:.6f}'.format(x) for x in data[t, :]]) + '],\n'
+
+            txt += ']'
+        else:
+            txt = '[' + ', '.join(['{0:.6f}'.format(x) for x in data[:, 0]]) + ']'
+    else:
+        txt = '[]'
+
+    return txt
\ No newline at end of file
