@@ -28,11 +28,9 @@ class ExcelDialog(QtWidgets.QDialog):
     def accepted_action(self):
         if len(self.ui.sheets_list.selectedIndexes()):
             self.excel_sheet = self.ui.sheets_list.selectedIndexes()[0].row()
-        print('Accepted: self.excel_sheet: ', self.excel_sheet)
         self.close()
 
     def rejected_action(self):
-        print('Rejected: self.excel_sheet: ', self.excel_sheet)
         self.close()
 
 
