@@ -1,17 +1,19 @@
-# This file is part of GridCal.
-#
-# GridCal is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# GridCal is distributed in the hope that it will be useful,
+# GridCal
+# Copyright (C) 2022 Santiago Peñate Vera
+# 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3 of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
 
@@ -24,7 +26,7 @@ from GridCal.Gui.GuiFunctions import PandasModel
 from GridCal.Gui.GeneralDialogues import LogsDialogue
 
 
-class TowerBuilderGUI(QtWidgets.QDialog):
+class TowerBuilderGUI(QDialog):
 
     def __init__(self, parent=None, tower: Tower = None, wires_catalogue=list()):
         """
@@ -33,7 +35,7 @@ class TowerBuilderGUI(QtWidgets.QDialog):
         :param tower:
         :param wires_catalogue:
         """
-        QtWidgets.QDialog.__init__(self, parent)
+        QDialog.__init__(self, parent)
         self.ui = Ui_Dialog()
         self.ui.setupUi(self)
         self.setWindowTitle('Line builder')
@@ -60,13 +62,17 @@ class TowerBuilderGUI(QtWidgets.QDialog):
         self.ui.wires_tableView.setModel(self.wires_table)
         self.ui.tower_tableView.setModel(self.tower_driver)
 
+        # set divider
+        self.ui.main_splitter.setStretchFactor(0, 2)
+        self.ui.main_splitter.setStretchFactor(1, 3)
+
         # button clicks
         # self.ui.add_wire_pushButton.clicked.connect(self.add_wire_to_collection)
         # self.ui.delete_wire_pushButton.clicked.connect(self.delete_wire_from_collection)
         self.ui.add_to_tower_pushButton.clicked.connect(self.add_wire_to_tower)
         self.ui.delete_from_tower_pushButton.clicked.connect(self.delete_wire_from_tower)
         self.ui.compute_pushButton.clicked.connect(self.compute)
-
+        self.ui.acceptButton.clicked.connect(self.accept)
         self.ui.name_lineEdit.textChanged.connect(self.name_changed)
 
     def msg(self, text, title="Warning"):
@@ -147,7 +153,7 @@ class TowerBuilderGUI(QtWidgets.QDialog):
 
             self.plot()
         else:
-            self.msg('Select a wire from the tower')
+            self.msg('Select a wire from the wire composition')
 
     def compute(self):
         """
@@ -163,7 +169,7 @@ class TowerBuilderGUI(QtWidgets.QDialog):
         all_ok = self.tower_driver.tower.check(logs)
 
         if not all_ok:
-            logger_diag = LogsDialogue(name='Tower computation', logs=logs)
+            logger_diag = LogsDialogue(name='Tower computation', logger=logs)
             logger_diag.exec_()
         else:
             try:
@@ -185,17 +191,26 @@ class TowerBuilderGUI(QtWidgets.QDialog):
 
                 # Admittances in uS/km
                 cols = ['Phase' + str(i) for i in self.tower_driver.tower.y_phases_abcn]
-                z_df = pd.DataFrame(data=self.tower_driver.tower.y_abcn * 1e6, columns=cols, index=cols)
+                z_df = pd.DataFrame(data=self.tower_driver.tower.y_abcn.imag * 1e6, columns=cols, index=cols)
                 self.ui.y_tableView_abcn.setModel(PandasModel(z_df))
 
                 cols = ['Phase' + str(i) for i in self.tower_driver.tower.y_phases_abc]
-                z_df = pd.DataFrame(data=self.tower_driver.tower.y_abc * 1e6, columns=cols, index=cols)
+                z_df = pd.DataFrame(data=self.tower_driver.tower.y_abc.imag * 1e6, columns=cols, index=cols)
                 self.ui.y_tableView_abc.setModel(PandasModel(z_df))
 
                 cols = ['Sequence ' + str(i) for i in range(3)]
-                z_df = pd.DataFrame(data=self.tower_driver.tower.y_seq * 1e6, columns=cols, index=cols)
+                z_df = pd.DataFrame(data=self.tower_driver.tower.y_seq.imag * 1e6, columns=cols, index=cols)
                 self.ui.y_tableView_seq.setModel(PandasModel(z_df))
 
+                # set auto adjust headers
+                self.ui.z_tableView_abcn.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.ui.z_tableView_abc.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.ui.z_tableView_seq.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+
+                self.ui.y_tableView_abcn.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.ui.y_tableView_abc.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.ui.y_tableView_seq.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+
                 # plot
                 self.plot()
 
@@ -262,13 +277,13 @@ class TowerBuilderGUI(QtWidgets.QDialog):
 
 if __name__ == "__main__":
 
-    app = QtWidgets.QApplication(sys.argv)
+    app = QApplication(sys.argv)
     window = TowerBuilderGUI()
 
     window.example_2()
     window.compute()
 
-    window.resize(1.81 * 700.0, 700.0)  # golden ratio
+    window.resize(1.61 * 600.0, 600.0)  # golden ratio
     window.show()
     sys.exit(app.exec_())
 
