@@ -55,7 +55,8 @@ def colour_sub_schematic(Sbase,
                          loadings,
                          types=None,
                          losses=None,
-                         hvdc_sending_power=None,
+                         hvdc_Pf=None,
+                         hvdc_Pt=None,
                          hvdc_losses=None,
                          hvdc_loading=None,
                          failed_br_idx=None,
@@ -81,7 +82,7 @@ def colour_sub_schematic(Sbase,
     :param loadings: Branches load
     :param types: Buses type
     :param losses: Branches losses
-    :param hvdc_sending_power:
+    :param hvdc_Pf:
     :param hvdc_losses:
     :param hvdc_loading:
     :param failed_br_idx: failed branches
@@ -136,7 +137,7 @@ def colour_sub_schematic(Sbase,
                 tooltip += '\n'
 
                 tooltip += "%-10s %10.4f < %10.4fº [p.u.]\n" % ("V", vabs[i], vang[i])
-                tooltip += "%-10s %10.4f < %10.4fº [kV]\n" % ("V", vabs[i], vang[i])
+                tooltip += "%-10s %10.4f < %10.4fº [kV]\n" % ("V", vabs[i] * bus.Vnom, vang[i])
 
                 if Sbus is not None:
                     tooltip += "%-10s %10.4f [MW]\n" % ("P", Sbus[i].real)
@@ -159,11 +160,14 @@ def colour_sub_schematic(Sbase,
             Sfabs = np.abs(Sf)
             max_flow = Sfabs.max()
 
-            if hvdc_sending_power is not None:
-                if len(hvdc_sending_power) > 0:
-                    max_flow = max(max_flow, np.abs(hvdc_sending_power).max())
+            if hvdc_Pf is not None:
+                if len(hvdc_Pf) > 0:
+                    max_flow = max(max_flow, np.abs(hvdc_Pf).max())
 
-            Sfnorm = Sfabs / max_flow
+            if max_flow != 0:
+                Sfnorm = Sfabs / max_flow
+            else:
+                Sfnorm = Sfabs
 
             for i, branch in enumerate(branches):
                 if branch.graphic_obj is not None:
@@ -220,9 +224,9 @@ def colour_sub_schematic(Sbase,
                 color = QtCore.Qt.gray
                 branches[i].graphic_obj.set_pen(QtGui.QPen(color, w, style))
 
-    if hvdc_sending_power is not None:
+    if hvdc_Pf is not None:
 
-        hvdc_sending_power_norm = np.abs(hvdc_sending_power) / max_flow
+        hvdc_sending_power_norm = np.abs(hvdc_Pf) / (max_flow + 1e-20)
 
         for i, elm in enumerate(hvdc_lines):
 
@@ -244,10 +248,10 @@ def colour_sub_schematic(Sbase,
                 tooltip = str(i) + ': ' + elm.name
                 tooltip += '\n' + loading_label + ': ' + "{:10.4f}".format(abs(hvdc_loading[i]) * 100) + ' [%]'
 
-                tooltip += '\nPower (from):\t' + "{:10.4f}".format(hvdc_sending_power[i]) + ' [MW]'
+                tooltip += '\nPower (from):\t' + "{:10.4f}".format(hvdc_Pf[i]) + ' [MW]'
 
                 if hvdc_losses is not None:
-                    tooltip += '\nPower (to):\t' + "{:10.4f}".format(-hvdc_sending_power[i] + hvdc_losses[i]) + ' [MW]'
+                    tooltip += '\nPower (to):\t' + "{:10.4f}".format(hvdc_Pt[i]) + ' [MW]'
                     tooltip += '\nLosses: \t\t' + "{:10.4f}".format(hvdc_losses[i]) + ' [MW]'
 
                 elm.graphic_obj.setToolTipText(tooltip)
@@ -256,7 +260,7 @@ def colour_sub_schematic(Sbase,
 
 def colour_the_schematic(circuit: MultiCircuit, Sbus, Sf, voltages, loadings,
                          types=None, losses=None, St=None,
-                         hvdc_sending_power=None, hvdc_losses=None, hvdc_loading=None,
+                         hvdc_Pf=None, hvdc_Pt=None, hvdc_losses=None, hvdc_loading=None,
                          failed_br_idx=None, loading_label='loading',
                          ma=None,
                          theta=None,
@@ -277,7 +281,7 @@ def colour_the_schematic(circuit: MultiCircuit, Sbus, Sf, voltages, loadings,
     :param types: Buses type
     :param losses: Branches losses
     :param St: power seen from the "to" bus
-    :param hvdc_sending_power:
+    :param hvdc_Pf:
     :param hvdc_losses:
     :param hvdc_loading:
     :param failed_br_idx: failed branches
@@ -300,7 +304,8 @@ def colour_the_schematic(circuit: MultiCircuit, Sbus, Sf, voltages, loadings,
                          loadings=loadings,
                          types=types,
                          losses=losses,
-                         hvdc_sending_power=hvdc_sending_power,
+                         hvdc_Pf=hvdc_Pf,
+                         hvdc_Pt=hvdc_Pt,
                          hvdc_losses=hvdc_losses,
                          hvdc_loading=hvdc_loading,
                          failed_br_idx=failed_br_idx,
@@ -345,13 +350,26 @@ def get_base_map(location, zoom_start=5):
     return my_map, marker_cluster
 
 
-def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, losses=None,
+def plot_html_map(circuit: MultiCircuit,
+                  Sbus,
+                  Sf,
+                  voltages,
+                  loadings,
+                  types=None,
+                  losses=None,
                   St=None,
+                  hvdc_Pf=None,
+                  hvdc_Pt=None,
+                  hvdc_losses=None,
+                  hvdc_loading=None,
                   failed_br_idx=None,
                   loading_label='loading',
+                  ma=None,
+                  theta=None,
+                  Beq=None,
                   use_flow_based_width=False,
-                  min_branch_width=5,
-                  max_branch_width=5,
+                  min_branch_width=1,
+                  max_branch_width=1,
                   min_bus_width=20,
                   max_bus_width=20,
                   file_name='map.html'):
@@ -364,8 +382,15 @@ def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, lo
     :param loadings: Branches load
     :param types: Buses type
     :param losses: Branches losses
+    :param St:
     :param failed_br_idx: failed branches
     :param loading_label:
+    :param use_flow_based_width:
+    :param min_branch_width:
+    :param max_branch_width:
+    :param min_bus_width:
+    :param max_bus_width:
+    :param file_name:
     :return:
     """
 
@@ -393,6 +418,8 @@ def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, lo
     # create map at he average location
     my_map, marker_cluster = get_base_map(location=circuit.get_center_location(), zoom_start=5)
 
+    Pnorm = np.abs(Sbus.real) / np.max(Sbus.real)
+
     # add node positions
     for i, bus in enumerate(circuit.buses):
 
@@ -409,19 +436,25 @@ def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, lo
         color = QtGui.QColor(r * 255, g * 255, b * 255, a * 255)
         html_color = color.name()
 
+        if use_flow_based_width:
+            radius = int(np.floor(min_bus_width + Pnorm[i] * (max_bus_width - min_bus_width)))
+        else:
+            radius = 50
+
         position = bus.get_coordinates()
         html = '<i>' + tooltip + '</i>'
         folium.Circle(position,
                       popup=html,
-                      radius=50,
+                      radius=radius,
                       color=html_color,
                       tooltip=tooltip).add_to(marker_cluster)
 
     # add lines
     lnorm = np.abs(loadings)
     lnorm[lnorm == np.inf] = 0
-    branches = circuit.get_branches()
-    for i, branch in enumerate(branches):
+    Sfabs = np.abs(Sf)
+    Sfnorm = Sfabs / np.max(Sfabs)
+    for i, branch in enumerate(circuit.get_branches_wo_hvdc()):
 
         points = branch.get_coordinates()
 
@@ -438,7 +471,10 @@ def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, lo
             r, g, b, a = loading_cmap(lnorm[i])
             color = QtGui.QColor(r * 255, g * 255, b * 255, a * 255)
             html_color = color.name()
-            weight = 3
+            if use_flow_based_width:
+                weight = int(np.floor(min_branch_width + Sfnorm[i] * (max_branch_width - min_branch_width)))
+            else:
+                weight = 3
 
             # draw the line
             folium.PolyLine(points,
@@ -447,6 +483,40 @@ def plot_html_map(circuit: MultiCircuit, Sbus, Sf, voltages, loadings, types, lo
                             opacity=1,
                             tooltip=tooltip).add_to(marker_cluster)
 
+    if len(circuit.get_hvdc()) > 0:
+        lnorm = np.abs(hvdc_loading)
+        lnorm[lnorm == np.inf] = 0
+        Sfabs = np.abs(hvdc_Pf)
+        Sfnorm = Sfabs / np.max(Sfabs)
+        for i, branch in enumerate(circuit.get_hvdc()):
+
+            points = branch.get_coordinates()
+
+            if not has_null_coordinates(points):
+                # compose the tooltip
+                tooltip = str(i) + ': ' + branch.name
+                tooltip += '\n' + loading_label + ': ' + "{:10.4f}".format(lnorm[i] * 100) + ' [%]'
+                if Sf is not None:
+                    tooltip += '\nPower: ' + "{:10.4f}".format(hvdc_Pf[i]) + ' [MW]'
+                if losses is not None:
+                    tooltip += '\nLosses: ' + "{:10.4f}".format(hvdc_losses[i]) + ' [MW]'
+
+                # get the line colour
+                r, g, b, a = loading_cmap(lnorm[i])
+                color = QtGui.QColor(r * 255, g * 255, b * 255, a * 255)
+                html_color = color.name()
+                if use_flow_based_width:
+                    weight = int(np.floor(min_branch_width + Sfnorm[i] * (max_branch_width - min_branch_width)))
+                else:
+                    weight = 3
+
+                # draw the line
+                folium.PolyLine(points,
+                                color=html_color,
+                                weight=weight,
+                                opacity=1,
+                                tooltip=tooltip).add_to(marker_cluster)
+
     # save the map
     my_map.save(file_name)
 
