@@ -2,7 +2,7 @@
 import datetime
 _current_year_ = datetime.datetime.now().year
 
-__GridCal_VERSION__ = "4.2.0"
+__GridCal_VERSION__ = "4.5.4"
 
 url = 'https://github.com/SanPen/GridCal'
 
@@ -14,12 +14,15 @@ serve as a platform for research and consultancy.
 Visit https://gridcal.org for more details.\n"""
 
 about_msg += """
-This program comes with ABSOLUTELY NO WARRANTY. 
-This is free software, and you are welcome to 
-redistribute it under certain conditions.\n
-GridCal is licensed under the GNU general public 
-license V.3. See the license file for more 
-details.
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 3 of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
 The source of GridCal can be found at:
 """ + url + "\n\n"
@@ -31,5 +34,7 @@ contributors_msg += 'Bengt Lüers (Better testing)\n'
 contributors_msg += 'Josep Fanals Batllori (HELM)\n'
 contributors_msg += 'Manuel Navarro Catalán (Better documentation)\n'
 contributors_msg += 'Paul Schultz (Grid Generator)\n'
+contributors_msg += 'Andrés Ramiro (Optimal net transfer capacity)\n'
+contributors_msg += 'Ameer Carlo Lubang (Improved short-circuit)\n'
 
 about_msg += copyright_msg + '\n' + contributors_msg
